@@ -14,6 +14,7 @@ from openmdao.components.add_subtract_comp import AddSubtractComp
 from openmdao.components.balance_comp import BalanceComp
 from openmdao.components.bsplines_comp import BsplinesComp
 from openmdao.components.cross_product_comp import CrossProductComp
+from openmdao.components.demux_comp import DemuxComp
 from openmdao.components.dot_product_comp import DotProductComp
 from openmdao.components.eq_constraint_comp import EQConstraintComp
 from openmdao.components.exec_comp import ExecComp
@@ -32,6 +33,7 @@ from openmdao.components.meta_model_unstructured_comp import MetaModelUnStructur
 from openmdao.components.multifi_meta_model_unstructured_comp import MultiFiMetaModelUnStructured
 from openmdao.components.multifi_meta_model_unstructured_comp \
     import MultiFiMetaModelUnStructuredComp
+from openmdao.components.mux_comp import MuxComp
 from openmdao.components.vector_magnitude_comp import VectorMagnitudeComp
 
 
@@ -91,6 +93,9 @@ from openmdao.utils.options_dictionary import OptionsDictionary
 from openmdao.recorders.sqlite_recorder import SqliteRecorder
 from openmdao.recorders.case_reader import CaseReader
 
+# Visualizations
+from openmdao.utils.visualization import partial_deriv_plot
+
 # set up tracing or memory profiling if env vars are set.
 import os
 if os.environ.get('OPENMDAO_TRACE'):
