@@ -6,6 +6,7 @@ import math
 from six import iteritems
 
 import numpy as np
+from numpy.testing import assert_almost_equal
 import scipy
 
 from parameterized import parameterized
@@ -16,64 +17,64 @@ from openmdao.utils.assert_utils import assert_rel_error
 
 _ufunc_test_data = {'abs': {'str': 'f=abs(x)',
                             'check_func': np.abs,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': np.random.random(6)}}},
+                            'args': {'f': {'value': np.zeros(6)},
+                                     'x': {'value': np.random.random(6)}}},
                     'acos': {'str': 'f=acos(x)',
-                               'check_func': np.arccos,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)-0.5}}},
+                             'check_func': np.arccos,
+                             'args': {'f': {'value': np.zeros(6)},
+                                      'x': {'value': np.random.random(6)-0.5}}},
                     'arccos': {'str': 'f=arccos(x)',
                                'check_func': np.arccos,
                                'args': {'f': {'value': np.zeros(6)},
                                         'x': {'value': np.random.random(6)-0.5}}},
                     'arccosh': {'str': 'f=arccosh(x)',
-                             'check_func': np.arccosh,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': 1.1+np.random.random(6)}}},
+                                'check_func': np.arccosh,
+                                'args': {'f': {'value': np.zeros(6)},
+                                         'x': {'value': 1.1+np.random.random(6)}}},
                     'acosh': {'str': 'f=acosh(x)',
-                             'check_func': np.arccosh,
-                             'args': {'f': {'value': np.zeros(6)},
-                                      'x': {'value': 1.1+np.random.random(6)}}},
+                              'check_func': np.arccosh,
+                              'args': {'f': {'value': np.zeros(6)},
+                                       'x': {'value': 1.1+np.random.random(6)}}},
                     'arange': {'str': 'f=arange(0,10,2)',
-                             'check_val': np.arange(0,10,2),
-                             'args': {'f': {'value': np.zeros(5)}}},
+                               'check_val': np.arange(0, 10, 2),
+                               'args': {'f': {'value': np.zeros(5)}}},
                     'arcsin': {'str': 'f=arcsin(x)',
                                'check_func': np.arcsin,
                                'args': {'f': {'value': np.zeros(6)},
                                         'x': {'value': np.random.random(6)-.5}}},
                     'arcsinh': {'str': 'f=arcsinh(x)',
-                               'check_func': np.arcsinh,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)}}},
+                                'check_func': np.arcsinh,
+                                'args': {'f': {'value': np.zeros(6)},
+                                         'x': {'value': np.random.random(6)}}},
                     'asinh': {'str': 'f=asinh(x)',
-                               'check_func': np.arcsinh,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)}}},
+                              'check_func': np.arcsinh,
+                              'args': {'f': {'value': np.zeros(6)},
+                                       'x': {'value': np.random.random(6)}}},
                     'asin': {'str': 'f=asin(x)',
-                               'check_func': np.arcsin,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)-.5}}},
+                             'check_func': np.arcsin,
+                             'args': {'f': {'value': np.zeros(6)},
+                                      'x': {'value': np.random.random(6)-.5}}},
                     'arctan': {'str': 'f=arctan(x)',
                                'check_func': np.arctan,
                                'args': {'f': {'value': np.zeros(6)},
                                         'x': {'value': np.random.random(6)}}},
                     'atan': {'str': 'f=atan(x)',
-                               'check_func': np.arctan,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)}}},
+                             'check_func': np.arctan,
+                             'args': {'f': {'value': np.zeros(6)},
+                                      'x': {'value': np.random.random(6)}}},
                     'cos': {'str': 'f=cos(x)',
-                               'check_func': np.cos,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)}}},
+                            'check_func': np.cos,
+                            'args': {'f': {'value': np.zeros(6)},
+                                     'x': {'value': np.random.random(6)}}},
                     'cosh': {'str': 'f=cosh(x)',
-                               'check_func': np.cosh,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)}}},
+                             'check_func': np.cosh,
+                             'args': {'f': {'value': np.zeros(6)},
+                                      'x': {'value': np.random.random(6)}}},
                     'dot': {'str': 'f=dot(x, y)',
-                               'check_func': np.dot,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)},
-                                        'y': {'value': np.random.random(6)}}},
+                            'check_func': np.dot,
+                            'args': {'f': {'value': np.zeros(6)},
+                                     'x': {'value': np.random.random(6)},
+                                     'y': {'value': np.random.random(6)}}},
                     'e': {'str': 'f=e',
                           'check_val': np.e,
                           'args': {'f': {'value': 0.0}}},
@@ -98,20 +99,20 @@ _ufunc_test_data = {'abs': {'str': 'f=abs(x)',
                                   'args': {'f': {'value': np.zeros(6)},
                                            'x': {'value': np.random.random(6)}}},
                     'fmax': {'str': 'f=fmax(x, y)',
-                            'check_func': np.fmax,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)},
-                                     'y': {'value': np.random.random(6)}}},
+                             'check_func': np.fmax,
+                             'args': {'f': {'value': np.zeros(6)},
+                                      'x': {'value': np.random.random(6)},
+                                      'y': {'value': np.random.random(6)}}},
                     'fmin': {'str': 'f=fmin(x, y)',
-                            'check_func': np.fmin,
-                            'args': {'f': {'value': np.zeros(6)},
-                                     'x': {'value': np.random.random(6)},
-                                     'y': {'value': np.random.random(6)}}},
+                             'check_func': np.fmin,
+                             'args': {'f': {'value': np.zeros(6)},
+                                      'x': {'value': np.random.random(6)},
+                                      'y': {'value': np.random.random(6)}}},
                     'inner': {'str': 'f=inner(x, y)',
-                               'check_func': np.inner,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)},
-                                        'y': {'value': np.random.random(6)}}},
+                              'check_func': np.inner,
+                              'args': {'f': {'value': np.zeros(6)},
+                                       'x': {'value': np.random.random(6)},
+                                       'y': {'value': np.random.random(6)}}},
                     'isinf': {'str': 'f=isinf(x)',
                               'check_func': np.isinf,
                               'args': {'f': {'value': np.zeros(3)},
@@ -121,30 +122,30 @@ _ufunc_test_data = {'abs': {'str': 'f=abs(x)',
                               'args': {'f': {'value': np.zeros(3)},
                                        'x': {'value': [0, np.nan, np.nan]}}},
                     'kron': {'str': 'f=kron(x, y)',
-                            'check_func': np.kron,
-                            'args': {'f': {'value': np.zeros(36)},
-                                     'x': {'value': np.random.random(6)},
-                                     'y': {'value': np.random.random(6)}}},
+                             'check_func': np.kron,
+                             'args': {'f': {'value': np.zeros(36)},
+                                      'x': {'value': np.random.random(6)},
+                                      'y': {'value': np.random.random(6)}}},
                     'linspace': {'str': 'f=linspace(0,10,50)',
-                                 'check_val': np.linspace(0,10,50),
+                                 'check_val': np.linspace(0, 10, 50),
                                  'args': {'f': {'value': np.zeros(50)}}},
                     'log': {'str': 'f=log(x)',
-                               'check_func': np.log,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)+0.1}}},
+                            'check_func': np.log,
+                            'args': {'f': {'value': np.zeros(6)},
+                                     'x': {'value': np.random.random(6)+0.1}}},
                     'log10': {'str': 'f=log10(x)',
-                               'check_func': np.log10,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)+0.1}}},
+                              'check_func': np.log10,
+                              'args': {'f': {'value': np.zeros(6)},
+                                       'x': {'value': np.random.random(6)+0.1}}},
                     'log1p': {'str': 'f=log1p(x)',
-                               'check_func': np.log1p,
-                               'args': {'f': {'value': np.zeros(6)},
-                                        'x': {'value': np.random.random(6)}}},
+                              'check_func': np.log1p,
+                              'args': {'f': {'value': np.zeros(6)},
+                                       'x': {'value': np.random.random(6)}}},
                     'matmul': {'str': 'f=matmul(x, y)',
                                'check_func': np.matmul,
-                               'args': {'f': {'value': np.zeros((3,1))},
-                                        'x': {'value': np.random.random((3,3))},
-                                        'y': {'value': np.random.random((3,1))}}},
+                               'args': {'f': {'value': np.zeros((3, 1))},
+                                        'x': {'value': np.random.random((3, 3))},
+                                        'y': {'value': np.random.random((3, 1))}}},
                     'maximum': {'str': 'f=maximum(x, y)',
                                 'check_func': np.maximum,
                                 'args': {'f': {'value': np.zeros(6)},
@@ -197,9 +198,9 @@ _ufunc_test_data = {'abs': {'str': 'f=abs(x)',
                                       'x': {'value': np.random.random(6)}}},
                     'tensordot': {'str': 'f=tensordot(x, y)',
                                   'check_func': np.tensordot,
-                                 'args': {'f': {'value': 0.0},
-                                          'x': {'value': np.random.random((6,6))},
-                                          'y': {'value': np.random.random((6,6))}}},
+                                  'args': {'f': {'value': 0.0},
+                                           'x': {'value': np.random.random((6, 6))},
+                                           'y': {'value': np.random.random((6, 6))}}},
                     'zeros': {'str': 'f=zeros(21)',
                               'check_val': np.zeros(21),
                               'args': {'f': {'value': np.zeros(21)}}}, }
@@ -221,15 +222,18 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup(check=False)
         self.assertEqual(str(context.exception),
-                         "C1: arg 'xx' in call to ExecComp() does not refer to any variable in the expressions ['y=x+1.']")
+                         "C1: arg 'xx' in call to ExecComp() does not refer to any variable "
+                         "in the expressions ['y=x+1.']")
 
     def test_bad_kwargs_meta(self):
         prob = Problem(model=Group())
-        prob.model.add_subsystem('C1', ExecComp('y=x+1.', x={'val': 2.0, 'low': 0.0, 'high': 10.0, 'units': 'ft'}))
+        prob.model.add_subsystem('C1', ExecComp('y=x+1.',
+                                                x={'val': 2, 'low': 0, 'high': 10, 'units': 'ft'}))
         with self.assertRaises(Exception) as context:
             prob.setup(check=False)
         self.assertEqual(str(context.exception),
-                         "C1: the following metadata names were not recognized for variable 'x': ['high', 'low', 'val']")
+                         "C1: the following metadata names were not recognized for "
+                         "variable 'x': ['high', 'low', 'val']")
 
     def test_name_collision_const(self):
         prob = Problem(model=Group())
@@ -237,7 +241,8 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup(check=False)
         self.assertEqual(str(context.exception),
-                         "C1: cannot assign to variable 'e' because it's already defined as an internal function or constant.")
+                         "C1: cannot assign to variable 'e' because it's already defined "
+                         "as an internal function or constant.")
 
     def test_name_collision_func(self):
         prob = Problem(model=Group())
@@ -245,7 +250,8 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup(check=False)
         self.assertEqual(str(context.exception),
-                         "C1: cannot assign to variable 'sin' because it's already defined as an internal function or constant.")
+                         "C1: cannot assign to variable 'sin' because it's already defined "
+                         "as an internal function or constant.")
 
     def test_func_as_rhs_var(self):
         prob = Problem(model=Group())
@@ -253,12 +259,13 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup(check=False)
         self.assertEqual(str(context.exception),
-                         "C1: cannot use 'sin' as a variable because it's already defined as an internal function.")
+                         "C1: cannot use 'sin' as a variable because it's already defined "
+                         "as an internal function.")
 
     def test_mixed_type(self):
         prob = Problem(model=Group())
         C1 = prob.model.add_subsystem('C1', ExecComp('y=sum(x)',
-                                                     x=np.arange(10,dtype=float)))
+                                                     x=np.arange(10, dtype=float)))
         prob.setup(check=False)
 
         # Conclude setup but don't run model.
@@ -343,7 +350,7 @@ class TestExecComp(unittest.TestCase):
     def test_array(self):
         prob = Problem(model=Group())
         C1 = prob.model.add_subsystem('C1', ExecComp('y=x[1]',
-                                                     x=np.array([1.,2.,3.]),
+                                                     x=np.array([1., 2., 3.]),
                                                      y=0.0))
 
         prob.setup(check=False)
@@ -362,8 +369,8 @@ class TestExecComp(unittest.TestCase):
     def test_array_lhs(self):
         prob = Problem(model=Group())
         C1 = prob.model.add_subsystem('C1', ExecComp(['y[0]=x[1]', 'y[1]=x[0]'],
-                                                     x=np.array([1.,2.,3.]),
-                                                     y=np.array([0.,0.])))
+                                                     x=np.array([1., 2., 3.]),
+                                                     y=np.array([0., 0.])))
 
         prob.setup(check=False)
 
@@ -376,7 +383,7 @@ class TestExecComp(unittest.TestCase):
         prob.set_solver_print(level=0)
         prob.run_model()
 
-        assert_rel_error(self, C1._outputs['y'], np.array([2.,1.]), 0.00001)
+        assert_rel_error(self, C1._outputs['y'], np.array([2., 1.]), 0.00001)
 
     def test_simple_array_model(self):
         prob = Problem()
@@ -394,12 +401,12 @@ class TestExecComp(unittest.TestCase):
 
         data = prob.check_partials()
 
-        assert_rel_error(self, data['comp'][('y','x')]['abs error'][0], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['abs error'][1], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['abs error'][2], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['rel error'][0], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['rel error'][1], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['rel error'][2], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][0], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][1], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][2], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['rel error'][0], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['rel error'][1], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['rel error'][2], 0.0, 1e-5)
 
     def test_simple_array_model2(self):
         prob = Problem()
@@ -407,7 +414,7 @@ class TestExecComp(unittest.TestCase):
         prob.model.add_subsystem('p1', IndepVarComp('x', np.ones([2])))
         prob.model.add_subsystem('comp', ExecComp('y = mat.dot(x)',
                                                   x=np.zeros((2,)), y=np.zeros((2,)),
-                                                  mat=np.array([[2.,7.],[5.,-3.]])))
+                                                  mat=np.array([[2., 7.], [5., -3.]])))
 
         prob.model.connect('p1.x', 'comp.x')
 
@@ -417,12 +424,12 @@ class TestExecComp(unittest.TestCase):
 
         data = prob.check_partials()
 
-        assert_rel_error(self, data['comp'][('y','x')]['abs error'][0], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['abs error'][1], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['abs error'][2], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['rel error'][0], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['rel error'][1], 0.0, 1e-5)
-        assert_rel_error(self, data['comp'][('y','x')]['rel error'][2], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][0], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][1], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['abs error'][2], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['rel error'][0], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['rel error'][1], 0.0, 1e-5)
+        assert_rel_error(self, data['comp'][('y', 'x')]['rel error'][2], 0.0, 1e-5)
 
     def test_complex_step(self):
         prob = Problem(model=Group())
@@ -443,7 +450,7 @@ class TestExecComp(unittest.TestCase):
 
         C1._linearize()
 
-        assert_rel_error(self, C1._jacobian[('y','x')], [[2.0]], 0.00001)
+        assert_rel_error(self, C1._jacobian[('y', 'x')], [[2.0]], 0.00001)
 
     def test_complex_step2(self):
         prob = Problem(Group())
@@ -477,15 +484,15 @@ class TestExecComp(unittest.TestCase):
         # any positive C1.x should give a 2.0 derivative for dy/dx
         C1._inputs['x'] = 1.0e-10
         C1._linearize()
-        assert_rel_error(self, C1._jacobian['y','x'], [[2.0]], 0.00001)
+        assert_rel_error(self, C1._jacobian['y', 'x'], [[2.0]], 0.00001)
 
         C1._inputs['x'] = -3.0
         C1._linearize()
-        assert_rel_error(self, C1._jacobian['y','x'], [[-2.0]], 0.00001)
+        assert_rel_error(self, C1._jacobian['y', 'x'], [[-2.0]], 0.00001)
 
         C1._inputs['x'] = 0.0
         C1._linearize()
-        assert_rel_error(self, C1._jacobian['y','x'], [[2.0]], 0.00001)
+        assert_rel_error(self, C1._jacobian['y', 'x'], [[2.0]], 0.00001)
 
     def test_abs_array_complex_step(self):
         prob = Problem(model=Group())
@@ -501,24 +508,54 @@ class TestExecComp(unittest.TestCase):
         # any positive C1.x should give a 2.0 derivative for dy/dx
         C1._inputs['x'] = np.ones(3)*1.0e-10
         C1._linearize()
-        assert_rel_error(self, C1._jacobian['y','x'], np.eye(3)*2.0, 0.00001)
+        assert_rel_error(self, C1._jacobian['y', 'x'], np.eye(3)*2.0, 0.00001)
 
         C1._inputs['x'] = np.ones(3)*-3.0
         C1._linearize()
-        assert_rel_error(self, C1._jacobian['y','x'], np.eye(3)*-2.0, 0.00001)
+        assert_rel_error(self, C1._jacobian['y', 'x'], np.eye(3)*-2.0, 0.00001)
 
         C1._inputs['x'] = np.zeros(3)
         C1._linearize()
-        assert_rel_error(self, C1._jacobian['y','x'], np.eye(3)*2.0, 0.00001)
+        assert_rel_error(self, C1._jacobian['y', 'x'], np.eye(3)*2.0, 0.00001)
 
         C1._inputs['x'] = np.array([1.5, -0.6, 2.4])
         C1._linearize()
-        expect = np.zeros((3,3))
-        expect[0,0] = 2.0
-        expect[1,1] = -2.0
-        expect[2,2] = 2.0
+        expect = np.zeros((3, 3))
+        expect[0, 0] = 2.0
+        expect[1, 1] = -2.0
+        expect[2, 2] = 2.0
 
-        assert_rel_error(self, C1._jacobian['y','x'], expect, 0.00001)
+        assert_rel_error(self, C1._jacobian['y', 'x'], expect, 0.00001)
+
+    def test_vectorize_error(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(3)))
+        model.add_design_var('indep.x')
+
+        mat = np.arange(15).reshape((3,5))
+        model.add_subsystem('comp', ExecComp('y=A.dot(x)', vectorize=True, A=mat, x=np.ones(5), y=np.ones(3)))
+        model.connect('indep.x', 'comp.x')
+
+        with self.assertRaises(Exception) as context:
+            p.setup()
+        self.assertEqual(str(context.exception),
+                         "comp: vectorize is True but partial(y, A) is not square (shape=(3, 15)).")
+
+    def test_feature_vectorize(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indep', IndepVarComp('x', val=np.ones(5)))
+
+        model.add_subsystem('comp', ExecComp('y=3.0*x + 2.5', vectorize=True, x=np.ones(5), y=np.ones(5)))
+        model.connect('indep.x', 'comp.x')
+
+        p.setup()
+        p.run_model()
+
+        J = p.compute_totals(of=['comp.y'], wrt=['indep.x'], return_format='array')
+
+        assert_almost_equal(J, np.eye(5)*3., decimal=6)
 
     def test_feature_simple(self):
         from openmdao.api import IndepVarComp, Group, Problem, ExecComp
@@ -567,7 +604,7 @@ class TestExecComp(unittest.TestCase):
 
         model.add_subsystem('p', IndepVarComp('x', np.array([1., 2., 3.])))
         model.add_subsystem('comp', ExecComp('y=x[1]',
-                                             x=np.array([1.,2.,3.]),
+                                             x=np.array([1., 2., 3.]),
                                              y=0.0))
         model.connect('p.x', 'comp.x')
 
@@ -628,7 +665,7 @@ class TestExecComp(unittest.TestCase):
         model.add_subsystem('p1', IndepVarComp('x', 12.0, units='inch'))
         model.add_subsystem('p2', IndepVarComp('y', 1.0, units='ft'))
         model.add_subsystem('comp', ExecComp('z=x+y',
-                                             x={'value': 0.0, 'units':'inch'},
+                                             x={'value': 0.0, 'units': 'inch'},
                                              y={'value': 0.0, 'units': 'inch'},
                                              z={'value': 0.0, 'units': 'inch'}))
         model.connect('p1.x', 'comp.x')
@@ -641,10 +678,9 @@ class TestExecComp(unittest.TestCase):
 
         assert_rel_error(self, prob['comp.z'], 24.0, 0.00001)
 
-    @parameterized.expand(itertools.product(
-        [func_name for func_name in _expr_dict if not func_name.startswith('_')],
-    ), testcase_func_name=lambda f, n, p: 'test_exec_comp_value_' + '_'.join(a for a in p.args)
-    )
+    @parameterized.expand(itertools.product([
+        func_name for func_name in _expr_dict if not func_name.startswith('_')
+    ]), name_func=lambda f, n, p: 'test_exec_comp_value_' + '_'.join(a for a in p.args))
     def test_exec_comp_value(self, f):
         test_data = _ufunc_test_data[f]
 
@@ -670,19 +706,18 @@ class TestExecComp(unittest.TestCase):
             check_args = []
             try:
                 check_args.append(test_data['args']['x']['value'])
-            except:
+            except Exception:
                 pass
             try:
                 check_args.append(test_data['args']['y']['value'])
-            except:
+            except Exception:
                 pass
             check_args = tuple(check_args)
 
             expected = test_data['check_func'](*check_args)
         else:
-            expected =  test_data['check_val']
+            expected = test_data['check_val']
         np.testing.assert_almost_equal(prob['f'], expected)
-        #print('OK')
 
         if 'check_val' not in test_data:
             try:
@@ -690,10 +725,9 @@ class TestExecComp(unittest.TestCase):
             except TypeError as e:
                 print(f, 'does not support complex-step differentiation')
 
-    @parameterized.expand(itertools.product(
-        [func_name for func_name in _expr_dict if not func_name.startswith('_')],
-    ), testcase_func_name=lambda f, n, p: 'test_exec_comp_jac_' + '_'.join(a for a in p.args)
-    )
+    @parameterized.expand(itertools.product([
+        func_name for func_name in _expr_dict if not func_name.startswith('_')
+    ]), name_func=lambda f, n, p: 'test_exec_comp_jac_' + '_'.join(a for a in p.args))
     def test_exec_comp_jac(self, f):
 
         test_data = _ufunc_test_data[f]
