@@ -4,7 +4,7 @@ from __future__ import division
 
 from collections import OrderedDict, Iterable
 from itertools import product
-from six import string_types, iteritems, itervalues
+from six import string_types, iteritems
 
 import numpy as np
 from numpy import ndarray, isscalar, atleast_1d, atleast_2d, promote_types
@@ -17,9 +17,10 @@ from openmdao.jacobians.assembled_jacobian import SUBJAC_META_DEFAULTS
 from openmdao.jacobians.dictionary_jacobian import DictionaryJacobian
 from openmdao.utils.units import valid_units
 from openmdao.utils.general_utils import format_as_float_or_array, ensure_compatible, \
-    warn_deprecation, find_matches
+    warn_deprecation, find_matches, simple_warning
 from openmdao.vectors.vector import INT_DTYPE
 from openmdao.utils.name_maps import rel_key2abs_key, abs_key2rel_key
+from openmdao.utils.mpi import MPI
 
 
 # Suppored methods for derivatives
@@ -28,6 +29,14 @@ _supported_methods = {'fd': (FiniteDifference, DEFAULT_FD_OPTIONS),
                       'exact': (None, {})}
 
 
+# the following metadata will be accessible for vars on all procs
+global_meta_names = {
+    'input': ('units', 'shape', 'size'),
+    'output': ('units', 'shape', 'size',
+               'ref', 'ref0', 'res_ref', 'distributed', 'lower', 'upper'),
+}
+
+
 def _valid_var_name(name):
     """
     Determine if the proposed name is a valid variable name.
@@ -53,16 +62,13 @@ class Component(System):
 
     Attributes
     ----------
-    distributed : bool
-        This is True if the component has variables that are distributed across multiple
-        processes.
     _approx_schemes : OrderedDict
         A mapping of approximation types to the associated ApproximationScheme.
-    _var_rel2data_io : dict
-        Dictionary mapping relative names to dicts with keys (prom, rel, my_idx, type_, metadata).
+    _var_rel2meta : dict
+        Dictionary mapping relative names to metadata.
         This is only needed while adding inputs and outputs. During setup, these are used to
         build the dictionaries of metadata.
-    _static_var_rel2data_io : dict
+    _static_var_rel2meta : dict
         Static version of above - stores data for variables added outside of setup.
     _var_rel_names : {'input': [str, ...], 'output': [str, ...]}
         List of relative names of owned variables existing on current proc.
@@ -87,24 +93,58 @@ class Component(System):
         **kwargs : dict of keyword arguments
             available here and in all descendants of this system.
         """
-        # put these here to prevent them from possibly overriding values set
-        # by the user in initialize().
-        self.distributed = False
-
         super(Component, self).__init__(**kwargs)
 
         self._approx_schemes = OrderedDict()
 
         self._var_rel_names = {'input': [], 'output': []}
-        self._var_rel2data_io = {}
+        self._var_rel2meta = {}
 
         self._static_var_rel_names = {'input': [], 'output': []}
-        self._static_var_rel2data_io = {}
+        self._static_var_rel2meta = {}
 
         self._declared_partials = []
         self._approximated_partials = []
         self._declared_partial_checks = []
 
+    def _declare_options(self):
+        """
+        Declare options before kwargs are processed in the init method.
+        """
+        super(Component, self)._declare_options()
+
+        self.options.declare('distributed', False,
+                             desc='True if the component has variables that are distributed '
+                                  'across multiple processes.')
+
+    @property
+    def distributed(self):
+        """
+        Provide 'distributed' property for backwards compatibility.
+
+        Returns
+        -------
+        bool
+            reference to the 'distributed' option.
+        """
+        warn_deprecation("The 'distributed' property provides backwards compatibility "
+                         "with OpenMDAO <= 2.4.0 ; use the 'distributed' option instead.")
+        return self.options['distributed']
+
+    @distributed.setter
+    def distributed(self, val):
+        """
+        Provide for setting of the 'distributed' property for backwards compatibility.
+
+        Parameters
+        ----------
+        val : bool
+            True if the component has variables that are distributed across multiple processes.
+        """
+        warn_deprecation("The 'distributed' property provides backwards compatibility "
+                         "with OpenMDAO <= 2.4.0 ; use the 'distributed' option instead.")
+        self.options['distributed'] = val
+
     def setup(self):
         """
         Declare inputs and outputs.
@@ -134,63 +174,57 @@ class Component(System):
             reverse (adjoint). Default is 'rev'.
         """
         self.pathname = pathname
+
+        orig_comm = comm
+        if self._num_par_fd > 1:
+            if comm.size > 1:
+                comm = self._setup_par_fd_procs(comm)
+            elif not MPI:
+                msg = ("'%s': MPI is not active but num_par_fd = %d. No parallel finite difference "
+                       "will be performed." % (self.pathname, self._num_par_fd))
+                simple_warning(msg)
+
         self.comm = comm
         self._mode = mode
         self._subsystems_proc_range = []
 
         # Clear out old variable information so that we can call setup on the component.
         self._var_rel_names = {'input': [], 'output': []}
-        self._var_rel2data_io = {}
+        self._var_rel2meta = {}
         self._design_vars = OrderedDict()
         self._responses = OrderedDict()
 
         self._static_mode = False
-        self._var_rel2data_io.update(self._static_var_rel2data_io)
+        self._var_rel2meta.update(self._static_var_rel2meta)
         for type_ in ['input', 'output']:
             self._var_rel_names[type_].extend(self._static_var_rel_names[type_])
         self._design_vars.update(self._static_design_vars)
         self._responses.update(self._static_responses)
         self.setup()
+
+        # check to make sure that if num_par_fd > 1 that this system is actually doing FD.
+        # Unfortunately we have to do this check after system setup has been called because that's
+        # when declare_partials generally happens, so we raise an exception here instead of just
+        # resetting the value of num_par_fd (because the comm has already been split and possibly
+        # used by the system setup).
+        if self._num_par_fd > 1 and orig_comm.size > 1 and not (self._owns_approx_jac or
+                                                                self._approximated_partials):
+            raise RuntimeError("'%s': num_par_fd is > 1 but no FD is active." % self.pathname)
+
         self._static_mode = True
 
-        if self.distributed:
-            self._vector_class = self._distributed_vector_class
+        if self.options['distributed']:
+            if self._distributed_vector_class is not None:
+                self._vector_class = self._distributed_vector_class
+            else:
+                simple_warning("The 'distributed' option is set to True for Component %s, "
+                               "but there is no distributed vector implementation (MPI/PETSc) "
+                               "available. The default non-distributed vectors will be used."
+                               % pathname)
+                self._vector_class = self._local_vector_class
         else:
             self._vector_class = self._local_vector_class
 
-    def _setup_vars(self, recurse=True):
-        """
-        Count variables, total and by var_set.
-
-        Parameters
-        ----------
-        recurse : bool
-            Whether to call this method in subsystems.
-        """
-        super(Component, self)._setup_vars()
-        num_var = self._num_var
-        num_var_byset = self._num_var_byset
-        data = self._var_rel2data_io
-
-        for vec_name in self._lin_rel_vec_name_list:
-            num_var[vec_name] = {}
-            num_var_byset[vec_name] = {}
-            # Compute num_var
-            for type_ in ['input', 'output']:
-                relnames = self._var_allprocs_relevant_names[vec_name][type_]
-                num_var[vec_name][type_] = len(relnames)
-
-                num_var_byset[vec_name][type_] = vbyset = {}
-                # Compute num_var_byset
-                for name in relnames:
-                    set_name = data[name.rsplit('.', 1)[-1]]['metadata']['var_set']
-                    if set_name not in vbyset:
-                        vbyset[set_name] = 0
-                    vbyset[set_name] += 1
-
-        self._num_var['nonlinear'] = self._num_var['linear']
-        self._num_var_byset['nonlinear'] = self._num_var_byset['linear']
-
     def _setup_var_data(self, recurse=True):
         """
         Compute the list of abs var names, abs/prom name maps, and metadata dictionaries.
@@ -200,35 +234,24 @@ class Component(System):
         recurse : bool
             Whether to call this method in subsystems.
         """
+        global global_meta_names
         super(Component, self)._setup_var_data()
         allprocs_abs_names = self._var_allprocs_abs_names
-        abs_names = self._var_abs_names
         allprocs_prom2abs_list = self._var_allprocs_prom2abs_list
         abs2prom = self._var_abs2prom
         allprocs_abs2meta = self._var_allprocs_abs2meta
         abs2meta = self._var_abs2meta
 
         # Compute the prefix for turning rel/prom names into abs names
-        if self.pathname:
-            prefix = self.pathname + '.'
-        else:
-            prefix = ''
-
-        # the following metadata will be accessible for vars on all procs
-        global_meta_names = {
-            'input': ('units', 'shape', 'size', 'var_set'),
-            'output': ('units', 'shape', 'size', 'var_set',
-                       'ref', 'ref0', 'res_ref', 'distributed', 'lower', 'upper'),
-        }
+        prefix = self.pathname + '.' if self.pathname else ''
 
         for type_ in ['input', 'output']:
             for prom_name in self._var_rel_names[type_]:
                 abs_name = prefix + prom_name
-                metadata = self._var_rel2data_io[prom_name]['metadata']
+                metadata = self._var_rel2meta[prom_name]
 
                 # Compute allprocs_abs_names, abs_names
                 allprocs_abs_names[type_].append(abs_name)
-                abs_names[type_].append(abs_name)
 
                 # Compute allprocs_prom2abs_list, abs2prom
                 allprocs_prom2abs_list[type_][prom_name] = [abs_name]
@@ -243,6 +266,18 @@ class Component(System):
                 # Compute abs2meta
                 abs2meta[abs_name] = metadata
 
+            for prom_name, val in iteritems(self._var_discrete[type_]):
+                abs_name = prefix + prom_name
+                allprocs_prom2abs_list[type_][prom_name] = [abs_name]
+                self._var_allprocs_discrete[type_][abs_name] = val
+
+        self._var_abs_names = self._var_allprocs_abs_names
+        if self._var_discrete['input'] or self._var_discrete['output']:
+            self._discrete_inputs = _DictValues(self._var_discrete['input'])
+            self._discrete_outputs = _DictValues(self._var_discrete['output'])
+        else:
+            self._discrete_inputs = self._discrete_outputs = None
+
     def _setup_var_sizes(self, recurse=True):
         """
         Compute the arrays of local variable sizes for all variables/procs on this system.
@@ -256,50 +291,47 @@ class Component(System):
 
         iproc = self.comm.rank
         nproc = self.comm.size
-        vec_names = self._lin_rel_vec_name_list
 
         sizes = self._var_sizes
-        sizes_byset = self._var_sizes_byset
+        abs2meta = self._var_abs2meta
+
+        if self._use_derivatives:
+            vec_names = self._lin_rel_vec_name_list
+        else:
+            vec_names = self._vec_names
 
         # Initialize empty arrays
         for vec_name in vec_names:
+            # at component level, _var_allprocs_* is the same as var_* since all vars exist in all
+            # procs for a given component, so we don't have to mess with figuring out what vars are
+            # local.
+            if self._use_derivatives:
+                relnames = self._var_allprocs_relevant_names[vec_name]
+            else:
+                relnames = self._var_allprocs_abs_names
+
             sizes[vec_name] = {}
-            sizes_byset[vec_name] = {}
-
             for type_ in ('input', 'output'):
-                sizes[vec_name][type_] = np.zeros((nproc, self._num_var[vec_name][type_]), int)
+                sizes[vec_name][type_] = sz = np.zeros((nproc, len(relnames[type_])), int)
 
-                sizes_byset[vec_name][type_] = {}
-                for set_name, nvars in iteritems(self._num_var_byset[vec_name][type_]):
-                    sizes_byset[vec_name][type_][set_name] = np.zeros((nproc, nvars), int)
+                # Compute _var_sizes
+                for idx, abs_name in enumerate(relnames[type_]):
+                    sz[iproc, idx] = abs2meta[abs_name]['size']
 
-            allprocs_abs2idx_byset = self._var_allprocs_abs2idx_byset[vec_name]
-
-            # Compute _var_sizes and _var_sizes_byset
-            abs2meta = self._var_abs2meta
-            for type_ in ('input', 'output'):
-                sz = sizes[vec_name][type_]
-                sz_byset = sizes_byset[vec_name][type_]
-                for idx, abs_name in enumerate(self._var_allprocs_relevant_names[vec_name][type_]):
-                    meta = abs2meta[abs_name]
-                    set_name = meta['var_set']
-                    size = meta['size']
-                    idx_byset = allprocs_abs2idx_byset[abs_name]
-
-                    sz[iproc, idx] = size
-                    sz_byset[set_name][iproc, idx_byset] = size
-
-        if self.comm.size > 1:
+        if nproc > 1:
             for vec_name in vec_names:
                 sizes = self._var_sizes[vec_name]
-                sizes_byset = self._var_sizes_byset[vec_name]
-                for type_ in ['input', 'output']:
-                    self.comm.Allgather(sizes[type_][iproc, :], sizes[type_])
-                    for set_name, sbyset in iteritems(sizes_byset[type_]):
-                        self.comm.Allgather(sbyset[iproc, :], sbyset)
+                if self.options['distributed']:
+                    for type_ in ['input', 'output']:
+                        self.comm.Allgather(sizes[type_][iproc, :], sizes[type_])
+                else:
+                    # if component isn't distributed, we don't need to allgather sizes since
+                    # they'll all be the same.
+                    for type_ in ['input', 'output']:
+                        sizes[type_] = np.tile(sizes[type_][iproc], (nproc, 1))
 
-        self._var_sizes['nonlinear'] = self._var_sizes['linear']
-        self._var_sizes_byset['nonlinear'] = self._var_sizes_byset['linear']
+        if self._use_derivatives:
+            self._var_sizes['nonlinear'] = self._var_sizes['linear']
 
         self._setup_global_shapes()
 
@@ -322,7 +354,7 @@ class Component(System):
             self._approx_partials(of, wrt, method=method, **kwargs)
 
     def add_input(self, name, val=1.0, shape=None, src_indices=None, flat_src_indices=None,
-                  units=None, desc='', var_set=0):
+                  units=None, desc=''):
         """
         Add an input variable to the component.
 
@@ -350,9 +382,6 @@ class Component(System):
             during execution. Default is None, which means it is unitless.
         desc : str
             description of the variable
-        var_set : hashable object
-            For advanced users only. ID or color for this variable, relevant for
-            reconfigurability. Default is 0.
 
         Returns
         -------
@@ -403,40 +432,73 @@ class Component(System):
             metadata['src_indices'] = np.asarray(src_indices, dtype=INT_DTYPE)
         metadata['flat_src_indices'] = flat_src_indices
 
-        # units: taken as is
         metadata['units'] = units
-
-        # desc: taken as is
         metadata['desc'] = desc
 
-        # var_set: taken as is
-        metadata['var_set'] = var_set
-
         # We may not know the pathname yet, so we have to use name for now, instead of abs_name.
         if self._static_mode:
-            var_rel2data_io = self._static_var_rel2data_io
+            var_rel2meta = self._static_var_rel2meta
             var_rel_names = self._static_var_rel_names
         else:
-            var_rel2data_io = self._var_rel2data_io
+            var_rel2meta = self._var_rel2meta
             var_rel_names = self._var_rel_names
 
         # Disallow dupes
-        if name in var_rel2data_io:
+        if name in var_rel2meta:
             msg = "Variable name '{}' already exists.".format(name)
             raise ValueError(msg)
 
-        var_rel2data_io[name] = {
-            'prom': name, 'rel': name,
-            'my_idx': len(self._var_rel_names['input']),
-            'type': 'input',
-            'metadata': metadata
-        }
+        var_rel2meta[name] = metadata
         var_rel_names['input'].append(name)
 
         return metadata
 
+    def add_discrete_input(self, name, val, desc=''):
+        """
+        Add a discrete input variable to the component.
+
+        Parameters
+        ----------
+        name : str
+            name of the variable in this component's namespace.
+        val : a picklable object
+            The initial value of the variable being added.
+        desc : str
+            description of the variable
+
+        Returns
+        -------
+        dict
+            metadata for added variable
+        """
+        # First, type check all arguments
+        if not isinstance(name, str):
+            raise TypeError('The name argument should be a string')
+        if not _valid_var_name(name):
+            raise NameError("'%s' is not a valid input name." % name)
+
+        metadata = {
+            'value': val,
+            'type': type(val),
+            'desc': desc,
+        }
+
+        if self._static_mode:
+            var_rel2meta = self._static_var_rel2meta
+        else:
+            var_rel2meta = self._var_rel2meta
+
+        # Disallow dupes
+        if name in var_rel2meta:
+            msg = "Variable name '{}' already exists.".format(name)
+            raise ValueError(msg)
+
+        var_rel2meta[name] = self._var_discrete['input'][name] = metadata
+
+        return metadata
+
     def add_output(self, name, val=1.0, shape=None, units=None, res_units=None, desc='',
-                   lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=1.0, var_set=0):
+                   lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=1.0):
         """
         Add an output variable to the component.
 
@@ -476,9 +538,6 @@ class Component(System):
         res_ref : float or ndarray
             Scaling parameter. The value in the user-defined res_units of this output's residual
             when the scaled value is 1. Default is 1.
-        var_set : hashable object
-            For advanced users only. ID or color for this variable, relevant for reconfigurability.
-            Default is 0.
 
         Returns
         -------
@@ -494,7 +553,6 @@ class Component(System):
                              name)
             units = None
 
-        # First, type check all arguments
         if not isinstance(name, str):
             raise TypeError('The name argument should be a string')
         if not _valid_var_name(name):
@@ -573,39 +631,74 @@ class Component(System):
         ref0 = format_as_float_or_array('ref0', ref0, flatten=True)
         res_ref = format_as_float_or_array('res_ref', res_ref, flatten=True)
 
-        # ref, ref0, res_ref: taken as is
         metadata['ref'] = ref
         metadata['ref0'] = ref0
         metadata['res_ref'] = res_ref
 
-        # var_set: taken as is
-        metadata['var_set'] = var_set
-
-        metadata['distributed'] = self.distributed
+        metadata['distributed'] = self.options['distributed']
 
         # We may not know the pathname yet, so we have to use name for now, instead of abs_name.
         if self._static_mode:
-            var_rel2data_io = self._static_var_rel2data_io
+            var_rel2meta = self._static_var_rel2meta
             var_rel_names = self._static_var_rel_names
         else:
-            var_rel2data_io = self._var_rel2data_io
+            var_rel2meta = self._var_rel2meta
             var_rel_names = self._var_rel_names
 
         # Disallow dupes
-        if name in var_rel2data_io:
+        if name in var_rel2meta:
             msg = "Variable name '{}' already exists.".format(name)
             raise ValueError(msg)
 
-        var_rel2data_io[name] = {
-            'prom': name, 'rel': name,
-            'my_idx': len(self._var_rel_names['output']),
-            'type': 'output',
-            'metadata': metadata
-        }
+        var_rel2meta[name] = metadata
         var_rel_names['output'].append(name)
 
         return metadata
 
+    def add_discrete_output(self, name, val, desc=''):
+        """
+        Add an output variable to the component.
+
+        Parameters
+        ----------
+        name : str
+            name of the variable in this component's namespace.
+        val : a picklable object
+            The initial value of the variable being added.
+        desc : str
+            description of the variable.
+
+        Returns
+        -------
+        dict
+            metadata for added variable
+        """
+        if not isinstance(name, str):
+            raise TypeError('The name argument should be a string')
+        if not _valid_var_name(name):
+            raise NameError("'%s' is not a valid output name." % name)
+
+        metadata = {
+            'value': val,
+            'type': type(val),
+            'desc': desc
+        }
+
+        # We may not know the pathname yet, so we have to use name for now, instead of abs_name.
+        if self._static_mode:
+            var_rel2meta = self._static_var_rel2meta
+        else:
+            var_rel2meta = self._var_rel2meta
+
+        # Disallow dupes
+        if name in var_rel2meta:
+            msg = "Variable name '{}' already exists.".format(name)
+            raise ValueError(msg)
+
+        var_rel2meta[name] = self._var_discrete['output'][name] = metadata
+
+        return metadata
+
     def _approx_partials(self, of, wrt, method='fd', **kwargs):
         """
         Inform the framework that the specified derivatives are to be approximated.
@@ -899,11 +992,9 @@ class Component(System):
 
                 if rows.size > 0:
                     if rows.min() < 0:
-                        # of, wrt = abs_key2rel_key(self, abs_key)
                         msg = '{}: d({})/d({}): row indices must be non-negative'
                         raise ValueError(msg.format(self.pathname, of, wrt))
                     if cols.min() < 0:
-                        # of, wrt = abs_key2rel_key(self, abs_key)
                         msg = '{}: d({})/d({}): col indices must be non-negative'
                         raise ValueError(msg.format(self.pathname, of, wrt))
                     rows_max = rows.max()
@@ -1035,9 +1126,6 @@ class Component(System):
                 if method is not None and method in self._approx_schemes:
                     self._approx_schemes[method].add_approximation(key, meta)
 
-        for approx in itervalues(self._approx_schemes):
-            approx._init_approximations()
-
     def _guess_nonlinear(self):
         """
         Provide initial guess for states.
@@ -1053,3 +1141,24 @@ class Component(System):
         Components don't have nested solvers, so do nothing to prevent errors.
         """
         pass
+
+
+class _DictValues(object):
+    """
+    A dict-like wrapper for a dict of metadata, where getitem returns 'value' from metadata.
+    """
+
+    def __init__(self, dct):
+        self._dict = dct
+
+    def __getitem__(self, key):
+        return self._dict[key]['value']
+
+    def __setitem__(self, key, value):
+        self._dict[key]['value'] = value
+
+    def __contains__(self, key):
+        return key in self._dict
+
+    def __len__(self):
+        return len(self._dict)
