@@ -1,12 +1,10 @@
 """Define a base class for all Drivers in OpenMDAO."""
 from __future__ import print_function
 
-import os
 import json
 from collections import OrderedDict
 import pprint
 import sys
-import warnings
 
 from six import iteritems, itervalues, string_types
 
@@ -16,33 +14,31 @@ from openmdao.core.total_jac import _TotalJacInfo
 from openmdao.recorders.recording_manager import RecordingManager
 from openmdao.recorders.recording_iteration_stack import Recording
 from openmdao.utils.record_util import create_local_meta, check_path
+from openmdao.utils.general_utils import simple_warning
 from openmdao.utils.mpi import MPI
-from openmdao.recorders.recording_iteration_stack import recording_iteration
 from openmdao.utils.options_dictionary import OptionsDictionary
 import openmdao.utils.coloring as coloring_mod
 
 
-def _is_debug_print_opts_valid(opts):
+def _check_debug_print_opts_valid(name, opts):
     """
     Check validity of debug_print option for Driver.
 
     Parameters
     ----------
+    name : str
+        The name of the option.
     opts : list
         The value of the debug_print option set by the user.
-
-    Returns
-    -------
-    bool
-        True if the option is valid. Otherwise, False.
     """
     if not isinstance(opts, list):
-        return False
+        raise ValueError("Option '%s' with value %s is not a list." % (name, opts))
+
     _valid_opts = ['desvars', 'nl_cons', 'ln_cons', 'objs', 'totals']
     for opt in opts:
         if opt not in _valid_opts:
-            return False
-    return True
+            raise ValueError("Option '%s' contains value '%s' which is not one of %s." %
+                             (name, opt, _valid_opts))
 
 
 class Driver(object):
@@ -59,15 +55,13 @@ class Driver(object):
         Dictionary with general pyoptsparse options.
     recording_options : <OptionsDictionary>
         Dictionary with driver recording options.
-    debug_print : <OptionsDictionary>
-        Dictionary with debugging printing options.
     cite : str
-        Listing of relevant citataions that should be referenced when
+        Listing of relevant citations that should be referenced when
         publishing work that uses this class.
     _problem : <Problem>
         Pointer to the containing problem.
     supports : <OptionsDictionary>
-        Provides a consistant way for drivers to declare what features they support.
+        Provides a consistent way for drivers to declare what features they support.
     _designvars : dict
         Contains all design variable info.
     _cons : dict
@@ -82,17 +76,6 @@ class Driver(object):
         Dict of lists of var names indicating what to record
     _model_viewer_data : dict
         Structure of model, used to make n2 diagram.
-    _remote_dvs : dict
-        Dict of design variables that are remote on at least one proc. Values are
-        (owning rank, size).
-    _remote_cons : dict
-        Dict of constraints that are remote on at least one proc. Values are
-        (owning rank, size).
-    _remote_objs : dict
-        Dict of objectives that are remote on at least one proc. Values are
-        (owning rank, size).
-    _remote_responses : dict
-        A combined dict containing entries from _remote_cons and _remote_objs.
     _simul_coloring_info : tuple of dicts
         A data structure describing coloring for simultaneous derivs.
     _total_jac_sparsity : dict, str, or None
@@ -130,7 +113,7 @@ class Driver(object):
         # Driver options
         self.options = OptionsDictionary()
 
-        self.options.declare('debug_print', types=list, is_valid=_is_debug_print_opts_valid,
+        self.options.declare('debug_print', types=list, check_valid=_check_debug_print_opts_valid,
                              desc="List of what type of Driver variables to print at each "
                                   "iteration. Valid items in list are 'desvars', 'ln_cons', "
                                   "'nl_cons', 'objs', 'totals'",
@@ -139,8 +122,6 @@ class Driver(object):
         # Case recording options
         self.recording_options = OptionsDictionary()
 
-        self.recording_options.declare('record_metadata', types=bool, default=True,
-                                       desc='Record Driver metadata')
         self.recording_options.declare('record_model_metadata', types=bool, default=True,
                                        desc='Record metadata for all Systems in the model')
         self.recording_options.declare('record_desvars', types=bool, default=True,
@@ -153,7 +134,7 @@ class Driver(object):
         self.recording_options.declare('record_constraints', types=bool, default=True,
                                        desc='Set to True to record constraints at the '
                                             'driver level')
-        self.recording_options.declare('includes', types=list, default=['*'],
+        self.recording_options.declare('includes', types=list, default=[],
                                        desc='Patterns for variables to include in recording')
         self.recording_options.declare('excludes', types=list, default=[],
                                        desc='Patterns for vars to exclude in recording '
@@ -163,9 +144,6 @@ class Driver(object):
                                             'level')
         self.recording_options.declare('record_inputs', types=bool, default=True,
                                        desc='Set to True to record inputs at the driver level')
-        self.recording_options.declare('record_n2_data', types=bool, default=True,
-                                       desc='Set to True to record metadata required for '
-                                       'N^2 viewing')
 
         # What the driver supports.
         self.supports = OptionsDictionary()
@@ -179,8 +157,6 @@ class Driver(object):
         self.supports.declare('active_set', types=bool, default=False)
         self.supports.declare('simultaneous_derivatives', types=bool, default=False)
         self.supports.declare('total_jac_sparsity', types=bool, default=False)
-        # TODO, support these in OpenMDAO
-        self.supports.declare('integer_design_vars', types=bool, default=False)
 
         self.iter_count = 0
         self._model_viewer_data = None
@@ -202,7 +178,7 @@ class Driver(object):
 
         Parameters
         ----------
-        recorder : BaseRecorder
+        recorder : CaseRecorder
            A recorder instance.
         """
         self._rec_mgr.append(recorder)
@@ -250,8 +226,8 @@ class Driver(object):
             Pointer to the containing problem.
         """
         self._problem = problem
+        self._recording_iter = problem._recording_iter
         model = problem.model
-        mode = problem._mode
 
         self._total_jac = None
 
@@ -404,12 +380,6 @@ class Driver(object):
         }
 
         self._rec_mgr.startup(self)
-        if self.recording_options['record_metadata']:
-            if self.recording_options['record_n2_data']:
-                if self._rec_mgr._recorders:
-                    from openmdao.devtools.problem_viewer.problem_viewer import _get_viewer_data
-                    self._model_viewer_data = _get_viewer_data(problem)
-            self._rec_mgr.record_metadata(self)
 
         # Also record the system metadata to the recorders attached to this Driver
         if self.recording_options['record_model_metadata']:
@@ -454,9 +424,10 @@ class Driver(object):
                 else:
                     val = vec[name][indices]
             else:
-                if indices is not None:
+                if not (indices is None or ignore_indices):
                     size = len(indices)
                 val = np.empty(size)
+
             comm.Bcast(val, root=owner)
         else:
             if indices is None or ignore_indices:
@@ -516,8 +487,10 @@ class Driver(object):
         value : float or ndarray
             Value for the design variable.
         """
+        problem = self._problem
+
         if (name in self._remote_dvs and
-                self._problem.model._owning_rank[name] != self._problem.comm.rank):
+                problem.model._owning_rank[name] != problem.comm.rank):
             return
 
         meta = self._designvars[name]
@@ -525,7 +498,7 @@ class Driver(object):
         if indices is None:
             indices = slice(None)
 
-        desvar = self._problem.model._outputs._views_flat[name]
+        desvar = problem.model._outputs._views_flat[name]
         desvar[indices] = value
 
         if self._has_scaling:
@@ -559,7 +532,7 @@ class Driver(object):
 
         return {n: self._get_voi_val(n, self._responses[n], self._remote_objs) for n in resps}
 
-    def get_objective_values(self, unscaled=False, filter=None):
+    def get_objective_values(self, unscaled=False, filter=None, ignore_indices=False):
         """
         Return objective values.
 
@@ -569,6 +542,8 @@ class Driver(object):
             Set to True if unscaled (physical) design variables are desired.
         filter : list
             List of objective names used by recorders.
+        ignore_indices : bool
+            Set to True if the full array is desired, not just those indicated by indices.
 
         Returns
         -------
@@ -580,10 +555,12 @@ class Driver(object):
         else:
             objs = self._objs
 
-        return {n: self._get_voi_val(n, self._objs[n], self._remote_objs, unscaled=unscaled)
+        return {n: self._get_voi_val(n, self._objs[n], self._remote_objs, unscaled=unscaled,
+                                     ignore_indices=ignore_indices)
                 for n in objs}
 
-    def get_constraint_values(self, ctype='all', lintype='all', unscaled=False, filter=None):
+    def get_constraint_values(self, ctype='all', lintype='all', unscaled=False, filter=None,
+                              ignore_indices=False):
         """
         Return constraint values.
 
@@ -599,6 +576,8 @@ class Driver(object):
             Set to True if unscaled (physical) design variables are desired.
         filter : list
             List of constraint names used by recorders.
+        ignore_indices : bool
+            Set to True if the full array is desired, not just those indicated by indices.
 
         Returns
         -------
@@ -626,7 +605,8 @@ class Driver(object):
             if ctype == 'ineq' and meta['equals'] is not None:
                 continue
 
-            con_dict[name] = self._get_voi_val(name, meta, self._remote_cons, unscaled=unscaled)
+            con_dict[name] = self._get_voi_val(name, meta, self._remote_cons, unscaled=unscaled,
+                                               ignore_indices=ignore_indices)
 
         return con_dict
 
@@ -673,11 +653,11 @@ class Driver(object):
             else:
                 objs[name] = data
 
-        response_size = np.sum(resps[n]['size'] for n in self._get_ordered_nl_responses())
+        response_size = sum(resps[n]['size'] for n in self._get_ordered_nl_responses())
 
         # Gather up the information for design vars.
         self._designvars = designvars = model.get_design_vars(recurse=True)
-        desvar_size = np.sum(data['size'] for data in itervalues(designvars))
+        desvar_size = sum(data['size'] for data in itervalues(designvars))
 
         return response_size, desvar_size
 
@@ -693,7 +673,7 @@ class Driver(object):
         boolean
             Failure flag; True if failed to converge, False is successful.
         """
-        with RecordingDebugging(self._get_name(), self.iter_count, self) as rec:
+        with RecordingDebugging(self._get_name(), self.iter_count, self):
             failure_flag, _, _ = self._problem.model._solve_nonlinear()
 
         self.iter_count += 1
@@ -725,6 +705,7 @@ class Driver(object):
         derivs : object
             Derivatives in form requested by 'return_format'.
         """
+        problem = self._problem
         total_jac = self._total_jac
         debug_print = 'totals' in self.options['debug_print'] and (not MPI or
                                                                    MPI.COMM_WORLD.rank == 0)
@@ -734,35 +715,35 @@ class Driver(object):
             print(header)
             print(len(header) * '-' + '\n')
 
-        if self._problem.model._owns_approx_jac:
-            recording_iteration.stack.append(('_compute_totals_approx', 0))
+        if problem.model._owns_approx_jac:
+            self._recording_iter.stack.append(('_compute_totals_approx', 0))
 
             try:
                 if total_jac is None:
-                    total_jac = _TotalJacInfo(self._problem, of, wrt, global_names,
+                    total_jac = _TotalJacInfo(problem, of, wrt, global_names,
                                               return_format, approx=True, debug_print=debug_print)
                     self._total_jac = total_jac
                     totals = total_jac.compute_totals_approx(initialize=True)
                 else:
                     totals = total_jac.compute_totals_approx()
             finally:
-                recording_iteration.stack.pop()
+                self._recording_iter.stack.pop()
 
         else:
             if total_jac is None:
-                total_jac = _TotalJacInfo(self._problem, of, wrt, global_names, return_format,
+                total_jac = _TotalJacInfo(problem, of, wrt, global_names, return_format,
                                           debug_print=debug_print)
 
             # don't cache linear constraint jacobian
             if not total_jac.has_lin_cons:
                 self._total_jac = total_jac
 
-            recording_iteration.stack.append(('_compute_totals', 0))
+            self._recording_iter.stack.append(('_compute_totals', 0))
 
             try:
                 totals = total_jac.compute_totals()
             finally:
-                recording_iteration.stack.pop()
+                self._recording_iter.stack.pop()
 
         if self._rec_mgr._recorders and self.recording_options['record_derivatives']:
             metadata = create_local_meta(self._get_name())
@@ -782,17 +763,17 @@ class Driver(object):
         filt = self._filtered_vars_to_record
 
         if opts['record_desvars']:
-            des_vars = self.get_design_var_values(unscaled=True)
+            des_vars = self.get_design_var_values(unscaled=True, ignore_indices=True)
         else:
             des_vars = {}
 
         if opts['record_objectives']:
-            obj_vars = self.get_objective_values(unscaled=True)
+            obj_vars = self.get_objective_values(unscaled=True, ignore_indices=True)
         else:
             obj_vars = {}
 
         if opts['record_constraints']:
-            con_vars = self.get_constraint_values(unscaled=True)
+            con_vars = self.get_constraint_values(unscaled=True, ignore_indices=True)
         else:
             con_vars = {}
 
@@ -989,13 +970,25 @@ class Driver(object):
         if not coloring_mod._use_sparsity:
             return
 
+        problem = self._problem
+        if not problem.model._use_derivatives:
+            simple_warning("Derivatives are turned off.  Skipping simul deriv coloring.")
+            return
+
         if isinstance(self._simul_coloring_info, string_types):
             with open(self._simul_coloring_info, 'r') as f:
-                self._simul_coloring_info = json.load(f)
+                self._simul_coloring_info = coloring_mod._json2coloring(json.load(f))
 
-        if 'rev' in self._simul_coloring_info and self._problem._orig_mode not in ('rev', 'auto'):
-            raise RuntimeError("Simultaneous coloring does reverse solves but mode has "
-                               "been set to '%s'" % self._problem._orig_mode)
+        if 'rev' in self._simul_coloring_info and problem._orig_mode not in ('rev', 'auto'):
+            revcol = self._simul_coloring_info['rev'][0][0]
+            if revcol:
+                raise RuntimeError("Simultaneous coloring does reverse solves but mode has "
+                                   "been set to '%s'" % problem._orig_mode)
+        if 'fwd' in self._simul_coloring_info and problem._orig_mode not in ('fwd', 'auto'):
+            fwdcol = self._simul_coloring_info['fwd'][0][0]
+            if fwdcol:
+                raise RuntimeError("Simultaneous coloring does forward solves but mode has "
+                                   "been set to '%s'" % problem._orig_mode)
 
         # simul_coloring_info can contain data for either fwd, rev, or both, along with optional
         # sparsity patterns
@@ -1020,7 +1013,7 @@ class Driver(object):
 
         if not MPI or MPI.COMM_WORLD.rank == 0:
             header = 'Driver debug print for iter coord: {}'.format(
-                recording_iteration.get_formatted_iteration_coordinate())
+                self._recording_iter.get_formatted_iteration_coordinate())
             print(header)
             print(len(header) * '-')
 
