@@ -88,7 +88,7 @@ class ExplicitComponent(Component):
                                    val=np.full(meta['size'], -1.))
 
     def add_output(self, name, val=1.0, shape=None, units=None, res_units=None, desc='',
-                   lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=None, var_set=0):
+                   lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=None):
         """
         Add an output variable to the component.
 
@@ -131,9 +131,6 @@ class ExplicitComponent(Component):
             Scaling parameter. The value in the user-defined res_units of this output's residual
             when the scaled value is 1. Default is None, which means residual scaling matches
             output scaling.
-        var_set : hashable object
-            For advanced users only. ID or color for this variable, relevant for reconfigurability.
-            Default is 0.
 
         Returns
         -------
@@ -147,8 +144,7 @@ class ExplicitComponent(Component):
                                                          val=val, shape=shape, units=units,
                                                          res_units=res_units, desc=desc,
                                                          lower=lower, upper=upper,
-                                                         ref=ref, ref0=ref0, res_ref=res_ref,
-                                                         var_set=var_set)
+                                                         ref=ref, ref0=ref0, res_ref=res_ref)
 
     def _set_partials_meta(self):
         """
@@ -156,6 +152,7 @@ class ExplicitComponent(Component):
         """
         for abs_key, meta in iteritems(self._subjacs_info):
 
+            # if there isn't a declared partial value, set it to a dense matrix
             if meta['value'] is None:
                 meta['value'] = np.zeros(meta['shape'])
 
@@ -166,9 +163,6 @@ class ExplicitComponent(Component):
                         not in self._outputs._views_flat):
                     self._approx_schemes[method].add_approximation(abs_key, meta)
 
-        for approx in itervalues(self._approx_schemes):
-            approx._init_approximations()
-
     def _apply_nonlinear(self):
         """
         Compute residuals. The model is assumed to be in a scaled state.
@@ -182,12 +176,15 @@ class ExplicitComponent(Component):
                 # Sign of the residual is minus the sign of the output vector.
                 residuals *= -1.0
 
-                self.compute(self._inputs, outputs)
-
-                # Restore any complex views if under complex step.
-                if outputs._vector_info._under_complex_step:
-                    outputs._remove_complex_views()
-                    residuals._remove_complex_views()
+                self._inputs.read_only = True
+                try:
+                    if self._discrete_inputs or self._discrete_outputs:
+                        self.compute(self._inputs, self._outputs, self._discrete_inputs,
+                                     self._discrete_outputs)
+                    else:
+                        self.compute(self._inputs, self._outputs)
+                finally:
+                    self._inputs.read_only = False
 
                 residuals += outputs
                 outputs -= residuals
@@ -207,15 +204,18 @@ class ExplicitComponent(Component):
         """
         super(ExplicitComponent, self)._solve_nonlinear()
 
-        self._inputs.read_only = True
-
         with Recording(self.pathname + '._solve_nonlinear', self.iter_count, self):
-            with self._unscaled_context(
-                    outputs=[self._outputs], residuals=[self._residuals]):
+            with self._unscaled_context(outputs=[self._outputs], residuals=[self._residuals]):
                 self._residuals.set_const(0.0)
-                failed = self.compute(self._inputs, self._outputs)
-
-        self._inputs.read_only = False
+                self._inputs.read_only = True
+                try:
+                    if self._discrete_inputs or self._discrete_outputs:
+                        failed = self.compute(self._inputs, self._outputs, self._discrete_inputs,
+                                              self._discrete_outputs)
+                    else:
+                        failed = self.compute(self._inputs, self._outputs)
+                finally:
+                    self._inputs.read_only = False
 
         return bool(failed), 0., 0.
 
@@ -242,36 +242,34 @@ class ExplicitComponent(Component):
         """
         J = self._jacobian if jac is None else jac
 
-        with Recording(self.pathname + '._apply_linear', self.iter_count, self):
-            for vec_name in vec_names:
-                if vec_name not in self._rel_vec_names:
+        for vec_name in vec_names:
+            if vec_name not in self._rel_vec_names:
+                continue
+
+            with self._matvec_context(vec_name, scope_out, scope_in, mode) as vecs:
+                d_inputs, d_outputs, d_residuals = vecs
+
+                # Jacobian and vectors are all scaled, unitless
+                J._apply(self, d_inputs, d_outputs, d_residuals, mode)
+
+                # if we're not matrix free, we can skip the bottom of
+                # this loop because compute_jacvec_product does nothing.
+                if not self.matrix_free:
                     continue
 
-                with self._matvec_context(vec_name, scope_out, scope_in, mode) as vecs:
-                    d_inputs, d_outputs, d_residuals = vecs
+                # Jacobian and vectors are all unscaled, dimensional
+                with self._unscaled_context(
+                        outputs=[self._outputs], residuals=[d_residuals]):
 
-                    # Jacobian and vectors are all scaled, unitless
-                    with self.jacobian_context(J):
-                        J._apply(d_inputs, d_outputs, d_residuals, mode)
-
-                    # if we're not matrix free, we can skip the bottom of
-                    # this loop because compute_jacvec_product does nothing.
-                    if not self.matrix_free:
-                        continue
-
-                    # Jacobian and vectors are all unscaled, dimensional
-                    with self._unscaled_context(
-                            outputs=[self._outputs], residuals=[d_residuals]):
-
-                        # set appropriate vectors to read_only to help prevent user error
-                        self._inputs.read_only = True
-                        if mode == 'fwd':
-                            d_inputs.read_only = True
-                        elif mode == 'rev':
-                            d_residuals.read_only = True
-
-                        # We used to negate the residual here, and then re-negate after the hook.
+                    # set appropriate vectors to read_only to help prevent user error
+                    self._inputs.read_only = True
+                    if mode == 'fwd':
+                        d_inputs.read_only = True
+                    elif mode == 'rev':
+                        d_residuals.read_only = True
 
+                    try:
+                        # We used to negate the residual here, and then re-negate after the hook
                         if d_inputs._ncol > 1:
                             if self.supports_multivecs:
                                 self.compute_multi_jacvec_product(self._inputs, d_inputs,
@@ -287,9 +285,11 @@ class ExplicitComponent(Component):
                                 d_inputs._icol = None
                                 d_residuals._icol = None
                         else:
-                            self.compute_jacvec_product(self._inputs, d_inputs, d_residuals, mode)
-
-                        self._inputs.read_only = d_inputs.read_only = d_residuals.read_only = False
+                            self.compute_jacvec_product(self._inputs, d_inputs,
+                                                        d_residuals, mode)
+                    finally:
+                        self._inputs.read_only = False
+                        d_inputs.read_only = d_residuals.read_only = False
 
     def _solve_linear(self, vec_names, mode, rel_systems):
         """
@@ -313,33 +313,32 @@ class ExplicitComponent(Component):
         float
             relative error.
         """
-        with Recording(self.pathname + '._solve_linear', self.iter_count, self):
-            for vec_name in vec_names:
-                if vec_name in self._rel_vec_names:
-                    d_outputs = self._vectors['output'][vec_name]
-                    d_residuals = self._vectors['residual'][vec_name]
+        for vec_name in vec_names:
+            if vec_name in self._rel_vec_names:
+                d_outputs = self._vectors['output'][vec_name]
+                d_residuals = self._vectors['residual'][vec_name]
 
-                    if mode == 'fwd':
-                        if self._has_resid_scaling:
-                            with self._unscaled_context(outputs=[d_outputs],
-                                                        residuals=[d_residuals]):
-                                d_outputs.set_vec(d_residuals)
-                        else:
+                if mode == 'fwd':
+                    if self._has_resid_scaling:
+                        with self._unscaled_context(outputs=[d_outputs],
+                                                    residuals=[d_residuals]):
                             d_outputs.set_vec(d_residuals)
+                    else:
+                        d_outputs.set_vec(d_residuals)
 
-                        # ExplicitComponent jacobian defined with -1 on diagonal.
-                        d_outputs *= -1.0
+                    # ExplicitComponent jacobian defined with -1 on diagonal.
+                    d_outputs *= -1.0
 
-                    else:  # rev
-                        if self._has_resid_scaling:
-                            with self._unscaled_context(outputs=[d_outputs],
-                                                        residuals=[d_residuals]):
-                                d_residuals.set_vec(d_outputs)
-                        else:
+                else:  # rev
+                    if self._has_resid_scaling:
+                        with self._unscaled_context(outputs=[d_outputs],
+                                                    residuals=[d_residuals]):
                             d_residuals.set_vec(d_outputs)
+                    else:
+                        d_residuals.set_vec(d_outputs)
 
-                        # ExplicitComponent jacobian defined with -1 on diagonal.
-                        d_residuals *= -1.0
+                    # ExplicitComponent jacobian defined with -1 on diagonal.
+                    d_residuals *= -1.0
 
         return False, 0., 0.
 
@@ -350,7 +349,7 @@ class ExplicitComponent(Component):
         Parameters
         ----------
         jac : Jacobian or None
-            If None, use local jacobian, else use assembled jacobian jac.
+            Ignored.
         sub_do_ln : boolean
             Flag indicating if the children should call linearize on their linear solvers.
         """
@@ -366,12 +365,17 @@ class ExplicitComponent(Component):
             if self._has_compute_partials:
                 self._inputs.read_only = True
 
-                # We used to negate the jacobian here, and then re-negate after the hook.
-                self.compute_partials(self._inputs, self._jacobian)
+                # We don't need to set the _system attribute on jac here because jac (if not None)
+                # shares the _subjacs_info metadata with our _jacobian, and our _jacobian knows
+                # how to properly convert relative names (used by the component in compute_partials)
+                # to absolute names (used by all jacobians internally).
+                try:
+                    # We used to negate the jacobian here, and then re-negate after the hook.
+                    self.compute_partials(self._inputs, self._jacobian)
+                finally:
+                    self._inputs.read_only = False
 
-                self._inputs.read_only = False
-
-    def compute(self, inputs, outputs):
+    def compute(self, inputs, outputs, discrete_inputs=None, discrete_outputs=None):
         """
         Compute outputs given inputs. The model is assumed to be in an unscaled state.
 
@@ -381,6 +385,10 @@ class ExplicitComponent(Component):
             unscaled, dimensional input variables read via inputs[key]
         outputs : Vector
             unscaled, dimensional output variables read via outputs[key]
+        discrete_inputs : dict or None
+            If not None, dict containing discrete input values.
+        discrete_outputs : dict or None
+            If not None, dict containing discrete output values.
 
         Returns
         -------
