@@ -69,13 +69,17 @@ class ImplicitComponent(Component):
         """
         Compute residuals. The model is assumed to be in a scaled state.
         """
-        self._inputs.read_only = self._outputs.read_only = True
-
         with self._unscaled_context(outputs=[self._outputs], residuals=[self._residuals]):
             with Recording(self.pathname + '._apply_nonlinear', self.iter_count, self):
-                self.apply_nonlinear(self._inputs, self._outputs, self._residuals)
-
-        self._inputs.read_only = self._outputs.read_only = False
+                self._inputs.read_only = self._outputs.read_only = True
+                try:
+                    if self._discrete_inputs or self._discrete_outputs:
+                        self.apply_nonlinear(self._inputs, self._outputs, self._residuals,
+                                             self._discrete_inputs, self._discrete_outputs)
+                    else:
+                        self.apply_nonlinear(self._inputs, self._outputs, self._residuals)
+                finally:
+                    self._inputs.read_only = self._outputs.read_only = False
 
     def _solve_nonlinear(self):
         """
@@ -95,15 +99,21 @@ class ImplicitComponent(Component):
 
         self._inputs.read_only = True
 
-        if self._nonlinear_solver is not None:
-            with Recording(self.pathname + '._solve_nonlinear', self.iter_count, self):
-                result = self._nonlinear_solver.solve()
-        else:
-            with self._unscaled_context(outputs=[self._outputs]):
+        try:
+            if self._nonlinear_solver is not None:
                 with Recording(self.pathname + '._solve_nonlinear', self.iter_count, self):
-                    result = self.solve_nonlinear(self._inputs, self._outputs)
-
-        self._inputs.read_only = False
+                    result = self._nonlinear_solver.solve()
+            else:
+                with self._unscaled_context(outputs=[self._outputs]):
+                    with Recording(self.pathname + '._solve_nonlinear', self.iter_count, self):
+                        if self._discrete_inputs or self._discrete_outputs:
+                            result = self.solve_nonlinear(self._inputs, self._outputs,
+                                                          self._discrete_inputs,
+                                                          self._discrete_outputs)
+                        else:
+                            result = self.solve_nonlinear(self._inputs, self._outputs)
+        finally:
+            self._inputs.read_only = False
 
         if result is None:
             return False, 0., 0.
@@ -118,10 +128,11 @@ class ImplicitComponent(Component):
         """
         self._inputs.read_only = self._residuals.read_only = True
 
-        with self._unscaled_context(outputs=[self._outputs], residuals=[self._residuals]):
-            self.guess_nonlinear(self._inputs, self._outputs, self._residuals)
-
-        self._inputs.read_only = self._residuals.read_only = False
+        try:
+            with self._unscaled_context(outputs=[self._outputs], residuals=[self._residuals]):
+                self.guess_nonlinear(self._inputs, self._outputs, self._residuals)
+        finally:
+            self._inputs.read_only = self._residuals.read_only = False
 
     def _apply_linear(self, jac, vec_names, rel_systems, mode, scope_out=None, scope_in=None):
         """
@@ -155,25 +166,25 @@ class ImplicitComponent(Component):
                 d_inputs, d_outputs, d_residuals = vecs
 
                 # Jacobian and vectors are all scaled, unitless
-                with self.jacobian_context(jac):
-                    jac._apply(d_inputs, d_outputs, d_residuals, mode)
+                jac._apply(self, d_inputs, d_outputs, d_residuals, mode)
 
                 # if we're not matrix free, we can skip the bottom of
                 # this loop because apply_linear does nothing.
                 if not self.matrix_free:
                     continue
 
-                # set appropriate vectors to read_only to help prevent user error
-                self._inputs.read_only = self._outputs.read_only = True
-                if mode == 'fwd':
-                    d_inputs.read_only = d_outputs.read_only = True
-                elif mode == 'rev':
-                    d_residuals.read_only = True
-
                 # Jacobian and vectors are all unscaled, dimensional
                 with self._unscaled_context(
                         outputs=[self._outputs, d_outputs], residuals=[d_residuals]):
-                    with Recording(self.pathname + '._apply_linear', self.iter_count, self):
+
+                    # set appropriate vectors to read_only to help prevent user error
+                    self._inputs.read_only = self._outputs.read_only = True
+                    if mode == 'fwd':
+                        d_inputs.read_only = d_outputs.read_only = True
+                    elif mode == 'rev':
+                        d_residuals.read_only = True
+
+                    try:
                         if d_inputs._ncol > 1:
                             if self.has_apply_multi_linear:
                                 self.apply_multi_linear(self._inputs, self._outputs,
@@ -193,9 +204,9 @@ class ImplicitComponent(Component):
                         else:
                             self.apply_linear(self._inputs, self._outputs,
                                               d_inputs, d_outputs, d_residuals, mode)
-
-                self._inputs.read_only = self._outputs.read_only = False
-                d_inputs.read_only = d_outputs.read_only = d_residuals.read_only = False
+                    finally:
+                        self._inputs.read_only = self._outputs.read_only = False
+                        d_inputs.read_only = d_outputs.read_only = d_residuals.read_only = False
 
     def _solve_linear(self, vec_names, mode, rel_systems):
         """
@@ -220,9 +231,7 @@ class ImplicitComponent(Component):
             relative error.
         """
         if self._linear_solver is not None:
-            with Recording(self.pathname + '._solve_linear', self.iter_count, self):
-                result = self._linear_solver.solve(vec_names, mode, rel_systems)
-
+            result = self._linear_solver.solve(vec_names, mode, rel_systems)
             return result
 
         else:
@@ -235,14 +244,14 @@ class ImplicitComponent(Component):
                 d_outputs = self._vectors['output'][vec_name]
                 d_residuals = self._vectors['residual'][vec_name]
 
-                # set appropriate vectors to read_only to help prevent user error
-                if mode == 'fwd':
-                    d_residuals.read_only = True
-                elif mode == 'rev':
-                    d_outputs.read_only = True
-
                 with self._unscaled_context(outputs=[d_outputs], residuals=[d_residuals]):
-                    with Recording(self.pathname + '._solve_linear', self.iter_count, self):
+                    # set appropriate vectors to read_only to help prevent user error
+                    if mode == 'fwd':
+                        d_residuals.read_only = True
+                    elif mode == 'rev':
+                        d_outputs.read_only = True
+
+                    try:
                         if d_outputs._ncol > 1:
                             if self.has_solve_multi_linear:
                                 result = self.solve_multi_linear(d_outputs, d_residuals, mode)
@@ -264,6 +273,8 @@ class ImplicitComponent(Component):
                                 d_residuals._icol = None
                         else:
                             result = self.solve_linear(d_outputs, d_residuals, mode)
+                    finally:
+                        d_outputs.read_only = d_residuals.read_only = False
 
                 if isinstance(result, bool):
                     failed |= result
@@ -272,8 +283,6 @@ class ImplicitComponent(Component):
                     abs_errors.append(result[1])
                     rel_errors.append(result[2])
 
-                d_outputs.read_only = d_residuals.read_only = False
-
             return failed, np.linalg.norm(abs_errors), np.linalg.norm(rel_errors)
 
     def _linearize(self, jac=None, sub_do_ln=True):
@@ -287,6 +296,9 @@ class ImplicitComponent(Component):
         sub_do_ln : boolean
             Flag indicating if the children should call linearize on their linear solvers.
         """
+        if self._assembled_jac is not None:
+            self._assembled_jac._reset_mats()
+
         with self._unscaled_context(outputs=[self._outputs]):
             # Computing the approximation before the call to compute_partials allows users to
             # override FD'd values.
@@ -295,9 +307,10 @@ class ImplicitComponent(Component):
 
             self._inputs.read_only = self._outputs.read_only = True
 
-            self.linearize(self._inputs, self._outputs, self._jacobian)
-
-            self._inputs.read_only = self._outputs.read_only = False
+            try:
+                self.linearize(self._inputs, self._outputs, self._jacobian)
+            finally:
+                self._inputs.read_only = self._outputs.read_only = False
 
         if (jac is None or jac is self._assembled_jac) and self._assembled_jac is not None:
             self._assembled_jac._update(self)
