@@ -4,13 +4,11 @@ from __future__ import division, print_function
 
 import sys
 
-from collections import OrderedDict, defaultdict, namedtuple
+from collections import defaultdict, namedtuple
 from fnmatch import fnmatchcase
 from itertools import product
-import warnings
-import json
 
-from six import iteritems, iterkeys, itervalues, string_types
+from six import iteritems, iterkeys, itervalues
 from six.moves import range, cStringIO
 
 import numpy as np
@@ -20,16 +18,17 @@ from openmdao.approximation_schemes.complex_step import ComplexStep, DEFAULT_CS_
 from openmdao.approximation_schemes.finite_difference import FiniteDifference, DEFAULT_FD_OPTIONS
 from openmdao.core.component import Component
 from openmdao.core.driver import Driver
+from openmdao.solvers.solver import SolverInfo
 from openmdao.core.explicitcomponent import ExplicitComponent
 from openmdao.core.group import Group
+from openmdao.core.group import System
 from openmdao.core.indepvarcomp import IndepVarComp
 from openmdao.core.total_jac import _TotalJacInfo
-from openmdao.vectors.transfer import Transfer
 from openmdao.error_checking.check_config import check_config
-from openmdao.recorders.recording_iteration_stack import recording_iteration
-from openmdao.recorders.recording_manager import RecordingManager
+from openmdao.recorders.recording_iteration_stack import _RecIteration
+from openmdao.recorders.recording_manager import RecordingManager, record_viewer_data
 from openmdao.utils.record_util import create_local_meta, check_path
-from openmdao.utils.general_utils import warn_deprecation, ContainsAll, pad_name
+from openmdao.utils.general_utils import warn_deprecation, ContainsAll, pad_name, simple_warning
 from openmdao.utils.mpi import FakeComm
 from openmdao.utils.mpi import MPI
 from openmdao.utils.name_maps import prom_name2abs_name
@@ -37,6 +36,7 @@ from openmdao.utils.options_dictionary import OptionsDictionary
 from openmdao.utils.units import get_conversion
 from openmdao.utils import coloring
 from openmdao.vectors.default_vector import DefaultVector
+from openmdao.utils.name_maps import abs_key2rel_key
 
 try:
     from openmdao.vectors.petsc_vector import PETScVector
@@ -84,8 +84,6 @@ class Problem(object):
     _mode : 'fwd' or 'rev'
         Derivatives calculation mode, 'fwd' for forward, and 'rev' for
         reverse (adjoint).
-    _use_ref_vector : bool
-        If True, allocate vectors to store ref. values.
     _solver_print_cache : list
         Allows solver iprints to be set to requested values after setup calls.
     _initial_condition_cache : dict
@@ -97,7 +95,7 @@ class Problem(object):
         1 -- The `setup` method has been called, but vectors not initialized.
         2 -- The `final_setup` has been run, everything ready to run.
     cite : str
-        Listing of relevant citataions that should be referenced when
+        Listing of relevant citations that should be referenced when
         publishing work that uses this class.
     recording_options : <OptionsDictionary>
         Dictionary with problem recording options.
@@ -109,18 +107,18 @@ class Problem(object):
 
     _post_setup_func = None
 
-    def __init__(self, model=None, comm=None, use_ref_vector=True, root=None):
+    def __init__(self, model=None, driver=None, comm=None, root=None):
         """
         Initialize attributes.
 
         Parameters
         ----------
         model : <System> or None
-            Pointer to the top-level <System> object (root node in the tree).
+            The top-level <System>. If not specified, an empty <Group> will be created.
+        driver : <Driver> or None
+            The driver for the problem. If not specified, a simple "Run Once" driver will be used.
         comm : MPI.Comm or <FakeComm> or None
             The global communicator.
-        use_ref_vector : bool
-            If True, allocate vectors to store ref. values.
         root : <System> or None
             Deprecated kwarg for `model`.
         """
@@ -135,8 +133,8 @@ class Problem(object):
 
         if root is not None:
             if model is not None:
-                raise ValueError("cannot specify both `root` and `model`. `root` has been "
-                                 "deprecated, please use model")
+                raise ValueError("Cannot specify both 'root' and 'model'. "
+                                 "'root' has been deprecated, please use 'model'.")
 
             warn_deprecation("The 'root' argument provides backwards compatibility "
                              "with OpenMDAO <= 1.x ; use 'model' instead.")
@@ -144,19 +142,25 @@ class Problem(object):
             model = root
 
         if model is None:
-            model = Group()
+            self.model = Group()
+        elif isinstance(model, System):
+            self.model = model
+        else:
+            raise TypeError("The value provided for 'model' is not a valid System.")
+
+        if driver is None:
+            self.driver = Driver()
+        elif isinstance(driver, Driver):
+            self.driver = driver
+        else:
+            raise TypeError("The value provided for 'driver' is not a valid Driver.")
 
-        self.model = model
         self.comm = comm
-        self.driver = Driver()
 
-        self._use_ref_vector = use_ref_vector
         self._solver_print_cache = []
 
         self._mode = None  # mode is assigned in setup()
 
-        recording_iteration.stack = []
-
         self._initial_condition_cache = {}
 
         # Status of the setup of _model.
@@ -175,8 +179,6 @@ class Problem(object):
         # Case recording options
         self.recording_options = OptionsDictionary()
 
-        self.recording_options.declare('record_metadata', types=bool, default=True,
-                                       desc='Record metadata')
         self.recording_options.declare('record_desvars', types=bool, default=True,
                                        desc='Set to True to record design variables at the '
                                             'problem level')
@@ -206,6 +208,8 @@ class Problem(object):
             the requested output/input variable.
         """
         # Caching only needed if vectors aren't allocated yet.
+        proms = self.model._var_allprocs_prom2abs_list
+
         if self._setup_status == 1:
 
             # We have set and cached already
@@ -258,8 +262,20 @@ class Problem(object):
             val = self.model._inputs[name]
 
         else:
-            msg = 'Variable name "{}" not found.'
-            raise KeyError(msg.format(name))
+            if name in proms['input']:
+                abs_name = prom_name2abs_name(self.model, name, 'input')
+            else:
+                abs_name = prom_name2abs_name(self.model, name, 'output')
+
+            if self.model._discrete_outputs and name in self.model._discrete_outputs:
+                val = self.model._discrete_outputs[name]
+
+            elif self.model._discrete_inputs and name in self.model._discrete_inputs:
+                val = self.model._discrete_inputs[name]
+
+            else:
+                msg = 'Variable name "{}" not found.'
+                raise KeyError(msg.format(name))
 
         # Need to cache the "get" in case the user calls in-place numpy operations.
         self._initial_condition_cache[name] = val
@@ -377,10 +393,14 @@ class Problem(object):
         if self._setup_status == 1:
             self._initial_condition_cache[name] = value
         else:
-            if name in self.model._outputs:
+            if self.model._outputs and name in self.model._outputs:
                 self.model._outputs[name] = value
-            elif name in self.model._inputs:
+            elif self.model._discrete_outputs and name in self.model._discrete_outputs:
+                self.model._discrete_outputs[name] = value
+            elif self.model._inputs and name in self.model._inputs:
                 self.model._inputs[name] = value
+            elif self.model._discrete_inputs and name in self.model._discrete_inputs:
+                self.model._discrete_inputs[name] = value
             else:
                 msg = 'Variable name "{}" not found.'
                 raise KeyError(msg.format(name))
@@ -487,9 +507,9 @@ class Problem(object):
         if case_prefix:
             if not isinstance(case_prefix, str):
                 raise TypeError("The 'case_prefix' argument should be a string.")
-            recording_iteration.prefix = case_prefix
+            self._recording_iter.prefix = case_prefix
         else:
-            recording_iteration.prefix = None
+            self._recording_iter.prefix = None
 
         if self.model.iter_count > 0 and reset_iter_counts:
             self.driver.iter_count = 0
@@ -522,9 +542,9 @@ class Problem(object):
         if case_prefix:
             if not isinstance(case_prefix, str):
                 raise TypeError("The 'case_prefix' argument should be a string.")
-            recording_iteration.prefix = case_prefix
+            self._recording_iter.prefix = case_prefix
         else:
-            recording_iteration.prefix = None
+            self._recording_iter.prefix = None
 
         if self.model.iter_count > 0 and reset_iter_counts:
             self.driver.iter_count = 0
@@ -574,7 +594,7 @@ class Problem(object):
         model = self.model
         driver = self.driver
 
-        mydesvars = myobjectives = myconstraints = myresponses = set()
+        mydesvars = myobjectives = myconstraints = set()
 
         incl = self.recording_options['includes']
         excl = self.recording_options['excludes']
@@ -639,8 +659,8 @@ class Problem(object):
         }
 
         self._rec_mgr.startup(self)
-        if self.recording_options['record_metadata']:
-            self._rec_mgr.record_metadata(self)
+
+        record_viewer_data(self)
 
     def add_recorder(self, recorder):
         """
@@ -648,7 +668,7 @@ class Problem(object):
 
         Parameters
         ----------
-        recorder : BaseRecorder
+        recorder : CaseRecorder
            A recorder instance.
         """
         self._rec_mgr.append(recorder)
@@ -657,6 +677,10 @@ class Problem(object):
         """
         Clean up resources prior to exit.
         """
+        # shut down all recorders
+        self._rec_mgr.shutdown()
+
+        # clean up driver and model resources
         self.driver.cleanup()
         for system in self.model.system_iter(include_self=True, recurse=True):
             system.cleanup()
@@ -699,9 +723,6 @@ class Problem(object):
         obj_vars = {name: obj_vars[name] for name in filt['obj']}
         con_vars = {name: con_vars[name] for name in filt['con']}
 
-        names = model._outputs._names
-        views = model._outputs._views
-
         if MPI:
             des_vars = model._gather_vars(model, des_vars)
             obj_vars = model._gather_vars(model, obj_vars)
@@ -723,13 +744,13 @@ class Problem(object):
 
     def setup(self, vector_class=None, check=False, logger=None, mode='auto',
               force_alloc_complex=False, distributed_vector_class=PETScVector,
-              local_vector_class=DefaultVector):
+              local_vector_class=DefaultVector, derivatives=True):
         """
         Set up the model hierarchy.
 
         When `setup` is called, the model hierarchy is assembled, the processors are allocated
         (for MPI), and variables and connections are all assigned. This method traverses down
-        the model hierarchy to call `setup` on each subsystem, and then traverses up te model
+        the model hierarchy to call `setup` on each subsystem, and then traverses up the model
         hierarchy to call `configure` on each subsystem.
 
         Parameters
@@ -756,6 +777,8 @@ class Problem(object):
         local_vector_class : type
             Reference to the <Vector> class or factory function used to instantiate vectors
             and associated transfers involved in intraprocess communication.
+        derivatives : bool
+            If True, perform any memory allocations necessary for derivative computation.
 
         Returns
         -------
@@ -763,6 +786,7 @@ class Problem(object):
             this enables the user to instantiate and setup in one line.
         """
         model = self.model
+        model.force_alloc_complex = force_alloc_complex
         comm = self.comm
 
         if vector_class is not None:
@@ -786,9 +810,15 @@ class Problem(object):
 
         self._mode = self._orig_mode = mode
 
+        # this will be shared by all Solvers in the model
+        model._solver_info = SolverInfo()
+        self._recording_iter = _RecIteration()
+        model._recording_iter = self._recording_iter
+
         model_comm = self.driver._setup_comm(comm)
 
-        model._setup(model_comm, 'full', mode, distributed_vector_class, local_vector_class)
+        model._setup(model_comm, 'full', mode, distributed_vector_class, local_vector_class,
+                     derivatives)
 
         # Cache all args for final setup.
         self._check = check
@@ -809,8 +839,6 @@ class Problem(object):
         are created and populated, the drivers and solvers are initialized, and the recorders are
         started, and the rest of the framework is prepared for execution.
         """
-        orig_mode = self._mode
-
         response_size, desvar_size = self.driver._update_voi_meta(self.model)
 
         # update mode if it's been set to 'auto'
@@ -832,7 +860,7 @@ class Problem(object):
             # than the full size
             if 'fwd' in coloring_info and 'rev' in coloring_info:
                 pass  # we're doing both!
-            else:
+            elif mode in coloring_info:
                 lists = coloring_info[mode][0]
                 if lists:
                     size = len(lists[0])  # lists[0] is the uncolored row/col indices
@@ -844,10 +872,10 @@ class Problem(object):
 
         if ((mode == 'fwd' and desvar_size > response_size) or
                 (mode == 'rev' and response_size > desvar_size)):
-            warnings.warn("Inefficient choice of derivative mode.  You chose '%s' for a "
-                          "problem with %d design variables and %d response variables "
-                          "(objectives and nonlinear constraints)." %
-                          (mode, desvar_size, response_size), RuntimeWarning)
+            simple_warning("Inefficient choice of derivative mode.  You chose '%s' for a "
+                           "problem with %d design variables and %d response variables "
+                           "(objectives and nonlinear constraints)." %
+                           (mode, desvar_size, response_size), RuntimeWarning)
 
         self._setup_recording()
 
@@ -868,9 +896,7 @@ class Problem(object):
 
     def check_partials(self, out_stream=_DEFAULT_OUT_STREAM, includes=None, excludes=None,
                        compact_print=False, abs_err_tol=1e-6, rel_err_tol=1e-6,
-                       method='fd', step=DEFAULT_FD_OPTIONS['step'],
-                       form=DEFAULT_FD_OPTIONS['form'],
-                       step_calc=DEFAULT_FD_OPTIONS['step_calc'],
+                       method='fd', step=None, form='forward', step_calc='abs',
                        force_dense=True, show_only_incorrect=False):
         """
         Check partial derivatives comprehensively for all components in your model.
@@ -898,13 +924,14 @@ class Problem(object):
         method : str
             Method, 'fd' for finite difference or 'cs' for complex step. Default is 'fd'.
         step : float
-            Step size for approximation. Default is the default value of step for the 'fd' method.
+            Step size for approximation. Default is None, which means 1e-6 for 'fd' and 1e-40 for
+            'cs'.
         form : string
             Form for finite difference, can be 'forward', 'backward', or 'central'. Default
-            is the default value of step for the 'fd' method.
+            'forward'.
         step_calc : string
-            Step type for finite difference, can be 'abs' for absolute', or 'rel' for
-            relative. Default is the default value of step for the 'fd' method.
+            Step type for finite difference, can be 'abs' for absolute', or 'rel' for relative.
+            Default is 'abs'.
         force_dense : bool
             If True, analytic derivatives will be coerced into arrays. Default is True.
         show_only_incorrect : bool, optional
@@ -930,6 +957,9 @@ class Problem(object):
 
         model = self.model
 
+        if not model._use_derivatives:
+            raise RuntimeError("Can't check partials.  Derivative support has been turned off.")
+
         # TODO: Once we're tracking iteration counts, run the model if it has not been run before.
 
         includes = [includes] if isinstance(includes, str) else includes
@@ -986,7 +1016,9 @@ class Problem(object):
 
             for comp in comps:
 
-                comp.run_linearize()
+                # Only really need to linearize once.
+                if mode == 'fwd':
+                    comp.run_linearize()
 
                 explicit = isinstance(comp, ExplicitComponent)
                 matrix_free = comp.matrix_free
@@ -1092,7 +1124,7 @@ class Problem(object):
                             # them unless the fd is non zero. Note: subjacs_info is empty for
                             # undeclared partials, which is the default behavior now.
                             try:
-                                if subjacs[abs_key]['dependent'] is False:
+                                if not subjacs[abs_key]['dependent']:
                                     indep_key[c_name].add(rel_key)
                             except KeyError:
                                 indep_key[c_name].add(rel_key)
@@ -1162,7 +1194,7 @@ class Problem(object):
                     if local_method:
                         method = local_method
 
-                # We can't use CS if we havent' allocated a complex vector, so we fall back on fd.
+                # We can't use CS if we haven't allocated a complex vector, so we fall back on fd.
                 if method == 'cs' and not alloc_complex:
                     comps_could_not_cs.add(c_name)
                     method = 'fd'
@@ -1200,13 +1232,11 @@ class Problem(object):
 
             approx_jac = {}
             for approximation in itervalues(approximations):
-                approximation._init_approximations()
-
-                # Peform the FD here.
+                # Perform the FD here.
                 approximation.compute_approximations(comp, jac=approx_jac)
 
-            for rel_key, partial in iteritems(approx_jac):
-                abs_key = rel_key2abs_key(comp, rel_key)
+            for abs_key, partial in iteritems(approx_jac):
+                rel_key = abs_key2rel_key(comp, abs_key)
                 partials_data[c_name][rel_key][jac_key] = partial
 
         # Conversion of defaultdict to dicts
@@ -1217,9 +1247,11 @@ class Problem(object):
 
         if len(comps_could_not_cs) > 0:
             msg = "The following components requested complex step, but force_alloc_complex " + \
-                "has not been set to True, so finite difference was used: "
+                  "has not been set to True, so finite difference was used: "
             msg += str(list(comps_could_not_cs))
-            warnings.warn(msg)
+            msg += "\nTo enable complex step, specify 'force_alloc_complex=True' when calling " + \
+                   "setup on the problem, e.g. 'problem.setup(force_alloc_complex=True)'"
+            simple_warning(msg)
 
         _assemble_derivative_data(partials_data, rel_err_tol, abs_err_tol, out_stream,
                                   compact_print, comps, all_fd_options, indep_key=indep_key,
@@ -1230,7 +1262,7 @@ class Problem(object):
 
     def check_totals(self, of=None, wrt=None, out_stream=_DEFAULT_OUT_STREAM, compact_print=False,
                      driver_scaling=False, abs_err_tol=1e-6, rel_err_tol=1e-6,
-                     method='fd', step=1e-6, form='forward', step_calc='abs'):
+                     method='fd', step=None, form='forward', step_calc='abs'):
         """
         Check total derivatives for the model vs. finite difference.
 
@@ -1260,7 +1292,8 @@ class Problem(object):
         method : str
             Method, 'fd' for finite difference or 'cs' for complex step. Default is 'fd'
         step : float
-            Step size for approximation. Default is 1e-6.
+            Step size for approximation. Default is None, which means 1e-6 for 'fd' and 1e-40 for
+            'cs'.
         form : string
             Form for finite difference, can be 'forward', 'backward', or 'central'. Default
             'forward'.
@@ -1281,7 +1314,11 @@ class Problem(object):
                 forward - fd, adjoint - fd, forward - adjoint.
         """
         model = self.model
-        global_names = False
+
+        if method == 'cs' and not model._outputs._alloc_complex:
+            msg = "\nTo enable complex step, specify 'force_alloc_complex=True' when calling " + \
+                  "setup on the problem, e.g. 'problem.setup(force_alloc_complex=True)'"
+            raise RuntimeError(msg)
 
         # TODO: Once we're tracking iteration counts, run the model if it has not been run before.
 
@@ -1292,6 +1329,12 @@ class Problem(object):
                                        driver_scaling=driver_scaling)
             Jcalc = total_info.compute_totals()
 
+            if step is None:
+                if method == 'cs':
+                    step = DEFAULT_CS_OPTIONS['step']
+                else:
+                    step = DEFAULT_FD_OPTIONS['step']
+
             # Approximate FD
             fd_args = {
                 'step': step,
@@ -1317,7 +1360,7 @@ class Problem(object):
         data[''] = {}
         for key, val in iteritems(Jcalc):
             data[''][key] = {}
-            data[''][key]['J_fwd'] = Jcalc[key]
+            data[''][key]['J_fwd'] = val
             data[''][key]['J_fd'] = Jfd[key]
         fd_args['method'] = 'fd'
 
@@ -1562,21 +1605,21 @@ class Problem(object):
         case : Case object
             A Case from a CaseRecorder file.
         """
-        inputs = case.inputs._values if case.inputs is not None else None
+        inputs = case.inputs if case.inputs is not None else None
         if inputs:
-            for name, val in zip(inputs.dtype.names, inputs):
+            for name in inputs.absolute_names():
                 if name not in self.model._var_abs_names['input']:
                     raise KeyError("Input variable, '{}', recorded in the case is not "
                                    "found in the model".format(name))
-                self[name] = val
+                self[name] = inputs[name]
 
-        outputs = case.outputs._values if case.outputs is not None else None
+        outputs = case.outputs if case.outputs is not None else None
         if outputs:
-            for name, val in zip(outputs.dtype.names, outputs):
+            for name in outputs.absolute_names():
                 if name not in self.model._var_abs_names['output']:
                     raise KeyError("Output variable, '{}', recorded in the case is not "
                                    "found in the model".format(name))
-                self[name] = val
+                self[name] = outputs[name]
 
         return
 
@@ -1651,7 +1694,7 @@ def _assemble_derivative_data(derivative_data, rel_error_tol, abs_error_tol, out
 
         if sys_name not in derivative_data:
             msg = "No derivative data found for %s '%s'." % (sys_type, sys_name)
-            warnings.warn(msg)
+            simple_warning(msg)
             continue
 
         derivatives = derivative_data[sys_name]
@@ -1853,16 +1896,16 @@ def _assemble_derivative_data(derivative_data, rel_error_tol, abs_error_tol, out
 
                     # Magnitudes
                     if out_stream:
-                        out_buffer.write("  {}: '{}' wrt '{}'".format(sys_name, of, wrt) + '\n')
-                        out_buffer.write('    Forward Magnitude : {:.6e}'.format(magnitude.forward)
-                                         + '\n')
+                        out_buffer.write("  {}: '{}' wrt '{}'\n".format(sys_name, of, wrt))
+                        out_buffer.write('    Forward Magnitude : {:.6e}\n'.format(
+                            magnitude.forward))
                     if not totals and system.matrix_free:
                         txt = '    Reverse Magnitude : {:.6e}'
                         if out_stream:
                             out_buffer.write(txt.format(magnitude.reverse) + '\n')
                     if out_stream:
-                        out_buffer.write('         Fd Magnitude : {:.6e} ({})'.format(magnitude.fd,
-                                         fd_desc) + '\n')
+                        out_buffer.write('         Fd Magnitude : {:.6e} ({})\n'.format(
+                            magnitude.fd, fd_desc))
                     # Absolute Errors
                     if totals or not system.matrix_free:
                         error_descs = ('(Jfor  - Jfd) ', )
@@ -1873,8 +1916,7 @@ def _assemble_derivative_data(derivative_data, rel_error_tol, abs_error_tol, out
                         if error_str.endswith('*'):
                             num_bad_jacs += 1
                         if out_stream:
-                            out_buffer.write('    Absolute Error {}: {}'.format(desc, error_str) +
-                                             '\n')
+                            out_buffer.write('    Absolute Error {}: {}\n'.format(desc, error_str))
                     if out_stream:
                         out_buffer.write('\n')
 
@@ -1884,10 +1926,11 @@ def _assemble_derivative_data(derivative_data, rel_error_tol, abs_error_tol, out
                         if error_str.endswith('*'):
                             num_bad_jacs += 1
                         if out_stream:
-                            out_buffer.write('    Relative Error {}: {}'.format(desc, error_str) +
-                                             '\n')
+                            out_buffer.write('    Relative Error {}: {}\n'.format(desc, error_str))
 
                     if out_stream:
+                        if MPI and MPI.COMM_WORLD.size > 1:
+                            out_buffer.write('    MPI Rank {}\n'.format(MPI.COMM_WORLD.rank))
                         out_buffer.write('\n')
 
                     # Raw Derivatives
