@@ -10,12 +10,13 @@ import numpy as np
 
 from openmdao.api import Problem, Group, ExplicitComponent, ImplicitComponent, \
     IndepVarComp, ExecComp, NonlinearRunOnce, NonlinearBlockGS, ScipyKrylov, NewtonSolver, \
-    DirectSolver
+    DirectSolver, LinearBlockGS, BroydenSolver
 from openmdao.core.tests.test_impl_comp import QuadraticLinearize, QuadraticJacVec
 from openmdao.core.tests.test_matmat import MultiJacVec
 from openmdao.test_suite.components.impl_comp_array import TestImplCompArrayMatVec
 from openmdao.test_suite.components.paraboloid_mat_vec import ParaboloidMatVec
-from openmdao.test_suite.components.sellar import SellarDerivatives
+from openmdao.test_suite.components.sellar import SellarDerivatives, SellarDis1withDerivatives, \
+     SellarDis2withDerivatives
 from openmdao.test_suite.components.simple_comps import DoubleArrayComp
 from openmdao.test_suite.groups.parallel_groups import FanInSubbedIDVC
 from openmdao.utils.assert_utils import assert_rel_error
@@ -239,6 +240,8 @@ class TestProblemCheckPartials(unittest.TestCase):
 
                 self.declare_partials(of='*', wrt='*')
 
+                self.lin_count = 0
+
             def compute(self, inputs, outputs):
                 outputs['y'] = 3.0*inputs['x1'] + 4.0*inputs['x2']
 
@@ -246,6 +249,7 @@ class TestProblemCheckPartials(unittest.TestCase):
                 """Intentionally left out derivative."""
                 J = partials
                 J['y', 'x1'] = np.array([3.0])
+                self.lin_count += 1
 
         prob = Problem()
         prob.model = Group()
@@ -263,6 +267,8 @@ class TestProblemCheckPartials(unittest.TestCase):
 
         data = prob.check_partials(out_stream=None)
 
+        self.assertEqual(prob.model.comp.lin_count, 1)
+
         abs_error = data['comp']['y', 'x1']['abs error']
         rel_error = data['comp']['y', 'x1']['rel error']
         self.assertAlmostEqual(abs_error.forward, 0.)
@@ -633,7 +639,7 @@ class TestProblemCheckPartials(unittest.TestCase):
         prob.setup(check=False)
         prob.run_model()
 
-        data = prob.check_partials(out_stream=None)
+        data = prob.check_partials(out_stream=None, compact_print=True)
 
         # This will fail unless you set the check_step.
         x_error = data['comp']['f_xy', 'x']['rel error']
@@ -687,8 +693,9 @@ class TestProblemCheckPartials(unittest.TestCase):
         self.assertEqual(len(w), 1)
 
         msg = "The following components requested complex step, but force_alloc_complex " + \
-            "has not been set to True, so finite difference was used: ['comp']"
-
+              "has not been set to True, so finite difference was used: ['comp']\n" + \
+              "To enable complex step, specify 'force_alloc_complex=True' when calling " + \
+              "setup on the problem, e.g. 'problem.setup(force_alloc_complex=True)'"
         self.assertEqual(str(w[0].message), msg)
 
         # Derivative still calculated, but with fd instead.
@@ -714,7 +721,7 @@ class TestProblemCheckPartials(unittest.TestCase):
         prob.setup(check=False, force_alloc_complex=True)
         prob.run_model()
 
-        data = prob.check_partials(out_stream=None)
+        data = prob.check_partials(out_stream=None, compact_print=True)
 
         x_error = data['comp']['f_xy', 'x']['rel error']
         self.assertLess(x_error.forward, 1e-5)
@@ -760,7 +767,7 @@ class TestProblemCheckPartials(unittest.TestCase):
         prob.setup(check=False)
         prob.run_model()
 
-        data = prob.check_partials(out_stream=None)
+        data = prob.check_partials(out_stream=None, compact_print=True)
 
         # This will fail unless you set the check_step.
         x_error = data['comp']['f_xy', 'x']['rel error']
@@ -808,7 +815,7 @@ class TestProblemCheckPartials(unittest.TestCase):
         prob.setup(check=False)
         prob.run_model()
 
-        data = prob.check_partials(out_stream=None)
+        data = prob.check_partials(out_stream=None, compact_print=True)
 
         # This will fail unless you set the check_step.
         x_error = data['comp']['f_xy', 'x']['rel error']
@@ -965,9 +972,13 @@ class TestProblemCheckPartials(unittest.TestCase):
         with self.assertRaises(ValueError) as cm:
             prob.check_partials()
 
-        self.assertEqual(str(cm.exception),
-                         "'foo' is not a valid form of finite difference; "
-                         "must be one of ['forward', 'backward', 'central']")
+        # The form options sometimes print out in different order.
+        msg = str(cm.exception)
+        self.assertTrue("'foo' is not a valid form of finite difference; "
+                         "must be one of [" in msg, 'error message not correct.')
+        self.assertTrue('forward' in msg, 'error message not correct.')
+        self.assertTrue('backward' in msg, 'error message not correct.')
+        self.assertTrue('central' in msg, 'error message not correct.')
 
         # check invalid step
         with self.assertRaises(ValueError) as cm:
@@ -1449,7 +1460,7 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.setup(check=False)
         prob.run_model()
 
-        data = prob.check_partials(out_stream=None)
+        data = prob.check_partials(out_stream=None, compact_print=True)
         print(data)
 
     def test_set_step_on_comp(self):
@@ -1476,7 +1487,7 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.setup()
         prob.run_model()
 
-        prob.check_partials()
+        prob.check_partials(compact_print=True)
 
     def test_set_step_global(self):
         from openmdao.api import Problem, Group, IndepVarComp
@@ -1500,7 +1511,7 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.setup()
         prob.run_model()
 
-        prob.check_partials(step=1e-2)
+        prob.check_partials(step=1e-2, compact_print=True)
 
     def test_set_method_on_comp(self):
         from openmdao.api import Problem, Group, IndepVarComp
@@ -1526,7 +1537,7 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.setup(force_alloc_complex=True)
         prob.run_model()
 
-        prob.check_partials()
+        prob.check_partials(compact_print=True)
 
     def test_set_method_global(self):
         from openmdao.api import Problem, Group, IndepVarComp
@@ -1550,33 +1561,9 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.setup(force_alloc_complex=True)
         prob.run_model()
 
-        prob.check_partials(method='cs')
+        prob.check_partials(method='cs', compact_print=True)
 
-    def test_set_form_on_comp(self):
-        from openmdao.api import Problem, Group, IndepVarComp
-        from openmdao.core.tests.test_check_derivs import ParaboloidTricky
-        from openmdao.test_suite.components.paraboloid_mat_vec import ParaboloidMatVec
 
-        prob = Problem()
-        prob.model = Group()
-
-        prob.model.add_subsystem('p1', IndepVarComp('x', 3.0))
-        prob.model.add_subsystem('p2', IndepVarComp('y', 5.0))
-        comp = prob.model.add_subsystem('comp', ParaboloidTricky())
-        prob.model.add_subsystem('comp2', ParaboloidMatVec())
-
-        prob.model.connect('p1.x', 'comp.x')
-        prob.model.connect('p2.y', 'comp.y')
-        prob.model.connect('comp.f_xy', 'comp2.x')
-
-        prob.set_solver_print(level=0)
-
-        comp.set_check_partial_options(wrt='*', form='central')
-
-        prob.setup()
-        prob.run_model()
-
-        prob.check_partials()
 
     def test_set_form_global(self):
         from openmdao.api import Problem, Group, IndepVarComp
@@ -1600,33 +1587,8 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.setup()
         prob.run_model()
 
-        prob.check_partials(form='central')
+        prob.check_partials(form='central', compact_print=True)
 
-    def test_set_step_calc_on_comp(self):
-        from openmdao.api import Problem, Group, IndepVarComp
-        from openmdao.core.tests.test_check_derivs import ParaboloidTricky
-        from openmdao.test_suite.components.paraboloid_mat_vec import ParaboloidMatVec
-
-        prob = Problem()
-        prob.model = Group()
-
-        prob.model.add_subsystem('p1', IndepVarComp('x', 3.0))
-        prob.model.add_subsystem('p2', IndepVarComp('y', 5.0))
-        comp = prob.model.add_subsystem('comp', ParaboloidTricky())
-        prob.model.add_subsystem('comp2', ParaboloidMatVec())
-
-        prob.model.connect('p1.x', 'comp.x')
-        prob.model.connect('p2.y', 'comp.y')
-        prob.model.connect('comp.f_xy', 'comp2.x')
-
-        prob.set_solver_print(level=0)
-
-        comp.set_check_partial_options(wrt='*', step_calc='rel')
-
-        prob.setup()
-        prob.run_model()
-
-        prob.check_partials()
 
     def test_set_step_calc_global(self):
         from openmdao.api import Problem, Group, IndepVarComp
@@ -1647,33 +1609,7 @@ class TestCheckPartialsFeature(unittest.TestCase):
         prob.setup()
         prob.run_model()
 
-        prob.check_partials(step_calc='rel')
-
-    def test_feature_compact_print_formatting(self):
-        from openmdao.api import Problem, Group, IndepVarComp
-        from openmdao.core.tests.test_check_derivs import ParaboloidTricky
-        from openmdao.test_suite.components.paraboloid_mat_vec import ParaboloidMatVec
-
-        prob = Problem()
-        prob.model = Group()
-
-        prob.model.add_subsystem('p1', IndepVarComp('x', 3.0))
-        prob.model.add_subsystem('p2', IndepVarComp('y', 5.0))
-        comp = prob.model.add_subsystem('comp', ParaboloidTricky())
-        prob.model.add_subsystem('comp2', ParaboloidMatVec())
-
-        prob.model.connect('p1.x', 'comp.x')
-        prob.model.connect('p2.y', 'comp.y')
-        prob.model.connect('comp.f_xy', 'comp2.x')
-
-        prob.set_solver_print(level=0)
-
-        comp.set_check_partial_options(wrt='*', step_calc='rel')
-
-        prob.setup()
-        prob.run_model()
-
-        prob.check_partials(compact_print=True)
+        prob.check_partials(step_calc='rel', compact_print=True)
 
     def test_feature_check_partials_show_only_incorrect(self):
         from openmdao.api import Problem, Group, IndepVarComp, ExplicitComponent
@@ -1774,13 +1710,16 @@ class TestProblemCheckTotals(unittest.TestCase):
 
         prob.setup(force_alloc_complex=True)
 
+        prob.model.nonlinear_solver.options['atol'] = 1e-15
+        prob.model.nonlinear_solver.options['rtol'] = 1e-15
+
         # We don't call run_driver() here because we don't
         # actually want the optimizer to run
         prob.run_model()
 
         # check derivatives with complex step and a larger step size.
         stream = cStringIO()
-        totals = prob.check_totals(method='cs', step=1.0e-1, out_stream=stream)
+        totals = prob.check_totals(method='cs', out_stream=stream)
 
         lines = stream.getvalue().splitlines()
 
@@ -1808,7 +1747,7 @@ class TestProblemCheckTotals(unittest.TestCase):
 
         # check derivatives with complex step and a larger step size.
         stream = cStringIO()
-        totals = prob.check_totals(method='cs', step=1.0e-1, out_stream=stream)
+        totals = prob.check_totals(method='cs', out_stream=stream)
 
         lines = stream.getvalue().splitlines()
 
@@ -1934,7 +1873,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         # check derivatives with complex step and a larger step size.
-        totals = prob.check_totals(method='cs', step=1.0e-1, out_stream=None)
+        totals = prob.check_totals(method='cs', out_stream=None)
 
         data = totals['con_cmp2.con2', 'px.x']
         self.assertTrue('J_fwd' in data)
@@ -2126,7 +2065,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         # Try again with a direct solver and sparse assembled hierarchy.
 
         p = Problem()
-        sub = p.model.add_subsystem('sub', GaussLobattoPhase())
+        p.model.add_subsystem('sub', GaussLobattoPhase())
 
         p.model.sub.add_objective('time', index=-1)
 
@@ -2193,6 +2132,105 @@ class TestProblemCheckTotals(unittest.TestCase):
         cderiv = prob.check_totals(out_stream=None)
         assert_rel_error(self, cderiv['comp.y1', 'px.x']['J_fwd'], J, 1.0e-3)
 
+    def test_cs_around_newton(self):
+        # Basic sellar test.
+
+        prob = Problem()
+        model = prob.model
+        sub = model.add_subsystem('sub', Group(), promotes=['*'])
+
+        model.add_subsystem('px', IndepVarComp('x', 1.0), promotes=['x'])
+        model.add_subsystem('pz', IndepVarComp('z', np.array([5.0, 2.0])), promotes=['z'])
+
+        sub.add_subsystem('d1', SellarDis1withDerivatives(), promotes=['x', 'z', 'y1', 'y2'])
+        sub.add_subsystem('d2', SellarDis2withDerivatives(), promotes=['z', 'y1', 'y2'])
+
+        model.add_subsystem('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
+                                                z=np.array([0.0, 0.0]), x=0.0),
+                            promotes=['obj', 'x', 'z', 'y1', 'y2'])
+
+        model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
+        model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
+
+        sub.nonlinear_solver = NewtonSolver()
+        sub.linear_solver = DirectSolver()
+
+        # Need this.
+        model.linear_solver = LinearBlockGS()
+
+        prob.model.add_design_var('x', lower=-100, upper=100)
+        prob.model.add_design_var('z', lower=-100, upper=100)
+        prob.model.add_objective('obj')
+        prob.model.add_constraint('con1', upper=0.0)
+        prob.model.add_constraint('con2', upper=0.0)
+
+        prob.setup(check=False, force_alloc_complex=True)
+        prob.set_solver_print(level=0)
+
+        prob.run_model()
+
+        totals = prob.check_totals(method='cs', out_stream=None)
+
+        for key, val in iteritems(totals):
+            assert_rel_error(self, val['rel error'][0], 0.0, 1e-10)
+
+    def test_cs_around_broyden(self):
+        # Basic sellar test.
+
+        prob = Problem()
+        model = prob.model
+        sub = model.add_subsystem('sub', Group(), promotes=['*'])
+
+        model.add_subsystem('px', IndepVarComp('x', 1.0), promotes=['x'])
+        model.add_subsystem('pz', IndepVarComp('z', np.array([5.0, 2.0])), promotes=['z'])
+
+        sub.add_subsystem('d1', SellarDis1withDerivatives(), promotes=['x', 'z', 'y1', 'y2'])
+        sub.add_subsystem('d2', SellarDis2withDerivatives(), promotes=['z', 'y1', 'y2'])
+
+        model.add_subsystem('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
+                                                z=np.array([0.0, 0.0]), x=0.0),
+                            promotes=['obj', 'x', 'z', 'y1', 'y2'])
+
+        model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
+        model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
+
+        sub.nonlinear_solver = BroydenSolver()
+        sub.linear_solver = DirectSolver()
+
+        # Need this.
+        model.linear_solver = LinearBlockGS()
+
+        prob.model.add_design_var('x', lower=-100, upper=100)
+        prob.model.add_design_var('z', lower=-100, upper=100)
+        prob.model.add_objective('obj')
+        prob.model.add_constraint('con1', upper=0.0)
+        prob.model.add_constraint('con2', upper=0.0)
+
+        prob.setup(check=False, force_alloc_complex=True)
+        prob.set_solver_print(level=0)
+
+        prob.run_model()
+
+        totals = prob.check_totals(method='cs', out_stream=None)
+
+        for key, val in iteritems(totals):
+            assert_rel_error(self, val['rel error'][0], 0.0, 1e-6)
+
+    def test_cs_error_allocate(self):
+        prob = Problem()
+        model = prob.model
+        model.add_subsystem('p', IndepVarComp('x', 3.0), promotes=['*'])
+        model.add_subsystem('comp', ParaboloidTricky(), promotes=['*'])
+        prob.setup(check=False)
+        prob.run_model()
+
+        with self.assertRaises(RuntimeError) as cm:
+            prob.check_totals(method='cs')
+
+        msg = "\nTo enable complex step, specify 'force_alloc_complex=True' when calling " + \
+                "setup on the problem, e.g. 'problem.setup(force_alloc_complex=True)'"
+        self.assertEqual(str(cm.exception), msg)
+
 
 @unittest.skipUnless(MPI and PETScVector, "only run under MPI with PETSc.")
 class TestProblemCheckTotalsMPI(unittest.TestCase):
