@@ -3,19 +3,35 @@ from __future__ import print_function
 import os
 import shutil
 import tempfile
+import warnings
 
 import unittest
 import numpy as np
 import math
 
+from distutils.version import LooseVersion
 from numpy.testing import assert_array_almost_equal, assert_almost_equal
+import scipy
+try:
+    from scipy.sparse import load_npz
+except ImportError:
+    load_npz = None
 
 from openmdao.api import Problem, IndepVarComp, ExecComp, DirectSolver,\
-    ExplicitComponent, LinearRunOnce, ScipyOptimizeDriver
+    ExplicitComponent, LinearRunOnce, ScipyOptimizeDriver, ParallelGroup, Group
 from openmdao.utils.assert_utils import assert_rel_error
 
 from openmdao.utils.general_utils import set_pyoptsparse_opt
 from openmdao.utils.coloring import get_simul_meta, _solves_info
+from openmdao.utils.mpi import MPI
+from openmdao.test_suite.tot_jac_builder import TotJacBuilder
+import openmdao.test_suite
+
+try:
+    from openmdao.vectors.petsc_vector import PETScVector
+except ImportError:
+    PETScVector = None
+
 
 # check that pyoptsparse is installed
 OPT, OPTIMIZER = set_pyoptsparse_opt('SNOPT')
@@ -23,23 +39,28 @@ if OPTIMIZER:
     from openmdao.drivers.pyoptsparse_driver import pyOptSparseDriver
 
 
-class RunOnceCounter(LinearRunOnce):
+class CounterGroup(Group):
     def __init__(self, *args, **kwargs):
         self._solve_count = 0
-        super(RunOnceCounter, self).__init__(*args, **kwargs)
+        super(CounterGroup, self).__init__(*args, **kwargs)
 
-    def _iter_execute(self):
-        super(RunOnceCounter, self)._iter_execute()
+    def _solve_linear(self, *args, **kwargs):
+        super(CounterGroup, self)._solve_linear(*args, **kwargs)
         self._solve_count += 1
 
+
 # note: size must be an even number
 SIZE = 10
 
-def run_opt(driver_class, mode, color_info=None, sparsity=None, **options):
+def run_opt(driver_class, mode, assemble_type=None, color_info=None, sparsity=None, derivs=True,
+           **options):
 
-    p = Problem()
+    p = Problem(model=CounterGroup())
+
+    if assemble_type is not None:
+        p.model.linear_solver = DirectSolver(assemble_jac=True)
+        p.model.options['assembled_jac_type'] = assemble_type
 
-    p.model.linear_solver = RunOnceCounter()
     indeps = p.model.add_subsystem('indeps', IndepVarComp(), promotes_outputs=['*'])
 
     # the following were randomly generated using np.random.random(10)*2-1 to randomly
@@ -50,29 +71,34 @@ def run_opt(driver_class, mode, color_info=None, sparsity=None, **options):
                                      -0.86236787, -0.97500023,  0.47739414,  0.51174103,  0.10052582]))
     indeps.add_output('r', .7)
 
+    p.model.add_subsystem('arctan_yox', ExecComp('g=arctan(y/x)', vectorize=True,
+                                                 g=np.ones(SIZE), x=np.ones(SIZE), y=np.ones(SIZE)))
+
     p.model.add_subsystem('circle', ExecComp('area=pi*r**2'))
 
-    p.model.add_subsystem('r_con', ExecComp('g=x**2 + y**2 - r',
+    p.model.add_subsystem('r_con', ExecComp('g=x**2 + y**2 - r', vectorize=True,
                                             g=np.ones(SIZE), x=np.ones(SIZE), y=np.ones(SIZE)))
 
     thetas = np.linspace(0, np.pi/4, SIZE)
-    p.model.add_subsystem('theta_con', ExecComp('g=arctan(y/x) - theta',
-                                                g=np.ones(SIZE), x=np.ones(SIZE), y=np.ones(SIZE),
+    p.model.add_subsystem('theta_con', ExecComp('g = x - theta', vectorize=True,
+                                                g=np.ones(SIZE), x=np.ones(SIZE),
                                                 theta=thetas))
-    p.model.add_subsystem('delta_theta_con', ExecComp('g = arctan(y/x)[::2]-arctan(y/x)[1::2]',
-                                                      g=np.ones(SIZE//2), x=np.ones(SIZE),
-                                                      y=np.ones(SIZE)))
+    p.model.add_subsystem('delta_theta_con', ExecComp('g = even - odd', vectorize=True,
+                                                      g=np.ones(SIZE//2), even=np.ones(SIZE//2),
+                                                      odd=np.ones(SIZE//2)))
 
-    thetas = np.linspace(0, np.pi/4, SIZE)
+    p.model.add_subsystem('l_conx', ExecComp('g=x-1', vectorize=True, g=np.ones(SIZE), x=np.ones(SIZE)))
 
-    p.model.add_subsystem('l_conx', ExecComp('g=x-1', g=np.ones(SIZE), x=np.ones(SIZE)))
+    IND = np.arange(SIZE, dtype=int)
+    ODD_IND = IND[1::2]  # all odd indices
+    EVEN_IND = IND[0::2]  # all even indices
 
     p.model.connect('r', ('circle.r', 'r_con.r'))
-    p.model.connect('x', ['r_con.x', 'theta_con.x', 'delta_theta_con.x'])
-
-    p.model.connect('x', 'l_conx.x')
-
-    p.model.connect('y', ['r_con.y', 'theta_con.y', 'delta_theta_con.y'])
+    p.model.connect('x', ['r_con.x', 'arctan_yox.x', 'l_conx.x'])
+    p.model.connect('y', ['r_con.y', 'arctan_yox.y'])
+    p.model.connect('arctan_yox.g', 'theta_con.x')
+    p.model.connect('arctan_yox.g', 'delta_theta_con.even', src_indices=EVEN_IND)
+    p.model.connect('arctan_yox.g', 'delta_theta_con.odd', src_indices=ODD_IND)
 
     p.driver = driver_class()
     p.driver.options.update(options)
@@ -84,9 +110,7 @@ def run_opt(driver_class, mode, color_info=None, sparsity=None, **options):
     # nonlinear constraints
     p.model.add_constraint('r_con.g', equals=0)
 
-    IND = np.arange(SIZE, dtype=int)
-    ODD_IND = IND[0::2]  # all odd indices
-    p.model.add_constraint('theta_con.g', lower=-1e-5, upper=1e-5, indices=ODD_IND)
+    p.model.add_constraint('theta_con.g', lower=-1e-5, upper=1e-5, indices=EVEN_IND)
     p.model.add_constraint('delta_theta_con.g', lower=-1e-5, upper=1e-5)
 
     # this constrains x[0] to be 1 (see definition of l_conx)
@@ -103,16 +127,16 @@ def run_opt(driver_class, mode, color_info=None, sparsity=None, **options):
     elif sparsity is not None:
         p.driver.set_total_jac_sparsity(sparsity)
 
-    p.setup(mode=mode)
+    p.setup(mode=mode, derivatives=derivs)
     p.run_driver()
 
     return p
 
 
-class SimulColoringTestCase(unittest.TestCase):
+class SimulColoringPyoptSparseTestCase(unittest.TestCase):
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
-    def test_simul_coloring_snopt(self):
+    def test_simul_coloring_snopt_fwd(self):
         # first, run w/o coloring
         p = run_opt(pyOptSparseDriver, 'fwd', optimizer='SNOPT', print_results=False)
 
@@ -173,7 +197,7 @@ class SimulColoringTestCase(unittest.TestCase):
                "indeps.r": [[], [], [5, 1]]
             }
         }}
-        p_color = run_opt(pyOptSparseDriver, 'fwd', color_info, optimizer='SNOPT', print_results=False)
+        p_color = run_opt(pyOptSparseDriver, 'fwd', color_info=color_info, optimizer='SNOPT', print_results=False)
 
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
@@ -181,11 +205,11 @@ class SimulColoringTestCase(unittest.TestCase):
         # - coloring saves 16 solves per driver iter  (5 vs 21)
         # - initial solve for linear constraints takes 21 in both cases (only done once)
         # - (total_solves - 21) / (solves_per_iter) should be equal between the two cases
-        self.assertEqual((p.model.linear_solver._solve_count - 21) / 21,
-                         (p_color.model.linear_solver._solve_count - 21) / 5)
+        self.assertEqual((p.model._solve_count - 21) / 21,
+                         (p_color.model._solve_count - 21) / 5)
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
-    def test_dynamic_simul_coloring_snopt(self):
+    def test_dynamic_simul_coloring_snopt_auto(self):
         # first, run w/o coloring
         p = run_opt(pyOptSparseDriver, 'auto', optimizer='SNOPT', print_results=False)
         p_color = run_opt(pyOptSparseDriver, 'auto', optimizer='SNOPT', print_results=False,
@@ -199,10 +223,28 @@ class SimulColoringTestCase(unittest.TestCase):
         # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
         # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
         # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model.linear_solver._solve_count - 21) / 21,
-                         (p_color.model.linear_solver._solve_count - 21 * 4) / 5)
+        self.assertEqual((p.model._solve_count - 21) / 21,
+                         (p_color.model._solve_count - 21 * 4) / 5)
 
-    def test_simul_coloring_pyoptsparse_slsqp(self):
+    @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
+    def test_dynamic_simul_coloring_snopt_auto_assembled(self):
+        # first, run w/o coloring
+        p = run_opt(pyOptSparseDriver, 'auto', assemble_type='dense', optimizer='SNOPT', print_results=False)
+        p_color = run_opt(pyOptSparseDriver, 'auto', assemble_type='dense', optimizer='SNOPT', print_results=False,
+                          dynamic_simul_derivs=True)
+
+        assert_almost_equal(p['circle.area'], np.pi, decimal=7)
+        assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
+
+        # - coloring saves 16 solves per driver iter  (5 vs 21)
+        # - initial solve for linear constraints takes 21 in both cases (only done once)
+        # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
+        # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
+        # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
+        self.assertEqual((p.model._solve_count - 21) / 21,
+                         (p_color.model._solve_count - 21 * 4) / 5)
+
+    def test_simul_coloring_pyoptsparse_slsqp_fwd(self):
         try:
             from pyoptsparse import OPT
         except ImportError:
@@ -271,7 +313,7 @@ class SimulColoringTestCase(unittest.TestCase):
             }
         }}
 
-        p_color = run_opt(pyOptSparseDriver, 'fwd', color_info, optimizer='SLSQP', print_results=False)
+        p_color = run_opt(pyOptSparseDriver, 'fwd', color_info=color_info, optimizer='SLSQP', print_results=False)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
         # run w/o coloring
@@ -281,10 +323,10 @@ class SimulColoringTestCase(unittest.TestCase):
         # - coloring saves 16 solves per driver iter  (5 vs 21)
         # - initial solve for linear constraints takes 21 in both cases (only done once)
         # - (total_solves - 21) / (solves_per_iter) should be equal between the two cases
-        self.assertEqual((p.model.linear_solver._solve_count - 21) / 21,
-                         (p_color.model.linear_solver._solve_count - 21) / 5)
+        self.assertEqual((p.model._solve_count - 21) / 21,
+                         (p_color.model._solve_count - 21) / 5)
 
-    def test_dynamic_simul_coloring_pyoptsparse_slsqp(self):
+    def test_dynamic_simul_coloring_pyoptsparse_slsqp_auto(self):
         try:
             from pyoptsparse import OPT
         except ImportError:
@@ -308,11 +350,11 @@ class SimulColoringTestCase(unittest.TestCase):
         # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
         # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
         # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model.linear_solver._solve_count - 21) / 21,
-                         (p_color.model.linear_solver._solve_count - 21 * 4) / 5)
+        self.assertEqual((p.model._solve_count - 21) / 21,
+                         (p_color.model._solve_count - 21 * 4) / 5)
 
 
-class SimulColoringRevTestCase(unittest.TestCase):
+class SimulColoringPyoptSparseRevTestCase(unittest.TestCase):
     """Reverse coloring tests for pyoptsparse."""
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
@@ -321,36 +363,36 @@ class SimulColoringRevTestCase(unittest.TestCase):
         p = run_opt(pyOptSparseDriver, 'rev', optimizer='SNOPT', print_results=False)
 
         color_info = {"rev": [[
-           [4, 5, 6, 7, 8, 9, 10],   # uncolored rows
-           [2, 21],   # color 1
-           [3, 16],   # color 2
-           [1, 17, 18, 19, 20],   # color 3
-           [0, 11, 12, 13, 14, 15]   # color 4
-        ],
-        [
-           [20],   # row 0
-           [0, 10, 20],   # row 1
-           [1, 11, 20],   # row 2
-           [2, 12, 20],   # row 3
-           None,   # row 4
-           None,   # row 5
-           None,   # row 6
-           None,   # row 7
-           None,   # row 8
-           None,   # row 9
-           None,   # row 10
-           [0, 10],   # row 11
-           [2, 12],   # row 12
-           [4, 14],   # row 13
-           [6, 16],   # row 14
-           [8, 18],   # row 15
-           [0, 1, 10, 11],   # row 16
-           [2, 3, 12, 13],   # row 17
-           [4, 5, 14, 15],   # row 18
-           [6, 7, 16, 17],   # row 19
-           [8, 9, 18, 19],   # row 20
-           [0]   # row 21
-        ]],
+            [4, 5, 6, 7, 8, 9, 10],   # uncolored rows
+            [1, 18, 19, 20, 21],   # color 1
+            [0, 17, 13, 14, 15, 16],   # color 2
+            [2, 11],   # color 3
+            [3, 12]   # color 4
+            ],
+            [
+            [20],   # row 0
+            [0, 10, 20],   # row 1
+            [1, 11, 20],   # row 2
+            [2, 12, 20],   # row 3
+            None,   # row 4
+            None,   # row 5
+            None,   # row 6
+            None,   # row 7
+            None,   # row 8
+            None,   # row 9
+            None,   # row 10
+            [0],   # row 11
+            [0, 10],   # row 12
+            [2, 12],   # row 13
+            [4, 14],   # row 14
+            [6, 16],   # row 15
+            [8, 18],   # row 16
+            [0, 1, 10, 11],   # row 17
+            [2, 3, 12, 13],   # row 18
+            [4, 5, 14, 15],   # row 19
+            [6, 7, 16, 17],   # row 20
+            [8, 9, 18, 19]   # row 21
+            ]],
         "sparsity": {
             "circle.area": {
                "indeps.x": [[], [], [1, 10]],
@@ -378,7 +420,7 @@ class SimulColoringRevTestCase(unittest.TestCase):
                "indeps.r": [[], [], [1, 1]]
             }
         }}
-        p_color = run_opt(pyOptSparseDriver, 'rev', color_info, optimizer='SNOPT', print_results=False)
+        p_color = run_opt(pyOptSparseDriver, 'rev', color_info=color_info, optimizer='SNOPT', print_results=False)
 
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
@@ -386,18 +428,11 @@ class SimulColoringRevTestCase(unittest.TestCase):
         # - coloring saves 11 solves per driver iter  (11 vs 22)
         # - initial solve for linear constraints takes 1 in both cases (only done once)
         # - (total_solves - 1) / (solves_per_iter) should be equal between the two cases
-
-        # for some reason in rev mode the formula above does not hold true.  But, the number of solves
-        # for the colored case is even more than 50% better, so we'll just test for that to make sure
-        # things are working...
-        #self.assertEqual((p.model.linear_solver._solve_count - 1) / 22,
-                         #(p_color.model.linear_solver._solve_count - 1) / 11)
-
-        # verify non-colored performs more than twice as many solves
-        self.assertGreater((p.model.linear_solver._solve_count - 1) / (p_color.model.linear_solver._solve_count - 1), 2.0)
+        self.assertEqual((p.model._solve_count - 1) / 22,
+                         (p_color.model._solve_count - 1) / 11)
 
     @unittest.skipUnless(OPTIMIZER == 'SNOPT', "This test requires SNOPT.")
-    def test_dynamic_simul_coloring_snopt(self):
+    def test_dynamic_rev_simul_coloring_snopt(self):
         # first, run w/o coloring
         p = run_opt(pyOptSparseDriver, 'rev', optimizer='SNOPT', print_results=False)
         p_color = run_opt(pyOptSparseDriver, 'rev', optimizer='SNOPT', print_results=False,
@@ -406,13 +441,13 @@ class SimulColoringRevTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - bidirectional coloring saves 17 solves per driver iter  (5 vs 22)
+        # - bidirectional coloring saves 11 solves per driver iter  (11 vs 22)
         # - initial solve for linear constraints takes 1 in both cases (only done once)
         # - dynamic case does 3 full compute_totals to compute coloring, which adds 22 * 3 solves
         # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
         # - where N is 1 for the uncolored case and 22 * 3 + 1 for the dynamic colored case.
-        self.assertEqual((p.model.linear_solver._solve_count - 1) / 22,
-                         (p_color.model.linear_solver._solve_count - 1 - 22 * 3) / 5)
+        self.assertEqual((p.model._solve_count - 1) / 22,
+                         (p_color.model._solve_count - 1 - 22 * 3) / 11)
 
     def test_simul_coloring_pyoptsparse_slsqp(self):
         try:
@@ -483,7 +518,7 @@ class SimulColoringRevTestCase(unittest.TestCase):
             }
         }}
 
-        p_color = run_opt(pyOptSparseDriver, 'rev', color_info, optimizer='SLSQP', print_results=False)
+        p_color = run_opt(pyOptSparseDriver, 'rev', color_info=color_info, optimizer='SLSQP', print_results=False)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
         # run w/o coloring
@@ -493,10 +528,10 @@ class SimulColoringRevTestCase(unittest.TestCase):
         # - coloring saves 11 solves per driver iter  (11 vs 22)
         # - initial solve for linear constraints takes 1 in both cases (only done once)
         # - (total_solves - 1) / (solves_per_iter) should be equal between the two cases
-        self.assertEqual((p.model.linear_solver._solve_count - 1) / 22,
-                         (p_color.model.linear_solver._solve_count - 1) / 11)
+        self.assertEqual((p.model._solve_count - 1) / 22,
+                         (p_color.model._solve_count - 1) / 11)
 
-    def test_dynamic_simul_coloring_pyoptsparse_slsqp(self):
+    def test_dynamic_rev_simul_coloring_pyoptsparse_slsqp(self):
         try:
             from pyoptsparse import OPT
         except ImportError:
@@ -515,23 +550,19 @@ class SimulColoringRevTestCase(unittest.TestCase):
         p = run_opt(pyOptSparseDriver, 'rev', optimizer='SLSQP', print_results=False)
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
 
-        # - coloring saves 17 solves per driver iter  (5 vs 22)
+        # - coloring saves 11 solves per driver iter  (11 vs 22)
         # - initial solve for linear constraints takes 1 in both cases (only done once)
         # - dynamic case does 3 full compute_totals to compute coloring, which adds 22 * 3 solves
         # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
         # - where N is 1 for the uncolored case and 22 * 3 + 1 for the dynamic colored case.
-        self.assertEqual((p.model.linear_solver._solve_count - 1) / 22,
-                         (p_color.model.linear_solver._solve_count - 1 - 22 * 3) / 5)
+        self.assertEqual((p.model._solve_count - 1) / 22,
+                         (p_color.model._solve_count - 1 - 22 * 3) / 11)
 
 
 class SimulColoringScipyTestCase(unittest.TestCase):
 
-    def test_simul_coloring(self):
-
-        # first, run w/o coloring
-        p = run_opt(ScipyOptimizeDriver, 'fwd', optimizer='SLSQP', disp=False)
-
-        color_info = {"fwd": [[
+    def setUp(self):
+        self.color_info = {"fwd": [[
                [20],   # uncolored columns
                [0, 2, 4, 6, 8],   # color 1
                [1, 3, 5, 7, 9],   # color 2
@@ -564,7 +595,11 @@ class SimulColoringScipyTestCase(unittest.TestCase):
             "sparsity": None
         }
 
-        p_color = run_opt(ScipyOptimizeDriver, 'fwd', color_info, optimizer='SLSQP', disp=False)
+    def test_simul_coloring_fwd(self):
+
+        # first, run w/o coloring
+        p = run_opt(ScipyOptimizeDriver, 'fwd', optimizer='SLSQP', disp=False)
+        p_color = run_opt(ScipyOptimizeDriver, 'fwd', color_info=self.color_info, optimizer='SLSQP', disp=False)
 
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
@@ -572,10 +607,25 @@ class SimulColoringScipyTestCase(unittest.TestCase):
         # - coloring saves 16 solves per driver iter  (5 vs 21)
         # - initial solve for linear constraints takes 21 in both cases (only done once)
         # - (total_solves - 21) / (solves_per_iter) should be equal between the two cases
-        self.assertEqual((p.model.linear_solver._solve_count - 21) / 21,
-                         (p_color.model.linear_solver._solve_count - 21) / 5)
+        self.assertEqual((p.model._solve_count - 21) / 21,
+                         (p_color.model._solve_count - 21) / 5)
 
-    def test_dynamic_simul_coloring(self):
+        # check for proper handling if someone calls compute_totals on Problem with different set or different order
+        # of desvars/responses than were used to define the coloring.  Behavior should be that coloring is turned off
+        # and a warning is issued.
+        with warnings.catch_warnings(record=True) as w:
+            p_color.compute_totals(of=['delta_theta_con.g', 'circle.area', 'r_con.g', 'theta_con.g', 'l_conx.g'],
+                                   wrt=['x', 'y', 'r'])
+        self.assertEqual(len(w), 1)
+        self.assertEqual(str(w[0].message), "compute_totals called using a different list of design vars and/or responses than those used to define coloring, so coloring will be turned off.\ncoloring design vars: ['indeps.x', 'indeps.y', 'indeps.r'], current design vars: ['indeps.x', 'indeps.y', 'indeps.r']\ncoloring responses: ['circle.area', 'r_con.g', 'theta_con.g', 'delta_theta_con.g', 'l_conx.g'], current responses: ['delta_theta_con.g', 'circle.area', 'r_con.g', 'theta_con.g', 'l_conx.g'].")
+
+    def test_bad_mode(self):
+        with self.assertRaises(Exception) as context:
+            p_color = run_opt(ScipyOptimizeDriver, 'rev', color_info=self.color_info, optimizer='SLSQP', disp=False)
+        self.assertEqual(str(context.exception),
+                         "Simultaneous coloring does forward solves but mode has been set to 'rev'")
+
+    def test_dynamic_simul_coloring_auto(self):
 
         # first, run w/o coloring
         p = run_opt(ScipyOptimizeDriver, 'auto', optimizer='SLSQP', disp=False)
@@ -589,8 +639,8 @@ class SimulColoringScipyTestCase(unittest.TestCase):
         # - dynamic case does 3 full compute_totals to compute coloring, which adds 21 * 3 solves
         # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
         # - where N is 21 for the uncolored case and 21 * 4 for the dynamic colored case.
-        self.assertEqual((p.model.linear_solver._solve_count - 21) / 21,
-                         (p_color.model.linear_solver._solve_count - 21 * 4) / 5)
+        self.assertEqual((p.model._solve_count - 21) / 21,
+                         (p_color.model._solve_count - 21 * 4) / 5)
 
     def test_simul_coloring_example(self):
 
@@ -609,29 +659,34 @@ class SimulColoringScipyTestCase(unittest.TestCase):
                                           -0.86236787, -0.97500023,  0.47739414,  0.51174103,  0.10052582]))
         indeps.add_output('r', .7)
 
+        p.model.add_subsystem('arctan_yox', ExecComp('g=arctan(y/x)', vectorize=True,
+                                                    g=np.ones(SIZE), x=np.ones(SIZE), y=np.ones(SIZE)))
+
         p.model.add_subsystem('circle', ExecComp('area=pi*r**2'))
 
-        p.model.add_subsystem('r_con', ExecComp('g=x**2 + y**2 - r',
+        p.model.add_subsystem('r_con', ExecComp('g=x**2 + y**2 - r', vectorize=True,
                                                 g=np.ones(SIZE), x=np.ones(SIZE), y=np.ones(SIZE)))
 
         thetas = np.linspace(0, np.pi/4, SIZE)
-        p.model.add_subsystem('theta_con', ExecComp('g=arctan(y/x) - theta',
-                                                    g=np.ones(SIZE), x=np.ones(SIZE), y=np.ones(SIZE),
+        p.model.add_subsystem('theta_con', ExecComp('g = x - theta', vectorize=True,
+                                                    g=np.ones(SIZE), x=np.ones(SIZE),
                                                     theta=thetas))
-        p.model.add_subsystem('delta_theta_con', ExecComp('g = arctan(y/x)[::2]-arctan(y/x)[1::2]',
-                                                          g=np.ones(SIZE//2), x=np.ones(SIZE),
-                                                          y=np.ones(SIZE)))
+        p.model.add_subsystem('delta_theta_con', ExecComp('g = even - odd', vectorize=True,
+                                                        g=np.ones(SIZE//2), even=np.ones(SIZE//2),
+                                                        odd=np.ones(SIZE//2)))
 
-        thetas = np.linspace(0, np.pi/4, SIZE)
+        p.model.add_subsystem('l_conx', ExecComp('g=x-1', vectorize=True, g=np.ones(SIZE), x=np.ones(SIZE)))
 
-        p.model.add_subsystem('l_conx', ExecComp('g=x-1', g=np.ones(SIZE), x=np.ones(SIZE)))
+        IND = np.arange(SIZE, dtype=int)
+        ODD_IND = IND[1::2]  # all odd indices
+        EVEN_IND = IND[0::2]  # all even indices
 
         p.model.connect('r', ('circle.r', 'r_con.r'))
-        p.model.connect('x', ['r_con.x', 'theta_con.x', 'delta_theta_con.x'])
-
-        p.model.connect('x', 'l_conx.x')
-
-        p.model.connect('y', ['r_con.y', 'theta_con.y', 'delta_theta_con.y'])
+        p.model.connect('x', ['r_con.x', 'arctan_yox.x', 'l_conx.x'])
+        p.model.connect('y', ['r_con.y', 'arctan_yox.y'])
+        p.model.connect('arctan_yox.g', 'theta_con.x')
+        p.model.connect('arctan_yox.g', 'delta_theta_con.even', src_indices=EVEN_IND)
+        p.model.connect('arctan_yox.g', 'delta_theta_con.odd', src_indices=ODD_IND)
 
         p.driver = ScipyOptimizeDriver()
         p.driver.options['optimizer'] = 'SLSQP'
@@ -644,9 +699,7 @@ class SimulColoringScipyTestCase(unittest.TestCase):
         # nonlinear constraints
         p.model.add_constraint('r_con.g', equals=0)
 
-        IND = np.arange(SIZE, dtype=int)
-        ODD_IND = IND[0::2]  # all odd indices
-        p.model.add_constraint('theta_con.g', lower=-1e-5, upper=1e-5, indices=ODD_IND)
+        p.model.add_constraint('theta_con.g', lower=-1e-5, upper=1e-5, indices=EVEN_IND)
         p.model.add_constraint('delta_theta_con.g', lower=-1e-5, upper=1e-5)
 
         # this constrains x[0] to be 1 (see definition of l_conx)
@@ -700,11 +753,10 @@ class SimulColoringScipyTestCase(unittest.TestCase):
 
 
 class SimulColoringRevScipyTestCase(unittest.TestCase):
-    """Reverse mode coloring tests."""
+    """Rev mode coloring tests."""
 
-    def test_simul_coloring(self):
-
-        color_info = {"rev": [[
+    def setUp(self):
+        self.color_info = {"rev": [[
                [4, 5, 6, 7, 8, 9, 10],   # uncolored rows
                [2, 21],   # color 1
                [3, 16],   # color 2
@@ -737,8 +789,12 @@ class SimulColoringRevScipyTestCase(unittest.TestCase):
             ]],
             "sparsity": None}
 
+    def test_simul_coloring(self):
+
+        color_info = self.color_info
+
         p = run_opt(ScipyOptimizeDriver, 'rev', optimizer='SLSQP', disp=False)
-        p_color = run_opt(ScipyOptimizeDriver, 'rev', color_info, optimizer='SLSQP', disp=False)
+        p_color = run_opt(ScipyOptimizeDriver, 'rev', color_info=color_info, optimizer='SLSQP', disp=False)
 
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
@@ -746,8 +802,14 @@ class SimulColoringRevScipyTestCase(unittest.TestCase):
         # - coloring saves 11 solves per driver iter  (11 vs 22)
         # - initial solve for linear constraints takes 1 in both cases (only done once)
         # - (total_solves - 1) / (solves_per_iter) should be equal between the two cases
-        self.assertEqual((p.model.linear_solver._solve_count - 1) / 22,
-                         (p_color.model.linear_solver._solve_count - 1) / 11)
+        self.assertEqual((p.model._solve_count - 1) / 22,
+                         (p_color.model._solve_count - 1) / 11)
+
+    def test_bad_mode(self):
+        with self.assertRaises(Exception) as context:
+            p_color = run_opt(ScipyOptimizeDriver, 'fwd', color_info=self.color_info, optimizer='SLSQP', disp=False)
+        self.assertEqual(str(context.exception),
+                         "Simultaneous coloring does reverse solves but mode has been set to 'fwd'")
 
     def test_dynamic_simul_coloring(self):
 
@@ -757,13 +819,20 @@ class SimulColoringRevScipyTestCase(unittest.TestCase):
         assert_almost_equal(p['circle.area'], np.pi, decimal=7)
         assert_almost_equal(p_color['circle.area'], np.pi, decimal=7)
 
-        # - bidirectional coloring saves 17 solves per driver iter  (5 vs 22)
+        # - bidirectional coloring saves 11 solves per driver iter  (11 vs 22)
         # - initial solve for linear constraints takes 1 in both cases (only done once)
         # - dynamic case does 3 full compute_totals to compute coloring, which adds 22 * 3 solves
         # - (total_solves - N) / (solves_per_iter) should be equal between the two cases,
         # - where N is 1 for the uncolored case and 22 * 3 + 1 for the dynamic colored case.
-        self.assertEqual((p.model.linear_solver._solve_count - 1) / 22,
-                         (p_color.model.linear_solver._solve_count - 1 - 22 * 3) / 5)
+        self.assertEqual((p.model._solve_count - 1) / 22,
+                         (p_color.model._solve_count - 1 - 22 * 3) / 11)
+
+    def test_dynamic_simul_coloring_no_derivs(self):
+        with self.assertRaises(Exception) as context:
+            p_color = run_opt(ScipyOptimizeDriver, 'rev', optimizer='SLSQP', disp=False,
+                              dynamic_simul_derivs=True, derivs=False)
+        self.assertEqual(str(context.exception),
+                         "Derivative support has been turned off but compute_totals was called.")
 
 
 class SparsityTestCase(unittest.TestCase):
@@ -853,50 +922,148 @@ class SparsityTestCase(unittest.TestCase):
 
 
 class BidirectionalTestCase(unittest.TestCase):
+    def test_eisenstat(self):
+        for n in range(6, 20, 2):
+            builder = TotJacBuilder.eisenstat(n)
+            builder.color('auto', stream=None)
+            tot_size, tot_colors, fwd_solves, rev_solves, pct = _solves_info(builder.coloring)
+            if tot_colors == n // 2 + 3:
+                raise unittest.SkipTest("Current bicoloring algorithm requires n/2 + 3 solves, so skipping for now.")
+            self.assertLessEqual(tot_colors, n // 2 + 2,
+                                 "Eisenstat's example of size %d required %d colors but shouldn't "
+                                 "need more than %d." % (n, tot_colors, n // 2 + 2))
+
+            builder_fwd = TotJacBuilder.eisenstat(n)
+            builder_fwd.color('fwd', stream=None)
+            tot_size, tot_colors, fwd_solves, rev_solves, pct = _solves_info(builder_fwd.coloring)
+            # The columns of Eisenstat's example are pairwise nonorthogonal, so fwd coloring
+            # should require n colors.
+            self.assertEqual(n, tot_colors,
+                             "Eisenstat's example of size %d was not constructed properly. "
+                             "fwd coloring required only %d colors but should have required "
+                             "%d" % (n, tot_colors, n))
+
+    def test_arrowhead(self):
+        for n in range(5, 50, 55):
+            builder = TotJacBuilder(n, n)
+            builder.add_row(0)
+            builder.add_col(0)
+            builder.add_block_diag([(1,1)] * (n-1), 1, 1)
+            builder.color('auto', stream=None)
+            tot_size, tot_colors, fwd_solves, rev_solves, pct = _solves_info(builder.coloring)
+            self.assertEqual(tot_colors, 3)
+
+    @unittest.skipIf(LooseVersion(scipy.__version__) < LooseVersion("0.19.1"), "scipy version too old")
+    def test_can_715(self):
+        # this test is just to show the superiority of bicoloring vs. single coloring in
+        # either direction.  Bicoloring gives only 21 colors in this case vs. 105 for either
+        # fwd or rev.
+        matdir = os.path.join(os.path.dirname(openmdao.test_suite.__file__), 'matrices')
+
+        # uses matrix can_715 from the sparse matrix collection website
+        mat = load_npz(os.path.join(matdir, 'can_715.npz')).toarray()
+        mat = np.asarray(mat, dtype=bool)
+        coloring = get_simul_meta(None, 'auto', include_sparsity=False, setup=False,
+                                  run_model=False, bool_jac=mat,
+                                  stream=None)
+
+        tot_size, tot_colors, fwd_solves, rev_solves, pct = _solves_info(coloring)
+
+        self.assertEqual(tot_colors, 21)
+
+        # verify that unidirectional colorings are much worse (105 vs 21 for bidirectional)
+        coloring = get_simul_meta(None, 'fwd', include_sparsity=False, setup=False,
+                                  run_model=False, bool_jac=mat,
+                                  stream=None)
+
+        tot_size, tot_colors, fwd_solves, rev_solves, pct = _solves_info(coloring)
+
+        self.assertEqual(tot_colors, 105)
+
+        coloring = get_simul_meta(None, 'rev', include_sparsity=False, setup=False,
+                                  run_model=False, bool_jac=mat,
+                                  stream=None)
+
+        tot_size, tot_colors, fwd_solves, rev_solves, pct = _solves_info(coloring)
+
+        self.assertEqual(tot_colors, 105)
+
+
+def _get_mat(rows, cols):
+    if MPI:
+        if MPI.COMM_WORLD.rank == 0:
+            mat = np.random.random(rows * cols).reshape((rows, cols)) - 0.5
+        else:
+            mat = None
+        return MPI.COMM_WORLD.bcast(mat, root=0)
+    else:
+        return np.random.random(rows * cols).reshape((rows, cols)) - 0.5
+
+
+@unittest.skipUnless(PETScVector is not None and OPTIMIZER is not None, "PETSc and pyOptSparse required.")
+class MatMultMultipointTestCase(unittest.TestCase):
+    N_PROCS = 4
+
+    def test_multipoint_with_coloring(self):
+        size = 10
+        num_pts = self.N_PROCS
+
+        np.random.seed(11)
 
-    def test_exclude(self):
         p = Problem()
+        p.driver = pyOptSparseDriver()
+        p.driver.options['optimizer'] = OPTIMIZER
+        p.driver.options['dynamic_simul_derivs'] = True
+        if OPTIMIZER == 'SNOPT':
+            p.driver.opt_settings['Major iterations limit'] = 100
+            p.driver.opt_settings['Major feasibility tolerance'] = 1.0E-6
+            p.driver.opt_settings['Major optimality tolerance'] = 1.0E-6
+            p.driver.opt_settings['iSumm'] = 6
+
         model = p.model
+        for i in range(num_pts):
+            model.add_subsystem('indep%d' % i, IndepVarComp('x', val=np.ones(size)))
+            model.add_design_var('indep%d.x' % i)
 
-        indep = model.add_subsystem('indep', IndepVarComp())
-        indep.add_output("a", val=1.0)
-        indep.add_output("b", val=1.0)
-        indep.add_output("c", val=1.0)
-        indep.add_output("d", val=1.0)
-        indep.add_output("e", val=1.0)
+        par1 = model.add_subsystem('par1', ParallelGroup())
+        for i in range(num_pts):
+            mat = _get_mat(5, size)
+            par1.add_subsystem('comp%d' % i, ExecComp('y=A.dot(x)', A=mat, x=np.ones(size), y=np.ones(5)))
+            model.connect('indep%d.x' % i, 'par1.comp%d.x' % i)
 
-        obj = model.add_subsystem('obj', ExecComp('obj=a+b+c+d+ee'))
-        con1 = model.add_subsystem('con1', ExecComp('con=a*2.0 + b'))
-        con2 = model.add_subsystem('con2', ExecComp('con=c*3.0 + 2.0*d'))
-        con3 = model.add_subsystem('con3', ExecComp('con=ee*1.5 - 3.0*c + a'))
+        par2 = model.add_subsystem('par2', ParallelGroup())
+        for i in range(num_pts):
+            mat = _get_mat(size, 5)
+            par2.add_subsystem('comp%d' % i, ExecComp('y=A.dot(x)', A=mat, x=np.ones(5), y=np.ones(size)))
+            model.connect('par1.comp%d.y' % i, 'par2.comp%d.x' % i)
+            par2.add_constraint('comp%d.y' % i, lower=-1.)
 
-        model.connect("indep.a", ("obj.a", "con1.a", "con3.a"))
-        model.connect("indep.b", ("obj.b", "con1.b"))
-        model.connect("indep.c", ("obj.c", "con2.c", "con3.c"))
-        model.connect("indep.d", ("obj.d", "con2.d"))
-        model.connect("indep.e", ("obj.ee", "con3.ee"))
+            model.add_subsystem('normcomp%d' % i, ExecComp("y=sum(x*x)", x=np.ones(size)))
+            model.connect('par2.comp%d.y' % i, 'normcomp%d.x' % i)
 
-        model.add_design_var("indep.a")
-        model.add_design_var("indep.b")
-        model.add_design_var("indep.c")
-        model.add_design_var("indep.d")
-        model.add_design_var("indep.e")
+        model.add_subsystem('obj', ExecComp("y=" + '+'.join(['x%d' % i for i in range(num_pts)])))
 
-        model.add_objective('obj.obj')
-        model.add_constraint('con1.con')
-        model.add_constraint('con2.con')
-        model.add_constraint('con3.con', simul_coloring_excludes=True)
+        for i in range(num_pts):
+            model.connect('normcomp%d.y' % i, 'obj.x%d' % i)
 
-        p.setup(mode='fwd')
-        p.run_model()
+        model.add_objective('obj.y')
 
-        coloring = get_simul_meta(p, include_sparsity=False, setup=False, run_model=False, stream=None)
-        tot_size1, tot_colors1, colored_solves1, opp_solves1, pct1, dominant_mode1 = _solves_info(coloring)
+        p.setup()
 
-        # this is not a great coloring. It's just done to test simul_coloring_excludes
-        self.assertEqual(opp_solves1, 2)
-        self.assertEqual(tot_colors1, 4)
-        self.assertEqual(coloring['rev'][0][0], [3,0])
+        p.run_driver()
+
+        J = p.compute_totals()
+
+        for i in range(num_pts):
+            vname = 'par2.comp%d.A' % i
+            if vname in model._var_abs_names['input']:
+                norm = np.linalg.norm(J['par2.comp%d.y'%i,'indep%d.x'%i] -
+                                      getattr(par2, 'comp%d'%i)._inputs['A'].dot(getattr(par1, 'comp%d'%i)._inputs['A']))
+                self.assertLess(norm, 1.e-7)
+            elif vname not in model._var_allprocs_abs_names['input']:
+                self.fail("Can't find variable par2.comp%d.A" % i)
+
+        # print("final obj:", p['obj.y'])
 
 
 if __name__ == '__main__':
