@@ -8,8 +8,9 @@ from six import assertRaisesRegex, StringIO, assertRegex
 import numpy as np
 
 from openmdao.core.group import get_relevant_vars
+from openmdao.core.driver import Driver
 from openmdao.api import Problem, IndepVarComp, NonlinearBlockGS, ScipyOptimizeDriver, \
-    ExecComp, Group, NewtonSolver, ImplicitComponent, ScipyKrylov
+    ExecComp, Group, NewtonSolver, ImplicitComponent, ScipyKrylov, ExplicitComponent, NonlinearRunOnce
 from openmdao.utils.assert_utils import assert_rel_error
 from openmdao.test_suite.components.paraboloid import Paraboloid
 from openmdao.test_suite.components.sellar import SellarDerivatives
@@ -22,7 +23,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.test_suite.components.paraboloid import Paraboloid
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
 
         model.add_subsystem('p1', IndepVarComp('x', 3.0))
         model.add_subsystem('p2', IndepVarComp('y', -4.0))
@@ -41,15 +42,15 @@ class TestProblem(unittest.TestCase):
         from openmdao.test_suite.components.paraboloid import Paraboloid
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
 
         model.add_subsystem('p1', IndepVarComp('x', 3.0))
 
-        #promote the two inputs to the same name
+        # promote the two inputs to the same name
         model.add_subsystem('comp1', Paraboloid(), promotes_inputs=['x'])
         model.add_subsystem('comp2', Paraboloid(), promotes_inputs=['x'])
 
-        #connect the source to the common name
+        # connect the source to the common name
         model.connect('p1.x', 'x')
 
         prob.setup()
@@ -63,7 +64,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.test_suite.components.paraboloid import Paraboloid
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
 
         model.add_subsystem('p1', IndepVarComp('x', 3.0))
         model.add_subsystem('p2', IndepVarComp('y', -4.0))
@@ -76,8 +77,8 @@ class TestProblem(unittest.TestCase):
         prob.run_model()
 
         totals = prob.compute_totals(of=['comp.f_xy'], wrt=['p1.x', 'p2.y'])
-        assert_rel_error(self, totals[('comp.f_xy','p1.x')][0][0], -4.0)
-        assert_rel_error(self, totals[('comp.f_xy','p2.y')][0][0], 3.0)
+        assert_rel_error(self, totals[('comp.f_xy', 'p1.x')][0][0], -4.0)
+        assert_rel_error(self, totals[('comp.f_xy', 'p2.y')][0][0], 3.0)
 
         totals = prob.compute_totals(of=['comp.f_xy'], wrt=['p1.x', 'p2.y'], return_format='dict')
         assert_rel_error(self, totals['comp.f_xy']['p1.x'][0][0], -4.0)
@@ -88,7 +89,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.test_suite.components.paraboloid import Paraboloid
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
 
         model.add_subsystem('p1', IndepVarComp('x', 3.0))
         model.add_subsystem('p2', IndepVarComp('y', -4.0))
@@ -105,15 +106,15 @@ class TestProblem(unittest.TestCase):
         prob.run_model()
 
         totals = prob.compute_totals(of=['comp.f_xy'], wrt=['p1.x', 'p2.y'], driver_scaling=True)
-        assert_rel_error(self, totals[('comp.f_xy','p1.x')][0][0], 196.0)
-        assert_rel_error(self, totals[('comp.f_xy','p2.y')][0][0], 3.0)
+        assert_rel_error(self, totals[('comp.f_xy', 'p1.x')][0][0], 196.0)
+        assert_rel_error(self, totals[('comp.f_xy', 'p2.y')][0][0], 3.0)
 
     def test_feature_simple_run_once_set_deriv_mode(self):
         from openmdao.api import Problem, Group, IndepVarComp
         from openmdao.test_suite.components.paraboloid import Paraboloid
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
 
         model.add_subsystem('p1', IndepVarComp('x', 3.0))
         model.add_subsystem('p2', IndepVarComp('y', -4.0))
@@ -123,19 +124,48 @@ class TestProblem(unittest.TestCase):
         model.connect('p2.y', 'comp.y')
 
         prob.setup(mode='rev')
-        #prob.setup(mode='fwd')
+        # prob.setup(mode='fwd')
         prob.run_model()
 
         assert_rel_error(self, prob['comp.f_xy'], -15.0)
 
         prob.compute_totals(of=['comp.f_xy'], wrt=['p1.x', 'p2.y'])
 
+    def test_compute_totals_cleanup(self):
+        p = Problem()
+        model = p.model
+        model.add_subsystem('indeps1', IndepVarComp('x', np.ones(5)))
+        model.add_subsystem('indeps2', IndepVarComp('x', np.ones(3)))
+
+        model.add_subsystem('MP1', ExecComp('y=7*x', x=np.zeros(5), y=np.zeros(5)))
+        model.add_subsystem('MP2', ExecComp('y=-3*x', x=np.zeros(3), y=np.zeros(3)))
+
+        model.add_design_var('indeps1.x')
+        model.add_design_var('indeps2.x')
+
+        model.add_constraint('MP1.y')
+        model.add_constraint('MP2.y')
+
+        model.connect('indeps1.x', 'MP1.x')
+        model.connect('indeps2.x', 'MP2.x')
+
+        p.setup(mode='rev')
+        p.run_model()
+
+        J = p.compute_totals()
+        assert_rel_error(self, J[('MP1.y', 'indeps1.x')], np.eye(5)*7., 1e-10)
+        assert_rel_error(self, J[('MP2.y', 'indeps2.x')], np.eye(3)*-3., 1e-10)
+        # before the bug fix, the following two derivs contained nonzero values even
+        # though the variables involved were not dependent on each other.
+        assert_rel_error(self, J[('MP2.y', 'indeps1.x')], np.zeros((3, 5)), 1e-10)
+        assert_rel_error(self, J[('MP1.y', 'indeps2.x')], np.zeros((5, 3)), 1e-10)
+
     def test_set_2d_array(self):
         import numpy as np
 
         from openmdao.api import Problem, IndepVarComp, Group
 
-        prob = Problem(model=Group())
+        prob = Problem()
         model = prob.model
         model.add_subsystem(name='indeps',
                             subsys=IndepVarComp(name='X_c', shape=(3, 1)))
@@ -151,7 +181,7 @@ class TestProblem(unittest.TestCase):
         prob['indeps.X_c'][:, 0] = new_val
         prob.final_setup()
 
-        assert_rel_error(self, prob['indeps.X_c'], new_val.reshape((3,1)), 1e-10)
+        assert_rel_error(self, prob['indeps.X_c'], new_val.reshape((3, 1)), 1e-10)
         assert_rel_error(self, prob['indeps.X_c'][:, 0], new_val, 1e-10)
 
     def test_set_checks_shape(self):
@@ -209,7 +239,7 @@ class TestProblem(unittest.TestCase):
         # Basic test for the method using default solvers on simple model.
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
         model.add_subsystem('p1', IndepVarComp('x', 0.0), promotes=['x'])
         model.add_subsystem('p2', IndepVarComp('y', 0.0), promotes=['y'])
         model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
@@ -239,7 +269,7 @@ class TestProblem(unittest.TestCase):
         # Make sure 'dict' return_format works.
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
         model.add_subsystem('p1', IndepVarComp('x', 0.0), promotes=['x'])
         model.add_subsystem('p2', IndepVarComp('y', 0.0), promotes=['y'])
         model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
@@ -265,6 +295,48 @@ class TestProblem(unittest.TestCase):
         assert_rel_error(self, derivs['f_xy']['x'], [[-6.0]], 1e-6)
         assert_rel_error(self, derivs['f_xy']['y'], [[8.0]], 1e-6)
 
+    def test_compute_totals_no_args_no_desvar(self):
+        p = Problem()
+
+        dv = p.model.add_subsystem('des_vars', IndepVarComp())
+        dv.add_output('x', val=2.)
+
+        p.model.add_subsystem('calc', ExecComp('y=2*x'))
+
+        p.model.connect('des_vars.x', 'calc.x')
+
+        p.model.add_objective('calc.y')
+
+        p.setup()
+        p.run_model()
+
+        with self.assertRaises(RuntimeError) as cm:
+            p.compute_totals()
+
+        self.assertEqual(str(cm.exception),
+                         "Driver is not providing any design variables for compute_totals.")
+
+    def test_compute_totals_no_args_no_response(self):
+        p = Problem()
+
+        dv = p.model.add_subsystem('des_vars', IndepVarComp())
+        dv.add_output('x', val=2.)
+
+        p.model.add_subsystem('calc', ExecComp('y=2*x'))
+
+        p.model.connect('des_vars.x', 'calc.x')
+
+        p.model.add_design_var('des_vars.x')
+
+        p.setup()
+        p.run_model()
+
+        with self.assertRaises(RuntimeError) as cm:
+            p.compute_totals()
+
+        self.assertEqual(str(cm.exception),
+                         "Driver is not providing any response variables for compute_totals.")
+
     def test_compute_totals_no_args(self):
         p = Problem()
 
@@ -309,7 +381,7 @@ class TestProblem(unittest.TestCase):
 
         prob = Problem()
 
-        model = prob.model = Group()
+        model = prob.model
         model.add_subsystem('p1', IndepVarComp('x', 0.0), promotes=['x'])
         model.add_subsystem('p2', IndepVarComp('y', 0.0), promotes=['y'])
         model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
@@ -326,7 +398,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.test_suite.components.paraboloid import Paraboloid
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
         model.add_subsystem('p1', IndepVarComp('x', 0.0), promotes=['x'])
         model.add_subsystem('p2', IndepVarComp('y', 0.0), promotes=['y'])
         model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
@@ -356,7 +428,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.test_suite.components.paraboloid import Paraboloid
 
         prob = Problem()
-        model = prob.model = Group()
+        model = prob.model
         model.add_subsystem('p1', IndepVarComp('x', 0.0), promotes=['x'])
         model.add_subsystem('p2', IndepVarComp('y', 0.0), promotes=['y'])
         model.add_subsystem('comp', Paraboloid(), promotes=['x', 'y', 'f_xy'])
@@ -496,8 +568,84 @@ class TestProblem(unittest.TestCase):
         # actually want the optimizer to run
         prob.run_model()
 
+        prob.model.nonlinear_solver.options['atol'] = 1e-15
+        prob.model.nonlinear_solver.options['rtol'] = 1e-15
+
+        # check derivatives with complex step
+        prob.check_totals(method='cs')
+
+    def test_check_totals_user_detect(self):
+
+        class SimpleComp(ExplicitComponent):
+
+            def setup(self):
+                self.add_input('x', val=1.0)
+                self.add_output('y', val=1.0)
+
+                self.declare_partials(of='y', wrt='x')
+
+                if not self.force_alloc_complex:
+                    raise RuntimeError('force_alloc_complex not set in component.')
+
+            def compute(self, inputs, outputs):
+                outputs['y'] = 3.0*inputs['x']
+
+                if np.iscomplex(inputs._data[0]) and not self.under_complex_step:
+                    raise RuntimeError('under_complex_step not set in component.')
+
+            def compute_partials(self, inputs, partials):
+                partials['y', 'x'] = 3.
+
+        prob = Problem()
+        prob.model.add_subsystem('px', IndepVarComp('x', 2.0))
+        prob.model.add_subsystem('comp', SimpleComp())
+        prob.model.connect('px.x', 'comp.x')
+
+        prob.model.add_design_var('px.x', lower=-100, upper=100)
+        prob.model.add_objective('comp.y')
+
+        prob.setup(force_alloc_complex=True)
+
+        prob.run_model()
+
         # check derivatives with complex step and a larger step size.
-        prob.check_totals(method='cs', step=1.0e-1)
+        prob.check_totals(method='cs', out_stream=None)
+        self.assertFalse(prob.model.under_complex_step,
+                         msg="The under_complex_step flag should be reset.")
+
+    def test_feature_check_totals_user_detect_forced(self):
+        from openmdao.api import Problem, ExplicitComponent
+
+        class SimpleComp(ExplicitComponent):
+
+            def setup(self):
+                self.add_input('x', val=1.0)
+                self.add_output('y', val=1.0)
+
+                self.declare_partials(of='y', wrt='x')
+
+                if self.force_alloc_complex:
+                    print("Vectors allocated for complex step.")
+
+            def compute(self, inputs, outputs):
+                outputs['y'] = 3.0*inputs['x']
+
+            def compute_partials(self, inputs, partials):
+                partials['y', 'x'] = 3.
+
+        prob = Problem()
+        prob.model.add_subsystem('px', IndepVarComp('x', val=1.0))
+        prob.model.add_subsystem('comp', SimpleComp())
+        prob.model.connect('px.x', 'comp.x')
+
+        prob.model.add_design_var('px.x', lower=-100, upper=100)
+        prob.model.add_objective('comp.y')
+
+        prob.setup(force_alloc_complex=True)
+
+        prob.run_model()
+
+        prob.check_totals(method='cs')
 
     def test_feature_run_driver(self):
         import numpy as np
@@ -505,8 +653,8 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, NonlinearBlockGS, ScipyOptimizeDriver
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
-        prob = Problem()
-        model = prob.model = SellarDerivatives()
+        prob = Problem(model=SellarDerivatives())
+        model = prob.model
         model.nonlinear_solver = NonlinearBlockGS()
 
         prob.driver = ScipyOptimizeDriver()
@@ -532,8 +680,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, NonlinearBlockGS
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
-        prob = Problem()
-        prob.model = SellarDerivatives()
+        prob = Problem(model=SellarDerivatives())
         prob.model.nonlinear_solver = NonlinearBlockGS()
 
         prob.setup()
@@ -550,8 +697,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, NonlinearBlockGS
         from openmdao.test_suite.components.sellar import SellarDerivativesConnected
 
-        prob = Problem()
-        prob.model = SellarDerivativesConnected()
+        prob = Problem(model= SellarDerivativesConnected())
         prob.model.nonlinear_solver = NonlinearBlockGS()
 
         prob.setup()
@@ -568,8 +714,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, NonlinearBlockGS
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
-        prob = Problem()
-        prob.model = SellarDerivatives()
+        prob = Problem(model=SellarDerivatives())
         prob.model.nonlinear_solver = NonlinearBlockGS()
 
         prob.setup()
@@ -589,16 +734,20 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, ExecComp
 
         prob = Problem()
-        comp = prob.model.add_subsystem('comp', ExecComp('y=x-25.', x={'value': 77.0, 'units': 'degF'},
-                                                         y={'units': 'degC'}))
-        comp = prob.model.add_subsystem('prom', ExecComp('yy=xx-25.', xx={'value': 77.0, 'units': 'degF'},
-                                                         yy={'units': 'degC'}),
-                                        promotes=['xx', 'yy'])
-        comp = prob.model.add_subsystem('acomp', ExecComp('y=x-25.', x={'value': np.array([77.0, 95.0]), 'units': 'degF'},
-                                                          y={'units': 'degC'}))
-        comp = prob.model.add_subsystem('aprom', ExecComp('ayy=axx-25.', axx={'value': np.array([77.0, 95.0]), 'units': 'degF'},
-                                                          ayy={'units': 'degC'}),
-                                        promotes=['axx', 'ayy'])
+        prob.model.add_subsystem('comp', ExecComp('y=x-25.',
+                                                  x={'value': 77.0, 'units': 'degF'},
+                                                  y={'value': 0.0, 'units': 'degC'}))
+        prob.model.add_subsystem('prom', ExecComp('yy=xx-25.',
+                                                  xx={'value': 77.0, 'units': 'degF'},
+                                                  yy={'value': 0.0, 'units': 'degC'}),
+                                 promotes=['xx', 'yy'])
+        prob.model.add_subsystem('acomp', ExecComp('y=x-25.',
+                                                   x={'value': np.array([77.0, 95.0]), 'units': 'degF'},
+                                                   y={'value': 0.0, 'units': 'degC'}))
+        prob.model.add_subsystem('aprom', ExecComp('ayy=axx-25.',
+                                                   axx={'value': np.array([77.0, 95.0]), 'units': 'degF'},
+                                                   ayy={'value': 0.0, 'units': 'degC'}),
+                                 promotes=['axx', 'ayy'])
 
         prob.setup(check=False)
 
@@ -700,9 +849,10 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, ExecComp
 
         prob = Problem()
-        comp = prob.model.add_subsystem('comp', ExecComp('y=x+1.', x={'value': 100.0, 'units': 'cm'},
-                                                         y={'units': 'm'}))
-        comp = prob.model.add_subsystem('no_unit', ExecComp('y=x+1.', x={'value': 100.0}))
+        prob.model.add_subsystem('comp', ExecComp('y=x+1.',
+                                                  x={'value': 100.0, 'units': 'cm'},
+                                                  y={'units': 'm'}))
+        prob.model.add_subsystem('no_unit', ExecComp('y=x+1.', x={'value': 100.0}))
 
         prob.setup()
         prob.run_model()
@@ -727,8 +877,9 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, ExecComp
 
         prob = Problem()
-        comp = prob.model.add_subsystem('comp', ExecComp('y=x+1.', x={'value': 100.0, 'units': 'cm'},
-                                                         y={'units': 'm'}))
+        prob.model.add_subsystem('comp', ExecComp('y=x+1.',
+                                                  x={'value': 100.0, 'units': 'cm'},
+                                                  y={'units': 'm'}))
 
         prob.setup()
         prob.run_model()
@@ -743,8 +894,9 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, ExecComp
 
         prob = Problem()
-        comp = prob.model.add_subsystem('comp', ExecComp('y=x+1.', x={'value': np.array([100.0, 33.3]), 'units': 'cm'},
-                                                         y={'shape': (2, ), 'units': 'm'}))
+        prob.model.add_subsystem('comp', ExecComp('y=x+1.',
+                                                  x={'value': np.array([100.0, 33.3]), 'units': 'cm'},
+                                                  y={'shape': (2, ), 'units': 'm'}))
 
         prob.setup()
         prob.run_model()
@@ -767,8 +919,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, NonlinearBlockGS
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
-        prob = Problem()
-        prob.model = SellarDerivatives()
+        prob = Problem(model=SellarDerivatives())
         prob.model.nonlinear_solver = NonlinearBlockGS()
 
         prob.setup()
@@ -800,8 +951,7 @@ class TestProblem(unittest.TestCase):
         from openmdao.api import Problem, NonlinearBlockGS
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
-        prob = Problem()
-        prob.model = SellarDerivatives()
+        prob = Problem(model=SellarDerivatives())
         prob.model.nonlinear_solver = NonlinearBlockGS()
 
         prob.setup()
@@ -945,27 +1095,63 @@ class TestProblem(unittest.TestCase):
         # testing the root kwarg
         with self.assertRaises(ValueError) as cm:
             prob = Problem(root=Group(), model=Group())
-        err = cm.exception
-        self.assertEqual(str(err), "cannot specify both `root` and `model`. `root` has been "
-                         "deprecated, please use model")
+
+        self.assertEqual(str(cm.exception),
+                         "Cannot specify both 'root' and 'model'. "
+                         "'root' has been deprecated, please use 'model'.")
 
         with warnings.catch_warnings(record=True) as w:
-            prob = Problem(root=Group)
+            prob = Problem(root=Group())
 
         self.assertEqual(str(w[0].message), "The 'root' argument provides backwards "
                          "compatibility with OpenMDAO <= 1.x ; use 'model' instead.")
 
+    def test_args(self):
+        # defaults
+        prob = Problem()
+        self.assertTrue(isinstance(prob.model, Group))
+        self.assertTrue(isinstance(prob.driver, Driver))
+
+        # model
+        prob = Problem(SellarDerivatives())
+        self.assertTrue(isinstance(prob.model, SellarDerivatives))
+        self.assertTrue(isinstance(prob.driver, Driver))
+
+        # driver
+        prob = Problem(driver=ScipyOptimizeDriver())
+        self.assertTrue(isinstance(prob.model, Group))
+        self.assertTrue(isinstance(prob.driver, ScipyOptimizeDriver))
+
+        # model and driver
+        prob = Problem(model=SellarDerivatives(), driver=ScipyOptimizeDriver())
+        self.assertTrue(isinstance(prob.model, SellarDerivatives))
+        self.assertTrue(isinstance(prob.driver, ScipyOptimizeDriver))
+
+        # invalid model
+        with self.assertRaises(TypeError) as cm:
+            prob = Problem(ScipyOptimizeDriver())
+
+        self.assertEqual(str(cm.exception),
+                         "The value provided for 'model' is not a valid System.")
+
+        # invalid driver
+        with self.assertRaises(TypeError) as cm:
+            prob = Problem(driver=SellarDerivatives())
+
+        self.assertEqual(str(cm.exception),
+                         "The value provided for 'driver' is not a valid Driver.")
+
     def test_relevance(self):
         p = Problem()
         model = p.model
 
-        indep1 = model.add_subsystem("indep1", IndepVarComp('x', 1.0))
+        model.add_subsystem("indep1", IndepVarComp('x', 1.0))
         G1 = model.add_subsystem('G1', Group())
         G1.add_subsystem('C1', ExecComp(['x=2.0*a', 'y=2.0*b', 'z=2.0*a']))
         G1.add_subsystem('C2', ExecComp(['x=2.0*a', 'y=2.0*b', 'z=2.0*b']))
         model.add_subsystem("C3", ExecComp(['x=2.0*a', 'y=2.0*b+3.0*c']))
         model.add_subsystem("C4", ExecComp(['x=2.0*a', 'y=2.0*b']))
-        indep2 = model.add_subsystem("indep2", IndepVarComp('x', 1.0))
+        model.add_subsystem("indep2", IndepVarComp('x', 1.0))
         G2 = model.add_subsystem('G2', Group())
         G2.add_subsystem('C5', ExecComp(['x=2.0*a', 'y=2.0*b+3.0*c']))
         G2.add_subsystem('C6', ExecComp(['x=2.0*a', 'y=2.0*b+3.0*c']))
@@ -1056,7 +1242,6 @@ class TestProblem(unittest.TestCase):
                     2 * inputs['a']**2 * outputs['x']
                 jacobian['x', 'a'] = -2 * inputs['a'] * outputs['x']**2
 
-
         class Sub(Group):
 
             def setup(self):
@@ -1069,7 +1254,6 @@ class TestProblem(unittest.TestCase):
                 # This will not solve it either.
                 self.nonlinear_solver = NonlinearBlockGS()
 
-
         class Super(Group):
 
             def setup(self):
@@ -1080,9 +1264,7 @@ class TestProblem(unittest.TestCase):
                 self.sub.nonlinear_solver = NewtonSolver()
                 self.sub.linear_solver = ScipyKrylov()
 
-
-        top = Problem()
-        top.model = Super()
+        top = Problem(model=Super())
 
         top.setup(check=False)
 
@@ -1107,7 +1289,6 @@ class TestProblem(unittest.TestCase):
                     2 * inputs['a']**2 * outputs['x']
                 jacobian['x', 'a'] = -2 * inputs['a'] * outputs['x']**2
 
-
         class Sub(Group):
 
             def setup(self):
@@ -1120,15 +1301,12 @@ class TestProblem(unittest.TestCase):
                 # This solver will get over-ridden below
                 self.nonlinear_solver = NonlinearBlockGS()
 
-
         class Super(Group):
 
             def setup(self):
                 self.add_subsystem('sub', Sub())
 
-
-        top = Problem()
-        top.model = Super()
+        top = Problem(model=Super())
 
         top.setup(check=False)
 
@@ -1157,7 +1335,6 @@ class TestProblem(unittest.TestCase):
                     2 * inputs['a']**2 * outputs['x']
                 jacobian['x', 'a'] = -2 * inputs['a'] * outputs['x']**2
 
-
         class Sub(Group):
             def setup(self):
                 self.add_subsystem('comp', ImplSimple())
@@ -1167,7 +1344,6 @@ class TestProblem(unittest.TestCase):
                 # to override it in the parent.
                 self.nonlinear_solver = NonlinearBlockGS()
 
-
         class Super(Group):
             def setup(self):
                 self.add_subsystem('sub', Sub())
@@ -1177,9 +1353,7 @@ class TestProblem(unittest.TestCase):
                 self.sub.nonlinear_solver = NewtonSolver()
                 self.sub.linear_solver = ScipyKrylov()
 
-
-        top = Problem()
-        top.model = Super()
+        top = Problem(model=Super())
 
         top.setup(check=False)
 
@@ -1204,7 +1378,6 @@ class TestProblem(unittest.TestCase):
                     2 * inputs['a']**2 * outputs['x']
                 jacobian['x', 'a'] = -2 * inputs['a'] * outputs['x']**2
 
-
         class Sub(Group):
 
             def setup(self):
@@ -1217,15 +1390,12 @@ class TestProblem(unittest.TestCase):
                 # This will not solve it either.
                 self.nonlinear_solver = NonlinearBlockGS()
 
-
         class Super(Group):
 
             def setup(self):
                 self.add_subsystem('sub', Sub())
 
-
-        top = Problem()
-        top.model = Super()
+        top = Problem(model=Super())
 
         top.setup(check=False)
 
@@ -1261,11 +1431,10 @@ class TestProblem(unittest.TestCase):
             Problem._post_setup_func = None
 
     def test_list_problem_vars(self):
-
-        prob = Problem()
-        model = prob.model = SellarDerivatives()
+        model = SellarDerivatives()
         model.nonlinear_solver = NonlinearBlockGS()
 
+        prob = Problem(model)
         prob.driver = ScipyOptimizeDriver()
         prob.driver.options['optimizer'] = 'SLSQP'
         prob.driver.options['tol'] = 1e-9
@@ -1288,13 +1457,12 @@ class TestProblem(unittest.TestCase):
         finally:
             sys.stdout = stdout
         output = strout.getvalue().split('\n')
-        self.assertEquals(output[1], 'Design Variables')
-        assertRegex(self, output[5], '^pz.z +\|[0-9. e+-]+\| +2')
-        self.assertEquals(output[9], 'Constraints')
-        assertRegex(self, output[14], '^con_cmp2.con2 +\[[0-9. e+-]+\] +1')
-        self.assertEquals(output[17], 'Objectives')
-        assertRegex(self, output[21], '^obj_cmp.obj +\[[0-9. e+-]+\] +1')
-
+        self.assertEquals(output[1], r'Design Variables')
+        assertRegex(self, output[5], r'^pz.z +\|[0-9. e+-]+\| +2')
+        self.assertEquals(output[9], r'Constraints')
+        assertRegex(self, output[14], r'^con_cmp2.con2 +\[[0-9. e+-]+\] +1')
+        self.assertEquals(output[17], r'Objectives')
+        assertRegex(self, output[21], r'^obj_cmp.obj +\[[0-9. e+-]+\] +1')
 
         # With show_promoted_name=False
         stdout = sys.stdout
@@ -1305,9 +1473,9 @@ class TestProblem(unittest.TestCase):
         finally:
             sys.stdout = stdout
         output = strout.getvalue().split('\n')
-        assertRegex(self, output[5], '^z +\|[0-9. e+-]+\| +2')
-        assertRegex(self, output[14], '^con2 +\[[0-9. e+-]+\] +1')
-        assertRegex(self, output[21], '^obj +\[[0-9. e+-]+\] +1')
+        assertRegex(self, output[5], r'^z +\|[0-9. e+-]+\| +2')
+        assertRegex(self, output[14], r'^con2 +\[[0-9. e+-]+\] +1')
+        assertRegex(self, output[21], r'^obj +\[[0-9. e+-]+\] +1')
 
         # With all the optional columns
         stdout = sys.stdout
@@ -1335,12 +1503,12 @@ class TestProblem(unittest.TestCase):
             sys.stdout = stdout
         output = strout.getvalue().split('\n')
         assertRegex(self, output[3],
-                    '^name\s+value\s+size\s+lower\s+upper\s+ref\s+ref0\s+'
-                    'indices\s+adder\s+scaler\s+parallel_deriv_color\s+'
-                    'vectorize_derivs\s+cache_linear_solution')
+                    r'^name\s+value\s+size\s+lower\s+upper\s+ref\s+ref0\s+'
+                    r'indices\s+adder\s+scaler\s+parallel_deriv_color\s+'
+                    r'vectorize_derivs\s+cache_linear_solution')
         assertRegex(self, output[5],
-                    '^pz.z\s+\|[0-9.e+-]+\|\s+2\s+\|10.0\|\s+\|[0-9.e+-]+\|\s+None\s+'
-                    'None\s+None\s+None\s+None\s+None\s+False\s+False')
+                    r'^pz.z\s+\|[0-9.e+-]+\|\s+2\s+\|10.0\|\s+\|[0-9.e+-]+\|\s+None\s+'
+                    r'None\s+None\s+None\s+None\s+None\s+False\s+False')
 
         # With all the optional columns and print_arrays
         stdout = sys.stdout
@@ -1367,20 +1535,20 @@ class TestProblem(unittest.TestCase):
         finally:
             sys.stdout = stdout
         output = strout.getvalue().split('\n')
-        assertRegex(self, output[6], '^\s+value:')
-        assertRegex(self, output[7], '^\s+\[[0-9. e+-]+\]')
-        assertRegex(self, output[9], '^\s+lower:')
-        assertRegex(self, output[10], '^\s+\[[0-9. e+-]+\]')
-        assertRegex(self, output[12], '^\s+upper:')
-        assertRegex(self, output[13], '^\s+\[[0-9. e+-]+\]')
+        assertRegex(self, output[6], r'^\s+value:')
+        assertRegex(self, output[7], r'^\s+\[[0-9. e+-]+\]')
+        assertRegex(self, output[9], r'^\s+lower:')
+        assertRegex(self, output[10], r'^\s+\[[0-9. e+-]+\]')
+        assertRegex(self, output[12], r'^\s+upper:')
+        assertRegex(self, output[13], r'^\s+\[[0-9. e+-]+\]')
 
     def test_feature_list_problem_vars(self):
         import numpy as np
         from openmdao.api import Problem, ScipyOptimizeDriver, NonlinearBlockGS
         from openmdao.test_suite.components.sellar import SellarDerivatives
 
-        prob = Problem()
-        model = prob.model = SellarDerivatives()
+        prob = Problem(model=SellarDerivatives())
+        model = prob.model
         model.nonlinear_solver = NonlinearBlockGS()
 
         prob.driver = ScipyOptimizeDriver()
@@ -1410,6 +1578,32 @@ class TestProblem(unittest.TestCase):
                                           'cache_linear_solution'],
                                )
 
+class NestedProblemTestCase(unittest.TestCase):
+
+    def test_nested_prob(self):
+
+        class _ProblemSolver(NonlinearRunOnce):
+            def solve(self):
+                # create a simple subproblem and run it to test for global solver_info bug
+                p = Problem()
+                p.model.add_subsystem('indep', IndepVarComp('x', 1.0))
+                p.model.add_subsystem('comp', ExecComp('y=2*x'))
+                p.model.connect('indep.x', 'comp.x')
+                p.setup()
+                p.run_model()
+
+                return super(_ProblemSolver, self).solve()
+
+        p = Problem()
+        p.model.add_subsystem('indep', IndepVarComp('x', 1.0))
+        G = p.model.add_subsystem('G', Group())
+        G.add_subsystem('comp', ExecComp('y=2*x'))
+        G.nonlinear_solver = _ProblemSolver()
+        p.model.connect('indep.x', 'G.comp.x')
+        p.setup()
+        p.run_model()
+
+
 
 
 if __name__ == "__main__":
