@@ -51,8 +51,8 @@ class ScalingExample2(ImplicitComponent):
     def setup(self):
         self.add_input('x1', val=100.0)
         self.add_input('x2', val=5000.0)
-        self.add_output('y1', val=200., res_ref=1e5)
-        self.add_output('y2', val=6000., res_ref=1e-5)
+        self.add_output('y1', val=200., ref=300.0, ref0=100.0)
+        self.add_output('y2', val=6000., ref=11000.0, ref0=1000.0)
 
     def apply_nonlinear(self, inputs, outputs, residuals):
         x1 = inputs['x1']
@@ -379,15 +379,15 @@ class TestScaling(unittest.TestCase):
         prob.setup(check=False)
         prob.run_model()
 
-        res1 = -model.p1._residuals.get_data()[0]
-        out1 = model.p1._outputs.get_data()[0]
-        out2 = model.p2._outputs.get_data()[0]
+        res1 = -model.p1._residuals._data[0]
+        out1 = model.p1._outputs._data[0]
+        out2 = model.p2._outputs._data[0]
 
         self.assertEqual(res1, out1 - 2.0*(out2 + 1.0))
         with model._scaled_context_all():
-            res1 = -model.p1._residuals.get_data()[0]
-            out1 = model.p1._outputs.get_data()[0]
-            out2 = model.p2._outputs.get_data()[0]
+            res1 = -model.p1._residuals._data[0]
+            out1 = model.p1._outputs._data[0]
+            out2 = model.p2._outputs._data[0]
 
             self.assertEqual(res1, out1 - 2.0*(out2 + 1.0))
 
@@ -417,13 +417,13 @@ class TestScaling(unittest.TestCase):
         prob.setup(check=False)
         prob.run_model()
 
-        res1 = -model.p1._residuals.get_data()[0]
-        out1 = model.p1._outputs.get_data()[0]
-        out2 = model.p2._outputs.get_data()[0]
+        res1 = -model.p1._residuals._data[0]
+        out1 = model.p1._outputs._data[0]
+        out2 = model.p2._outputs._data[0]
 
         self.assertEqual(res1, (out1 - 2.0*(out2 + 1.0)))
         with model._scaled_context_all():
-            res1a = -model.p1._residuals.get_data()[0]
+            res1a = -model.p1._residuals._data[0]
 
             self.assertEqual(res1a, (res1)/(ref))
 
@@ -451,13 +451,13 @@ class TestScaling(unittest.TestCase):
         prob.setup(check=False)
         prob.run_model()
 
-        res1 = -model.p1._residuals.get_data()[0]
-        out1 = model.p1._outputs.get_data()[0]
-        out2 = model.p2._outputs.get_data()[0]
+        res1 = -model.p1._residuals._data[0]
+        out1 = model.p1._outputs._data[0]
+        out2 = model.p2._outputs._data[0]
 
         self.assertEqual(res1, out1 - 2.0*(out2+1.0))
         with model._scaled_context_all():
-            res1a = -model.p1._residuals.get_data()[0]
+            res1a = -model.p1._residuals._data[0]
 
             self.assertEqual(res1a, res1/res_ref)
 
@@ -487,13 +487,13 @@ class TestScaling(unittest.TestCase):
         prob.setup(check=False)
         prob.run_model()
 
-        res1 = -model.p1._residuals.get_data()[0]
-        out1 = model.p1._outputs.get_data()[0]
-        out2 = model.p2._outputs.get_data()[0]
+        res1 = -model.p1._residuals._data[0]
+        out1 = model.p1._outputs._data[0]
+        out2 = model.p2._outputs._data[0]
 
         self.assertEqual(res1, out1 - 2.0*(out2+1.0))
         with model._scaled_context_all():
-            res1a = -model.p1._residuals.get_data()[0]
+            res1a = -model.p1._residuals._data[0]
 
             self.assertEqual(res1a, (res1)/(res_ref))
 
@@ -920,9 +920,9 @@ class TestScaling(unittest.TestCase):
 
         with model._scaled_context_all():
             val = model.comp._outputs['y1']
-            assert_rel_error(self, val, 200.0)
+            assert_rel_error(self, val, 0.5)
             val = model.comp._outputs['y2']
-            assert_rel_error(self, val, 6000.0)
+            assert_rel_error(self, val, 0.5)
 
     def test_feature3(self):
         from openmdao.api import Problem, Group, IndepVarComp
