@@ -22,49 +22,6 @@ for mod_name in MOCK_MODULES:
 
 # start off running the monkeypatch to keep options/parameters
 # usable in docstring for autodoc.
-
-
-def __init__(self, docstring, config={}):
-    """
-    init
-    """
-    docstring = textwrap.dedent(docstring).split('\n')
-
-    self._doc = Reader(docstring)
-    self._parsed_data = {
-        'Signature': '',
-        'Summary': [''],
-        'Extended Summary': [],
-        'Parameters': [],
-        'Options': [],
-        'Returns': [],
-        'Yields': [],
-        'Raises': [],
-        'Warns': [],
-        'Other Parameters': [],
-        'Attributes': [],
-        'Methods': [],
-        'See Also': [],
-        'Notes': [],
-        'Warnings': [],
-        'References': '',
-        'Examples': '',
-        'index': {}
-    }
-
-    try:
-        self._parse()
-    except ParseError as e:
-        e.docstring = orig_docstring
-        raise
-
-    # In creation of docs, remove private Attributes (beginning with '_')
-    # with a crazy list comprehension
-    self._parsed_data["Attributes"][:] = [att for att in self._parsed_data["Attributes"]
-                                          if not att[0].startswith('_')]
-
-NumpyDocString.__init__ = __init__
-
 do_monkeypatch()
 
 # If extensions (or modules to document with autodoc) are in another directory,
