@@ -12,6 +12,7 @@ from six.moves import range
 import numpy as np
 from scipy.optimize import minimize
 
+import openmdao
 from openmdao.core.driver import Driver, RecordingDebugging
 from openmdao.utils.general_utils import warn_deprecation
 import openmdao.utils.coloring as coloring_mod
@@ -67,18 +68,9 @@ class ScipyOptimizeDriver(Driver):
         Cached result of constraint evaluations because scipy asks for them in a separate function.
     _con_idx : dict
         Used for constraint bookkeeping in the presence of 2-sided constraints.
-    _cons : dict
-        Contains all constraint info.
-    _designvars : dict
-        Contains all design variable info.
     _grad_cache : OrderedDict
         Cached result of nonlinear constraint derivatives because scipy asks for them in a separate
         function.
-    _lincon_grad_cache : OrderedDict
-        Cached result of linear constraint derivatives because scipy asks for them in a separate
-        function.
-    _objs : dict
-        Contains all objective info.
     _exc_info : 3 item tuple
         Storage for exception and traceback information.
     _obj_and_nlcons : list
@@ -115,7 +107,6 @@ class ScipyOptimizeDriver(Driver):
         self.result = None
         self.fail = 0
         self._grad_cache = None
-        self._lincon_grad_cache = None
         self._con_cache = None
         self._con_idx = {}
         self._obj_and_nlcons = None
@@ -174,6 +165,11 @@ class ScipyOptimizeDriver(Driver):
         self.supports['two_sided_constraints'] = opt in _constraint_optimizers
         self.supports['equality_constraints'] = opt in _eq_constraint_optimizers
 
+        # Raises error if multiple objectives are not supported, but more objectives were defined.
+        if not self.supports['multiple_objectives'] and len(self._objs) > 1:
+            msg = '{} currently does not support multiple objectives.'
+            raise RuntimeError(msg.format(self.__class__.__name__))
+
         # Since COBYLA does not support bounds, we
         #   need to add to the _cons metadata for any bounds that
         #   need to be translated into a constraint
@@ -181,8 +177,8 @@ class ScipyOptimizeDriver(Driver):
             for name, meta in iteritems(self._designvars):
                 lower = meta['lower']
                 upper = meta['upper']
-                if isinstance(lower, np.ndarray) or lower != - sys.float_info.max \
-                        or isinstance(upper, np.ndarray) or upper != sys.float_info.max:
+                if isinstance(lower, np.ndarray) or lower >= -openmdao.INF_BOUND \
+                        or isinstance(upper, np.ndarray) or upper <= openmdao.INF_BOUND:
                     d = OrderedDict()
                     d['lower'] = lower
                     d['upper'] = upper
@@ -209,7 +205,9 @@ class ScipyOptimizeDriver(Driver):
         self._total_jac = None
 
         # Initial Run
-        model._solve_nonlinear()
+        with RecordingDebugging(self.options['optimizer'], self.iter_count, self) as rec:
+            model._solve_nonlinear()
+            self.iter_count += 1
 
         self._con_cache = self.get_constraint_values()
         desvar_vals = self.get_design_var_values()
@@ -296,7 +294,7 @@ class ScipyOptimizeDriver(Driver):
                     if isinstance(lower, np.ndarray):
                         lower = lower[j]
 
-                    dblcon = (upper < sys.float_info.max) and (lower > -sys.float_info.max)
+                    dblcon = (upper < openmdao.INF_BOUND) and (lower > -openmdao.INF_BOUND)
 
                     # Add extra constraint if double-sided
                     if dblcon:
@@ -323,7 +321,7 @@ class ScipyOptimizeDriver(Driver):
 
         # compute dynamic simul deriv coloring if option is set
         if coloring_mod._use_sparsity and self.options['dynamic_simul_derivs']:
-            coloring_mod.dynamic_simul_coloring(self)
+            coloring_mod.dynamic_simul_coloring(self, run_model=False, do_sparsity=False)
 
         # optimize
         try:
@@ -458,7 +456,7 @@ class ScipyOptimizeDriver(Driver):
         if isinstance(lower, np.ndarray):
             lower = lower[idx]
 
-        if dbl or (lower == -sys.float_info.max):
+        if dbl or (lower <= -openmdao.INF_BOUND):
             return upper - cons[name][idx]
         else:
             return cons[name][idx] - lower
@@ -542,7 +540,7 @@ class ScipyOptimizeDriver(Driver):
         if isinstance(lower, np.ndarray):
             lower = lower[idx]
 
-        if dbl or (lower == -sys.float_info.max):
+        if dbl or (lower <= -openmdao.INF_BOUND):
             return -grad[grad_idx, :]
         else:
             return grad[grad_idx, :]
