@@ -1,6 +1,7 @@
 """Define the COOmatrix class."""
 from __future__ import division, print_function
 
+from collections import Counter, defaultdict
 import numpy as np
 from numpy import ndarray
 from scipy.sparse import coo_matrix
@@ -82,8 +83,8 @@ class COOMatrix(Matrix):
                                        "CSR partial jacobian and at least one of them is either "
                                        "not dense or uses src_indices.  This can occur when "
                                        "multiple inputs on the same "
-                                       "component are connected to the same output." %
-                                       sorted((key, otherkey)))
+                                       "component are connected to the same output. Try using "
+                                       "a dense jacobian instead." % sorted((key, otherkey)))
             else:
                 ind1 = counter
                 counter += delta
@@ -145,7 +146,7 @@ class COOMatrix(Matrix):
 
         return data, rows, cols
 
-    def _build(self, num_rows, num_cols):
+    def _build(self, num_rows, num_cols, in_ranges, out_ranges):
         """
         Allocate the matrix.
 
@@ -155,6 +156,10 @@ class COOMatrix(Matrix):
             number of rows in the matrix.
         num_cols : int
             number of cols in the matrix.
+        in_ranges : dict
+            Maps input var name to column range.
+        out_ranges : dict
+            Maps output var name to row range.
         """
         data, rows, cols = self._build_sparse(num_rows, num_cols)
 
@@ -213,14 +218,17 @@ class COOMatrix(Matrix):
                                                                   type(jac).__name__,
                                                                   jac_type.__name__))
         if isinstance(jac, ndarray):
-            val = jac.flatten()
+            if factor is None:
+                val = jac.flat
+            else:
+                val = (jac * factor).flat
         else:  # sparse
-            val = jac.data
+            if factor is None:
+                val = jac.data
+            else:
+                val = jac.data * factor
 
-        if factor is not None:
-            self._matrix.data[idxs] += val * factor
-        else:
-            self._matrix.data[idxs] += val
+        self._matrix.data[idxs] += val
 
     def _prod(self, in_vec, mode, ranges, mask=None):
         """
@@ -322,3 +330,28 @@ class COOMatrix(Matrix):
                     mask[ind1:ind2] = False
 
             return mask
+
+
+def _get_dup_partials(rows, cols, in_ranges, out_ranges):
+    counts = Counter((rows[i], cols[i]) for i in range(len(rows)))
+    dups = []
+    for entry, count in counts.most_common():
+        if count > 1:
+            dups.append(entry)
+        else:
+            break
+
+    entries = defaultdict(list)
+
+    for row, col in dups:
+        for in_name in in_ranges:
+            cstart, cend = in_ranges[in_name]
+            if cstart <= col < cend:
+                break
+        for out_name in out_ranges:
+            rstart, rend = out_ranges[out_name]
+            if rstart <= row < rend:
+                break
+        entries[(out_name, in_name)].append((row - rstart, col - cstart))
+
+    return entries
