@@ -12,7 +12,7 @@ class DenseMatrix(Matrix):
     Dense global matrix.
     """
 
-    def _build(self, num_rows, num_cols):
+    def _build(self, num_rows, num_cols, in_ranges, out_ranges):
         """
         Allocate the matrix.
 
@@ -22,6 +22,10 @@ class DenseMatrix(Matrix):
             number of rows in the matrix.
         num_cols : int
             number of cols in the matrix.
+        in_ranges : dict
+            Maps input var name to column range.
+        out_ranges : dict
+            Maps output var name to row range.
         """
         self._matrix = np.zeros((num_rows, num_cols))
         submats = self._submats
