@@ -65,7 +65,7 @@ class Matrix(object):
         """
         self._submats[key] = (info, (irow, icol), src_indices, shape, factor)
 
-    def _build(self, num_rows, num_cols):
+    def _build(self, num_rows, num_cols, in_ranges, out_ranges):
         """
         Allocate the matrix.
 
@@ -75,6 +75,10 @@ class Matrix(object):
             number of rows in the matrix.
         num_cols : int
             number of cols in the matrix.
+        in_ranges : dict
+            Maps input var name to column range.
+        out_ranges : dict
+            Maps output var name to row range.
         """
         pass
 
