@@ -1,10 +1,5 @@
 from six import iteritems, PY2, PY3
 
-if PY2:
-    import cPickle as pickle
-if PY3:
-    import pickle
-
 import sqlite3
 import numpy as np
 import json
@@ -15,6 +10,11 @@ from openmdao.utils.record_util import format_iteration_coordinate, json_to_np_a
 from openmdao.utils.assert_utils import assert_rel_error
 from openmdao.recorders.sqlite_recorder import blob_to_array, format_version
 
+if PY2:
+    import cPickle as pickle
+else:
+    import pickle
+
 
 @contextmanager
 def database_cursor(filename):
@@ -29,13 +29,38 @@ def database_cursor(filename):
     con.close()
 
 
+def get_format_version_abs2meta(db_cur):
+    """
+        Return the format version and abs2meta dict from metadata table in the case recorder file.
+    """
+    db_cur.execute("SELECT format_version, abs2meta FROM metadata")
+    row = db_cur.fetchone()
+
+    f_version = row[0]
+
+    # Need to also get abs2meta so that we can pass it to json_to_np_array
+    if f_version >= 3:
+        abs2meta = json.loads(row[1])
+    elif f_version in (1, 2):
+        if PY2:
+            abs2meta = pickle.loads(str(row[1])) if row[1] is not None else None
+
+        if PY3:
+            try:
+                abs2meta = pickle.loads(row[1]) if row[1] is not None else None
+            except TypeError:
+                # Reading in a python 2 pickle recorded pre-OpenMDAO 2.4.
+                abs2meta = pickle.loads(row[1].encode()) if row[1] is not None else None
+
+    return f_version, abs2meta
+
+
 def assertDriverIterDataRecorded(test, expected, tolerance, prefix=None):
     """
     Expected can be from multiple cases.
     """
     with database_cursor(test.filename) as db_cur:
-        db_cur.execute("SELECT format_version FROM metadata")
-        f_version = db_cur.fetchone()[0]
+        f_version, abs2meta = get_format_version_abs2meta(db_cur)
 
         # iterate through the cases
         for coord, (t0, t1), outputs_expected, inputs_expected in expected:
@@ -54,8 +79,8 @@ def assertDriverIterDataRecorded(test, expected, tolerance, prefix=None):
                 inputs_text, outputs_text = row_actual
 
             if f_version >= 3:
-                inputs_actual = json_to_np_array(inputs_text)
-                outputs_actual = json_to_np_array(outputs_text)
+                inputs_actual = json_to_np_array(inputs_text, abs2meta)
+                outputs_actual = json_to_np_array(outputs_text, abs2meta)
             elif f_version in (1, 2):
                 inputs_actual = blob_to_array(inputs_text)
                 outputs_actual = blob_to_array(outputs_text)
@@ -72,7 +97,7 @@ def assertDriverIterDataRecorded(test, expected, tolerance, prefix=None):
             ):
 
                 if expected is None:
-                    if f_version == 3:
+                    if f_version >= 3:
                         test.assertIsNone(actual)
                     if f_version in (1, 2):
                         test.assertEqual(actual, np.array(None, dtype=object))
@@ -116,6 +141,7 @@ def assertDriverDerivDataRecorded(test, expected, tolerance, prefix=None):
             counter, global_counter, iteration_coordinate, timestamp, success, msg,\
                 totals_blob = row_actual
             abs2meta = json.loads(row_abs2meta[0]) if row_abs2meta[0] is not None else None
+            test.assertTrue(isinstance(abs2meta, dict))
 
             totals_actual = blob_to_array(totals_blob)
 
@@ -145,8 +171,7 @@ def assertSystemIterDataRecorded(test, expected, tolerance, prefix=None):
         Expected can be from multiple cases.
     """
     with database_cursor(test.filename) as db_cur:
-        db_cur.execute("SELECT format_version FROM metadata")
-        f_version = db_cur.fetchone()[0]
+        f_version, abs2meta = get_format_version_abs2meta(db_cur)
 
         # iterate through the cases
         for coord, (t0, t1), inputs_expected, outputs_expected, residuals_expected in expected:
@@ -164,9 +189,9 @@ def assertSystemIterDataRecorded(test, expected, tolerance, prefix=None):
                 outputs_text, residuals_text = row_actual
 
             if f_version >= 3:
-                inputs_actual = json_to_np_array(inputs_text)
-                outputs_actual = json_to_np_array(outputs_text)
-                residuals_actual = json_to_np_array(residuals_text)
+                inputs_actual = json_to_np_array(inputs_text, abs2meta)
+                outputs_actual = json_to_np_array(outputs_text, abs2meta)
+                residuals_actual = json_to_np_array(residuals_text, abs2meta)
             elif f_version in (1, 2):
                 inputs_actual = blob_to_array(inputs_text)
                 outputs_actual = blob_to_array(outputs_text)
@@ -185,7 +210,7 @@ def assertSystemIterDataRecorded(test, expected, tolerance, prefix=None):
             ):
 
                 if expected is None:
-                    if f_version == 3:
+                    if f_version >= 3:
                         test.assertIsNone(actual)
                     if f_version in (1, 2):
                         test.assertEqual(actual, np.array(None, dtype=object))
@@ -206,8 +231,7 @@ def assertSolverIterDataRecorded(test, expected, tolerance, prefix=None):
         Expected can be from multiple cases.
     """
     with database_cursor(test.filename) as db_cur:
-        db_cur.execute("SELECT format_version FROM metadata")
-        f_version = db_cur.fetchone()[0]
+        f_version, abs2meta = get_format_version_abs2meta(db_cur)
 
         # iterate through the cases
         for coord, (t0, t1), expected_abs_error, expected_rel_error, expected_output, \
@@ -216,25 +240,26 @@ def assertSolverIterDataRecorded(test, expected, tolerance, prefix=None):
             iter_coord = format_iteration_coordinate(coord, prefix=prefix)
 
             # from the database, get the actual data recorded
-            db_cur.execute("SELECT * FROM solver_iterations WHERE iteration_coordinate=:iteration_coordinate",
+            db_cur.execute("SELECT * FROM solver_iterations "
+                           "WHERE iteration_coordinate=:iteration_coordinate",
                            {"iteration_coordinate": iter_coord})
             row_actual = db_cur.fetchone()
             test.assertTrue(row_actual, 'Solver iterations table does not contain the requested '
                                         'iteration coordinate: "{}"'.format(iter_coord))
 
-            counter, global_counter, iteration_coordinate, timestamp, success, msg, abs_err, rel_err, \
-                input_blob, output_text, residuals_text = row_actual
+            counter, global_counter, iteration_coordinate, timestamp, success, msg, \
+                abs_err, rel_err, input_blob, output_text, residuals_text = row_actual
 
             if f_version >= 3:
-                output_actual = json_to_np_array(output_text)
-                residuals_actual = json_to_np_array(residuals_text)
+                output_actual = json_to_np_array(output_text, abs2meta)
+                residuals_actual = json_to_np_array(residuals_text, abs2meta)
             elif f_version in (1, 2):
                 output_actual = blob_to_array(output_text)
                 residuals_actual = blob_to_array(residuals_text)
 
             # Does the timestamp make sense?
-            test.assertTrue(t0 <= timestamp and timestamp <= t1, 'timestamp should be between when the model '
-                                                                 'started and stopped')
+            test.assertTrue(t0 <= timestamp and timestamp <= t1,
+                            'timestamp should be between when the model started and stopped')
 
             test.assertEqual(success, 1)
             test.assertEqual(msg, '')
@@ -251,7 +276,7 @@ def assertSolverIterDataRecorded(test, expected, tolerance, prefix=None):
             ):
 
                 if expected is None:
-                    if f_version == 3:
+                    if f_version >= 3:
                         test.assertIsNone(actual)
                     if f_version in (1, 2):
                         test.assertEqual(actual, np.array(None, dtype=object))
@@ -260,8 +285,9 @@ def assertSolverIterDataRecorded(test, expected, tolerance, prefix=None):
                     test.assertEqual(len(actual[0]), len(expected))
                     for key, value in iteritems(expected):
                         # Check to see if the keys in the actual and expected match
-                        test.assertTrue(key in actual[0].dtype.names, '{} variable not found in actual '
-                                                                      'data from recorder'.format(key))
+                        test.assertTrue(key in actual[0].dtype.names,
+                                        '{} variable not found in actual data '
+                                        'from recorder'.format(key))
                         # Check to see if the values in actual and expected match
                         assert_rel_error(test, actual[0][key], expected[key], tolerance)
 
@@ -296,28 +322,25 @@ def assertMetadataRecorded(test, expected_prom2abs, expected_abs2prom):
         test.assertEqual(format_version_actual, format_version_expected)
 
 
-def assertDriverMetadataRecorded(test, expected, expect_none_viewer_data=False):
+def assertViewerDataRecorded(test, expected):
 
     with database_cursor(test.filename) as db_cur:
         db_cur.execute("SELECT format_version FROM metadata")
         f_version = db_cur.fetchone()[0]
+        test.assertTrue(isinstance(f_version, int))
 
         db_cur.execute("SELECT model_viewer_data FROM driver_metadata")
         row = db_cur.fetchone()
 
-        if expected is None and not expect_none_viewer_data:
+        if expected is None:
             test.assertIsNone(row)
             return
 
         model_viewer_data = json.loads(row[0])
 
-        if expect_none_viewer_data:
-            test.assertIsNone(model_viewer_data)
-            return
-
         test.assertTrue(isinstance(model_viewer_data, dict))
 
-        test.assertEqual(2, len(model_viewer_data))
+        test.assertEqual(3, len(model_viewer_data))
 
         test.assertTrue(isinstance(model_viewer_data['connections_list'], list))
 
@@ -327,13 +350,20 @@ def assertDriverMetadataRecorded(test, expected, expect_none_viewer_data=False):
         test.assertEqual(expected['tree_length'], len(model_viewer_data['tree']))
 
         tr = model_viewer_data['tree']
-        test.assertEqual(set(['name', 'type', 'subsystem_type', 'children']), set(tr.keys()))
-        test.assertEqual(expected['tree_children_length'], len(model_viewer_data['tree']['children']))
+        test.assertEqual(set(['name', 'type', 'subsystem_type', 'children']),
+                         set(tr.keys()))
+        test.assertEqual(expected['tree_children_length'],
+                         len(model_viewer_data['tree']['children']))
 
         cl = model_viewer_data['connections_list']
         for c in cl:
             test.assertTrue(set(c.keys()).issubset(set(['src', 'tgt', 'cycle_arrows'])))
 
+        abs2prom = model_viewer_data['abs2prom']
+        for io in ['input', 'output']:
+            for var in expected['abs2prom'][io]:
+                test.assertEqual(abs2prom[io][var], expected['abs2prom'][io][var])
+
 
 def assertSystemMetadataIdsRecorded(test, ids):
 
