@@ -61,8 +61,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'fwd')
 
         output = d_outputs._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 1e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 1e-15)
+        assert_rel_error(self, output, group.expected_solution, 1e-15)
 
         # reverse
         d_outputs.set_const(1.0)
@@ -70,13 +69,12 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'rev')
 
         output = d_residuals._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 1e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 1e-15)
+        assert_rel_error(self, output, group.expected_solution, 1e-15)
 
     def test_solve_linear_ksp_gmres(self):
         """Solve implicit system with PETScKrylov using 'gmres' method."""
 
-        group = TestImplicitGroup(lnSolverClass=PETScKrylov, use_varsets=False)
+        group = TestImplicitGroup(lnSolverClass=PETScKrylov)
         group.linear_solver.options['ksp_type'] = 'gmres'
 
         p = Problem(group)
@@ -94,7 +92,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'fwd')
 
         output = d_outputs._data
-        assert_rel_error(self, output[0], group.expected_solution[0], 1e-15)
+        assert_rel_error(self, output, group.expected_solution, 1e-15)
 
         # reverse
         d_outputs.set_const(1.0)
@@ -102,7 +100,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'rev')
 
         output = d_residuals._data
-        assert_rel_error(self, output[0], group.expected_solution[0], 1e-15)
+        assert_rel_error(self, output, group.expected_solution, 1e-15)
 
     def test_solve_linear_ksp_maxiter(self):
         """Verify that PETScKrylov abides by the 'maxiter' option."""
@@ -154,8 +152,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'fwd')
 
         output = d_outputs._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 1e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 1e-15)
+        assert_rel_error(self, output, group.expected_solution, 1e-15)
 
         self.assertTrue(precon._iter_count > 0)
 
@@ -165,8 +162,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'rev')
 
         output = d_residuals._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 3e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 3e-15)
+        assert_rel_error(self, output, group.expected_solution, 3e-15)
 
         self.assertTrue(precon._iter_count > 0)
 
@@ -186,8 +182,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'fwd')
 
         output = d_outputs._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 1e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 1e-15)
+        assert_rel_error(self, output, group.expected_solution, 1e-15)
 
         # reverse
         d_outputs.set_const(1.0)
@@ -196,8 +191,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'rev')
 
         output = d_residuals._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 3e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 3e-15)
+        assert_rel_error(self, output, group.expected_solution, 3e-15)
 
     def test_solve_linear_ksp_precon_left(self):
         """Solve implicit system with PETScKrylov using a preconditioner."""
@@ -223,8 +217,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'fwd')
 
         output = d_outputs._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 1e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 1e-15)
+        assert_rel_error(self, output, group.expected_solution, 1e-15)
 
         # reverse
         d_outputs.set_const(1.0)
@@ -233,8 +226,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'rev')
 
         output = d_residuals._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 3e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 3e-15)
+        assert_rel_error(self, output, group.expected_solution, 3e-15)
 
         # test the direct solver and make sure KSP correctly recurses for _linearize
         precon = group.linear_solver.precon = DirectSolver()
@@ -255,8 +247,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'fwd')
 
         output = d_outputs._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 1e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 1e-15)
+        assert_rel_error(self, output, group.expected_solution, 1e-15)
 
         # reverse
         d_outputs.set_const(1.0)
@@ -265,8 +256,7 @@ class TestPETScKrylov(unittest.TestCase):
         group.run_solve_linear(['linear'], 'rev')
 
         output = d_residuals._data
-        assert_rel_error(self, output[1], group.expected_solution[0], 3e-15)
-        assert_rel_error(self, output[5], group.expected_solution[1], 3e-15)
+        assert_rel_error(self, output, group.expected_solution, 3e-15)
 
     def test_preconditioner_deprecation(self):
 
@@ -317,10 +307,7 @@ class TestPETScKrylov(unittest.TestCase):
         g1.run_solve_linear(['linear'], 'fwd')
 
         output = d_outputs._data
-        # The empty first entry in _data is due to the dummy
-        #     variable being in a different variable set not owned by g1
-        assert_rel_error(self, output[1], g1.expected_solution[0], 1e-15)
-        assert_rel_error(self, output[5], g1.expected_solution[1], 1e-15)
+        assert_rel_error(self, output, g1.expected_solution, 1e-15)
 
         # reverse
         d_inputs, d_outputs, d_residuals = g1.get_linear_vectors()
@@ -331,8 +318,7 @@ class TestPETScKrylov(unittest.TestCase):
         g1.run_solve_linear(['linear'], 'rev')
 
         output = d_residuals._data
-        assert_rel_error(self, output[1], g1.expected_solution[0], 3e-15)
-        assert_rel_error(self, output[5], g1.expected_solution[1], 3e-15)
+        assert_rel_error(self, output, g1.expected_solution, 3e-15)
 
     def test_linear_solution_cache(self):
         # Test derivatives across a converged Sellar model. When caching
@@ -395,6 +381,39 @@ class TestPETScKrylov(unittest.TestCase):
         # Should take less iterations when starting from previous solution.
         self.assertTrue(icount2 < icount1)
 
+    def test_error_under_cs(self):
+        """Verify that PETScKrylov abides by the 'maxiter' option."""
+        prob = Problem()
+        model = prob.model = Group()
+
+        model.add_subsystem('px', IndepVarComp('x', 1.0), promotes=['x'])
+        model.add_subsystem('pz', IndepVarComp('z', np.array([5.0, 2.0])), promotes=['z'])
+
+        model.add_subsystem('d1', SellarDis1withDerivatives(), promotes=['x', 'z', 'y1', 'y2'])
+        model.add_subsystem('d2', SellarDis2withDerivatives(), promotes=['z', 'y1', 'y2'])
+
+        model.add_subsystem('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
+                                                z=np.array([0.0, 0.0]), x=0.0),
+                            promotes=['obj', 'x', 'z', 'y1', 'y2'])
+
+        model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
+        model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
+
+        model.nonlinear_solver = NewtonSolver()
+        model.linear_solver = PETScKrylov()
+
+        model.approx_totals(method='cs')
+
+        prob.setup(mode='fwd')
+        prob.set_solver_print(level=0)
+        prob.run_model()
+
+        with self.assertRaises(RuntimeError) as cm:
+            J = prob.compute_totals(of=['obj'], wrt=['z'])
+
+        msg = 'PETScKrylov solver is not supported under complex step.'
+        self.assertEqual(str(cm.exception), msg)
+
 
 @unittest.skipUnless(PETScVector, "PETSc is required.")
 class TestPETScKrylovSolverFeature(unittest.TestCase):
