@@ -13,6 +13,7 @@ import numpy as np
 from openmdao.recorders.recording_iteration_stack import Recording
 from openmdao.solvers.solver import NonlinearSolver
 from openmdao.utils.class_util import overrides_method
+from openmdao.utils.general_utils import simple_warning
 
 CITATION = """@ARTICLE{
               Broyden1965ACo,
@@ -114,6 +115,9 @@ class BroydenSolver(NonlinearSolver):
                                   "convergence is considered a failure. The Jacobian will be "
                                   "regenerated once this condition has been reached a number of "
                                   "consecutive times as specified in max_converge_failures.")
+        self.options.declare('cs_reconverge', default=True,
+                             desc='When True, when this driver solves under a complex step, nudge '
+                             'the Solution vector by a small amount so that it reconverges.')
         self.options.declare('diverge_limit', default=2.0,
                              desc="Ratio of current residual to previous residual above which the "
                                   "Jacobian will be immediately regenerated.")
@@ -147,6 +151,8 @@ class BroydenSolver(NonlinearSolver):
         self._recompute_jacobian = True
         self._computed_jacobians = 0
 
+        self._disallow_discrete_outputs()
+
         if self.linear_solver is not None:
             self.linear_solver._setup_solvers(self._system, self._depth + 1)
         else:
@@ -176,7 +182,7 @@ class BroydenSolver(NonlinearSolver):
 
         else:
             self._full_inverse = True
-            n = len(outputs.get_data())
+            n = len(outputs._data)
 
         self.n = n
         self.Gm = np.empty((n, n))
@@ -223,7 +229,7 @@ class BroydenSolver(NonlinearSolver):
             msg = "The following states are not covered by a solver, and may have been " + \
                   "omitted from the BroydenSolver 'state_vars': "
             msg += ', '.join(sorted(missing))
-            warnings.warn(msg)
+            simple_warning(msg)
 
     def _assembled_jac_solver_iter(self):
         """
@@ -275,17 +281,34 @@ class BroydenSolver(NonlinearSolver):
         float
             Initial absolute error in the user-specified residuals.
         """
+        system = self._system
         if self.options['debug_print']:
             self._err_cache['inputs'] = deepcopy(self._system._inputs)
             self._err_cache['outputs'] = deepcopy(self._system._outputs)
 
-        system = self._system
+        # Convert local storage if we are under complex step.
+        if system.under_complex_step:
+            if np.iscomplex(self.xm[0]):
+                self.Gm = self.Gm.astype(np.complex)
+                self.xm = self.xm.astype(np.complex)
+                self.fxm = self.fxm.astype(np.complex)
+        elif np.iscomplex(self.xm[0]):
+            self.Gm = self.Gm.real
+            self.xm = self.xm.real
+            self.fxm = self.fxm.real
+
         self._converge_failures = 0
         self._computed_jacobians = 0
 
         # Execute guess_nonlinear if specified.
         system._guess_nonlinear()
 
+        # When under a complex step from higher in the hierarchy, sometimes the step is too small
+        # to trigger reconvergence, so nudge the outputs slightly so that we always get at least
+        # one iteration of Broyden.
+        if system.under_complex_step and self.options['cs_reconverge']:
+            system._outputs._data += np.linalg.norm(self._system._outputs._data) * 1e-10
+
         # Start with initial states.
         self.xm = self.get_states()
 
@@ -320,9 +343,12 @@ class BroydenSolver(NonlinearSolver):
         fxm = self.get_residuals()
         if not self._full_inverse:
             # Use full model residual for driving the main loop convergence.
-            residuals = self._system._residuals
-            fxm = residuals.get_data()
-        return np.sum(fxm**2) ** 0.5
+            fxm = self._system._residuals._data
+
+        if self._system.under_complex_step:
+            return (np.sum(fxm.real**2) + np.sum(fxm.imag**2))**0.5
+        else:
+            return np.sum(fxm**2) ** 0.5
 
     def _iter_execute(self):
         """
@@ -426,7 +452,7 @@ class BroydenSolver(NonlinearSolver):
         # Set inverse Jacobian to identity scaled by alpha.
         # This is the default starting point used by scipy and the general broyden algorithm.
         else:
-            Gm = np.diag(np.full(self.n, -self.options['alpha']))
+            Gm = np.diag(np.full(self.n, -self.options['alpha'], dtype=Gm.dtype))
 
         return Gm
 
@@ -444,8 +470,7 @@ class BroydenSolver(NonlinearSolver):
         outputs = self._system._outputs
 
         if self._full_inverse:
-            xm = outputs.get_data().copy()
-
+            xm = outputs._data.copy()
         else:
             states = self.options['state_vars']
             xm = self.xm.copy()
@@ -467,7 +492,7 @@ class BroydenSolver(NonlinearSolver):
         outputs = self._system._outputs
 
         if self._full_inverse:
-            outputs.set_data(new_val)
+            outputs._data[:] = new_val
 
         else:
             states = self.options['state_vars']
@@ -487,7 +512,7 @@ class BroydenSolver(NonlinearSolver):
         residuals = self._system._residuals
 
         if self._full_inverse:
-            fxm = self.fxm = residuals.get_data()
+            fxm = self.fxm = residuals._data.copy()
 
         else:
             states = self.options['state_vars']
@@ -509,7 +534,7 @@ class BroydenSolver(NonlinearSolver):
         """
         linear = self._system._vectors['output']['linear']
         if self._full_inverse:
-            linear.set_data(dx)
+            linear._data[:] = dx
 
         else:
             linear.set_const(0.0)
@@ -547,7 +572,7 @@ class BroydenSolver(NonlinearSolver):
         do_sub_ln = ln_solver._linearize_children()
         my_asm_jac = ln_solver._assembled_jac
         system._linearize(my_asm_jac, sub_do_ln=do_sub_ln)
-        if my_asm_jac is not None and ln_solver._assembled_jac is not my_asm_jac:
+        if my_asm_jac is not None and system.linear_solver._assembled_jac is not my_asm_jac:
             my_asm_jac._update(system)
         self._linearize()
 
@@ -596,7 +621,7 @@ class BroydenSolver(NonlinearSolver):
         do_sub_ln = ln_solver._linearize_children()
         my_asm_jac = ln_solver._assembled_jac
         system._linearize(my_asm_jac, sub_do_ln=do_sub_ln)
-        if my_asm_jac is not None and ln_solver._assembled_jac is not my_asm_jac:
+        if my_asm_jac is not None and system.linear_solver._assembled_jac is not my_asm_jac:
             my_asm_jac._update(system)
 
         inv_jac = self.linear_solver._inverse()
