@@ -17,6 +17,7 @@ from openmdao.test_suite.components.sellar import SellarDerivativesGrouped, \
 from openmdao.solvers.linesearch.backtracking import ArmijoGoldsteinLS
 from openmdao.test_suite.components.implicit_newton_linesearch \
     import ImplCompTwoStates, ImplCompTwoStatesArrays
+from openmdao.core.tests.test_discrete import InternalDiscreteGroup
 
 class TestNewton(unittest.TestCase):
 
@@ -588,7 +589,7 @@ class TestNewton(unittest.TestCase):
             """ This version of Newton also counts how many times it linearizes"""
 
             def __init__(self, **kwargs):
-                super(DirectSolver, self).__init__(**kwargs)
+                super(CountDS, self).__init__(**kwargs)
                 self.lin_count = 0
 
             def _linearize(self):
@@ -823,7 +824,7 @@ class TestNewtonFeatures(unittest.TestCase):
         model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        model.linear_solver = LinearBlockGS()
+        model.linear_solver = DirectSolver()
 
         model.nonlinear_solver = NewtonSolver()
 
@@ -856,7 +857,7 @@ class TestNewtonFeatures(unittest.TestCase):
         model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        model.linear_solver = LinearBlockGS()
+        model.linear_solver = DirectSolver()
 
         nlgbs = model.nonlinear_solver = NewtonSolver()
         nlgbs.options['maxiter'] = 2
@@ -890,7 +891,7 @@ class TestNewtonFeatures(unittest.TestCase):
         model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        model.linear_solver = LinearBlockGS()
+        model.linear_solver = DirectSolver()
 
         nlgbs = model.nonlinear_solver = NewtonSolver()
         nlgbs.options['rtol'] = 1e-3
@@ -924,7 +925,7 @@ class TestNewtonFeatures(unittest.TestCase):
         model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        model.linear_solver = LinearBlockGS()
+        model.linear_solver = DirectSolver()
 
         nlgbs = model.nonlinear_solver = NewtonSolver()
         nlgbs.options['atol'] = 1e-4
@@ -1033,7 +1034,7 @@ class TestNewtonFeatures(unittest.TestCase):
         model.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        model.linear_solver = LinearBlockGS()
+        model.linear_solver = DirectSolver()
 
         nlgbs = model.nonlinear_solver = NewtonSolver()
         nlgbs.options['maxiter'] = 1
