@@ -2,7 +2,7 @@
 
 from __future__ import division, print_function
 
-from six import iteritems
+from six import iteritems, reraise
 from collections import OrderedDict
 import os
 import pprint
@@ -14,13 +14,11 @@ import numpy as np
 from copy import deepcopy
 
 from openmdao.core.analysis_error import AnalysisError
-from openmdao.jacobians.assembled_jacobian import AssembledJacobian, DenseJacobian, CSCJacobian
-from openmdao.recorders.recording_iteration_stack import Recording, recording_iteration
+from openmdao.recorders.recording_iteration_stack import Recording
 from openmdao.recorders.recording_manager import RecordingManager
 from openmdao.utils.mpi import MPI
 from openmdao.utils.options_dictionary import OptionsDictionary
 from openmdao.utils.record_util import create_local_meta, check_path
-from openmdao.recorders.recording_iteration_stack import recording_iteration
 
 _emptyset = set()
 
@@ -127,8 +125,6 @@ class Solver(object):
         Number of iterations for the current invocation of the solver.
     _rec_mgr : <RecordingManager>
         object that manages all recorders added to this solver
-    _solver_info : <SolverInfo>
-        Object to store some formatting for iprint that is shared across all solvers.
     cite : str
         Listing of relevant citations that should be referenced when
         publishing work that uses this class.
@@ -143,10 +139,12 @@ class Solver(object):
         Dict of list of var names to record
     _norm0: float
         Normalization factor
+    _solver_info : SolverInfo
+        A stack-like object shared by all Solvers in the model.
     """
 
+    # Object to store some formatting for iprint that is shared across all solvers.
     SOLVER = 'base_solver'
-    _solver_info = SolverInfo()
 
     def __init__(self, **kwargs):
         """
@@ -162,6 +160,7 @@ class Solver(object):
         self._vec_names = None
         self._mode = 'fwd'
         self._iter_count = 0
+        self._solver_info = None
 
         # Solver options
         self.options = OptionsDictionary()
@@ -222,11 +221,11 @@ class Solver(object):
 
     def add_recorder(self, recorder):
         """
-        Add a recorder to the driver's RecordingManager.
+        Add a recorder to the solver's RecordingManager.
 
         Parameters
         ----------
-        recorder : <BaseRecorder>
+        recorder : <CaseRecorder>
            A recorder instance to be added to RecManager.
         """
         if MPI:
@@ -255,6 +254,12 @@ class Solver(object):
         """
         self._system = system
         self._depth = depth
+        self._solver_info = system._solver_info
+        self._recording_iter = system._recording_iter
+
+        if isinstance(self, LinearSolver) and not system._use_derivatives:
+            return
+
         self._rec_mgr.startup(self)
         self._rec_mgr.record_metadata(self)
 
@@ -264,25 +269,25 @@ class Solver(object):
 
         if self.recording_options['record_solver_residuals']:
             if isinstance(self, NonlinearSolver):
-                residuals = self._system._residuals
+                residuals = system._residuals
             else:  # it's a LinearSolver
-                residuals = self._system._vectors['residual']['linear']
+                residuals = system._vectors['residual']['linear']
 
             myresiduals = {n for n in residuals._names if check_path(n, incl, excl)}
 
         if self.recording_options['record_outputs']:
             if isinstance(self, NonlinearSolver):
-                outputs = self._system._outputs
+                outputs = system._outputs
             else:  # it's a LinearSolver
-                outputs = self._system._vectors['output']['linear']
+                outputs = system._vectors['output']['linear']
 
             myoutputs = {n for n in outputs._names if check_path(n, incl, excl)}
 
         if self.recording_options['record_inputs']:
             if isinstance(self, NonlinearSolver):
-                inputs = self._system._inputs
+                inputs = system._inputs
             else:
-                inputs = self._system._vectors['input']['linear']
+                inputs = system._vectors['input']['linear']
 
             myinputs = {n for n in inputs._names if check_path(n, incl, excl)}
 
@@ -514,14 +519,15 @@ class Solver(object):
         else:
             data['rel'] = None
 
+        system = self._system
         if isinstance(self, NonlinearSolver):
-            outputs = self._system._outputs
-            inputs = self._system._inputs
-            residuals = self._system._residuals
+            outputs = system._outputs
+            inputs = system._inputs
+            residuals = system._residuals
         else:  # it's a LinearSolver
-            outputs = self._system._vectors['output']['linear']
-            inputs = self._system._vectors['input']['linear']
-            residuals = self._system._vectors['residual']['linear']
+            outputs = system._vectors['output']['linear']
+            inputs = system._vectors['input']['linear']
+            residuals = system._vectors['residual']['linear']
 
         if self.recording_options['record_outputs']:
             data['o'] = {}
@@ -572,36 +578,8 @@ class NonlinearSolver(Solver):
 
     Attributes
     ----------
-    _system : <System>
-        Pointer to the owning system.
-    _depth : int
-        How many subsolvers deep this solver is (0 means not a subsolver).
-    _vec_names : [str, ...]
-        List of right-hand-side (RHS) vector names.
-    _mode : str
-        'fwd' or 'rev', applicable to linear solvers only.
-    _iter_count : int
-        Number of iterations for the current invocation of the solver.
-    _rec_mgr : <RecordingManager>
-        object that manages all recorders added to this solver
-    _solver_info : <SolverInfo>
-        Object to store some formatting for iprint that is shared across all solvers.
     _err_cache : dict
         Dictionary holding input and output vectors at start of iteration, if requested.
-    cite : str
-        Listing of relevant citations that should be referenced when
-        publishing work that uses this class.
-    options : <OptionsDictionary>
-        Options dictionary.
-    recording_options : <OptionsDictionary>
-        Recording options dictionary.
-    supports : <OptionsDictionary>
-        Options dictionary describing what features are supported by this
-        solver.
-    _filtered_vars_to_record: Dict
-        Dict of list of var names to record
-    _norm0: float
-        Normalization factor
     """
 
     def __init__(self, **kwargs):
@@ -638,10 +616,16 @@ class NonlinearSolver(Solver):
         float
             relative error.
         """
-        fail, abs_err, rel_err = self._run_iterator()
+        raised = False
+        try:
+            fail, abs_err, rel_err = self._run_iterator()
+        except Exception:
+            fail = True
+            raised = True
+            exc = sys.exc_info()
 
         if fail and self.options['debug_print']:
-            coord = recording_iteration.get_formatted_iteration_coordinate()
+            coord = self._recording_iter.get_formatted_iteration_coordinate()
 
             out_str = "\n# Inputs and outputs at start of iteration '%s':\n" % coord
             for vec_type, vec in iteritems(self._err_cache):
@@ -660,6 +644,9 @@ class NonlinearSolver(Solver):
                       "saved to '%s'." % filename)
                 sys.stdout.flush()
 
+        if raised:
+            reraise(*exc)
+
         return fail, abs_err, rel_err
 
     def _iter_initialize(self):
@@ -689,11 +676,11 @@ class NonlinearSolver(Solver):
         """
         Run the apply_nonlinear method on the system.
         """
-        recording_iteration.stack.append(('_run_apply', 0))
+        self._recording_iter.stack.append(('_run_apply', 0))
         try:
             self._system._apply_nonlinear()
         finally:
-            recording_iteration.stack.pop()
+            self._recording_iter.stack.pop()
 
     def _iter_get_norm(self):
         """
@@ -706,6 +693,17 @@ class NonlinearSolver(Solver):
         """
         return self._system._residuals.get_norm()
 
+    def _disallow_discrete_outputs(self):
+        """
+        Raise an exception if any discrete outputs exist in our System.
+        """
+        system = self._system
+
+        if system._var_allprocs_discrete['output']:
+            raise RuntimeError("System '%s' has a %s solver and contains discrete outputs %s." %
+                               (system.pathname, type(self).__name__,
+                                sorted(system._var_allprocs_discrete['output'])))
+
 
 class LinearSolver(Solver):
     """
@@ -739,6 +737,17 @@ class LinearSolver(Solver):
         if self.options['assemble_jac']:
             yield self
 
+    def add_recorder(self, recorder):
+        """
+        Add a recorder to the solver's RecordingManager.
+
+        Parameters
+        ----------
+        recorder : <CaseRecorder>
+           A recorder instance to be added to RecManager.
+        """
+        raise RuntimeError('Recording is not supported on Linear Solvers.')
+
     def _declare_options(self):
         """
         Declare options before kwargs are processed in the init method.
@@ -762,19 +771,19 @@ class LinearSolver(Solver):
         super(LinearSolver, self)._setup_solvers(system, depth)
 
         if self._mode == 'fwd':
-            b_vecs = self._system._vectors['residual']
+            b_vecs = system._vectors['residual']
         else:  # rev
-            b_vecs = self._system._vectors['output']
+            b_vecs = system._vectors['output']
 
         self._rhs_vecs = {}
-        for vec_name in self._system._rel_vec_names:
-            self._rhs_vecs[vec_name] = rhs = {}
-            for varset, data in iteritems(b_vecs[vec_name]._data):
-                rhs[varset] = data.copy()
+
+        if system._use_derivatives:
+            for vec_name in system._lin_rel_vec_name_list:
+                self._rhs_vecs[vec_name] = b_vecs[vec_name]._data.copy()
 
         if self.options['assemble_jac'] and not self.supports['assembled_jac']:
             raise RuntimeError("Linear solver '%s' in system '%s' doesn't support assembled "
-                               "jacobians." % (self.SOLVER, self._system.pathname))
+                               "jacobians." % (self.SOLVER, system.pathname))
 
     def solve(self, vec_names, mode, rel_systems=None):
         """
@@ -822,9 +831,7 @@ class LinearSolver(Solver):
             b_vecs = system._vectors['output']
 
         for vec_name in self._vec_names:
-            rhs = self._rhs_vecs[vec_name]
-            for varset, data in iteritems(b_vecs[vec_name]._data):
-                rhs[varset][:] = data
+            self._rhs_vecs[vec_name][:] = b_vecs[vec_name]._data
 
         if self.options['maxiter'] > 1:
             self._run_apply()
@@ -838,7 +845,7 @@ class LinearSolver(Solver):
         """
         Run the apply_linear method on the system.
         """
-        recording_iteration.stack.append(('_run_apply', 0))
+        self._recording_iter.stack.append(('_run_apply', 0))
 
         system = self._system
         scope_out, scope_in = system._get_scope()
@@ -847,7 +854,7 @@ class LinearSolver(Solver):
             system._apply_linear(self._assembled_jac, self._vec_names, self._rel_systems,
                                  self._mode, scope_out, scope_in)
         finally:
-            recording_iteration.stack.pop()
+            self._recording_iter.stack.pop()
 
     def _iter_get_norm(self):
         """
@@ -867,12 +874,9 @@ class LinearSolver(Solver):
 
         norm = 0
         for vec_name in self._vec_names:
-            rhs = self._rhs_vecs[vec_name]
             if vec_name in system._rel_vec_names:
-                b_vec = b_vecs[vec_name]
-                for varset, data in iteritems(rhs):
-                    b_vec._data[varset] -= data
-                norm += b_vec.get_norm()**2
+                b_vecs[vec_name]._data -= self._rhs_vecs[vec_name]
+                norm += b_vecs[vec_name].get_norm()**2
 
         return norm ** 0.5
 
