@@ -3,17 +3,20 @@ from __future__ import division
 
 from contextlib import contextmanager
 import os
+import re
 import sys
 import warnings
 import unittest
 from mock import Mock
 from fnmatch import fnmatchcase
-from six import string_types
+from six import string_types, PY2
 from six.moves import range, cStringIO as StringIO
 from collections import Iterable
 import numbers
+import json
 
 import numpy as np
+import openmdao
 
 
 def warn_deprecation(msg):
@@ -29,10 +32,122 @@ def warn_deprecation(msg):
     warnings.simplefilter('always', DeprecationWarning)
 
     # note, stack level 3 should take us back to original caller.
-    warnings.warn(msg, DeprecationWarning, stacklevel=3)
+    simple_warning(msg, DeprecationWarning, stacklevel=3)
     warnings.simplefilter('ignore', DeprecationWarning)
 
 
+def _warn_simple_format(message, category, filename, lineno, file=None, line=None):
+    return '%s:%s: %s:%s\n' % (filename, lineno, category.__name__, message)
+
+
+def simple_warning(msg, category=UserWarning, stacklevel=2):
+    """
+    Display a simple warning message without the annoying extra line showing the warning call.
+
+    Parameters
+    ----------
+    msg : str
+        The warning message.
+    category : class
+        The warning class.
+    stacklevel : int
+        Number of levels up the stack to identify as the warning location.
+    """
+    old_format = warnings.formatwarning
+    warnings.formatwarning = _warn_simple_format
+    try:
+        warnings.warn(msg, category, stacklevel)
+    finally:
+        warnings.formatwarning = old_format
+
+
+class reset_warning_registry(object):
+    """
+    Context manager which archives & clears warning registry for duration of context.
+
+    From https://bugs.python.org/file40031/reset_warning_registry.py
+
+    Attributes
+    ----------
+    _pattern : regex pattern
+        Causes manager to only reset modules whose names match this pattern. defaults to ``".*"``.
+    """
+
+    #: regexp for filtering which modules are reset
+    _pattern = None
+
+    #: dict mapping module name -> old registry contents
+    _backup = None
+
+    def __init__(self, pattern=None):
+        """
+        Initialize all attributes.
+
+        Parameters
+        ----------
+        pattern : regex pattern
+            Causes manager to only reset modules whose names match pattern. defaults to ``".*"``.
+        """
+        self._pattern = re.compile(pattern or ".*")
+
+    def __enter__(self):
+        """
+        Enter the runtime context related to this object.
+
+        Returns
+        -------
+        reset_warning_registry
+            This context manager.
+
+        """
+        # archive and clear the __warningregistry__ key for all modules
+        # that match the 'reset' pattern.
+        pattern = self._pattern
+        backup = self._backup = {}
+        for name, mod in list(sys.modules.items()):
+            if pattern.match(name):
+                reg = getattr(mod, "__warningregistry__", None)
+                if reg:
+                    backup[name] = reg.copy()
+                    reg.clear()
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        """
+        Exit the runtime context related to this object.
+
+        Parameters
+        ----------
+        exc_type : Exception class
+            The type of the exception.
+        exc_value : Exception instance
+            The exception instance raised.
+        traceback : regex pattern
+            Traceback object.
+        """
+        # restore warning registry from backup
+        modules = sys.modules
+        backup = self._backup
+        for name, content in backup.items():
+            mod = modules.get(name)
+            if mod is None:
+                continue
+            reg = getattr(mod, "__warningregistry__", None)
+            if reg is None:
+                setattr(mod, "__warningregistry__", content)
+            else:
+                reg.clear()
+                reg.update(content)
+
+        # clear all registry entries that we didn't archive
+        pattern = self._pattern
+        for name, mod in list(modules.items()):
+            if pattern.match(name) and name not in backup:
+                reg = getattr(mod, "__warningregistry__", None)
+                if reg:
+                    reg.clear()
+
+
 def ensure_compatible(name, value, shape=None, indices=None):
     """
     Make value compatible with the specified shape or the shape of indices.
@@ -278,9 +393,9 @@ def format_as_float_or_array(name, values, val_if_none=0.0, flatten=False):
     elif values is None:
         values = val_if_none
     elif values == float('inf'):
-        values = sys.float_info.max
+        values = openmdao.INF_BOUND
     elif values == -float('inf'):
-        values = -sys.float_info.max
+        values = -openmdao.INF_BOUND
     elif isinstance(values, numbers.Number):
         values = float(values)
     else:
@@ -448,7 +563,7 @@ def printoptions(*args, **kwds):
 
     Set print options for the scope of the `with` block, and restore the old
     options at the end. See `numpy.set_printoptions` for the full description of
-    available options.
+    available options. If any invalid options are specified, they will be ignored.
 
     Parameters
     ----------
@@ -471,8 +586,13 @@ def printoptions(*args, **kwds):
     set_printoptions, get_printoptions
     """
     opts = np.get_printoptions()
+
+    # ignore any keyword args that are not valid in this version of numpy
+    # e.g. numpy <=1.13 does not have the 'floatmode' option
+    kw_opts = dict((key, val) for key, val in kwds.items() if key in opts)
+
     try:
-        np.set_printoptions(*args, **kwds)
+        np.set_printoptions(*args, **kw_opts)
         yield np.get_printoptions()
     finally:
         np.set_printoptions(**opts)
@@ -494,3 +614,87 @@ def do_nothing_context():
         yield None
 
     return contextmanager(nothing)()
+
+
+def _byteify(data, ignore_dicts=False):
+    """
+    Convert any unicode items in a data structure to bytes (object_hook for json load/loads).
+
+    Credit: Mirec Miskuf
+    stackoverflow.com/questions/956867/how-to-get-string-objects-instead-of-unicode-from-json
+
+    Parameters
+    ----------
+    data : any data item or structure
+        the data to be converted
+    ignore_dicts : bool
+        a flag to prevent recursion on dicts that have already been byteified.
+        False when object_hook passes a new dict to byteify, True at all other times.
+
+    Returns
+    -------
+    data item or structure
+        data item or structure with unicode converted to bytes
+    """
+    # if this is a unicode string, return its string representation
+    if isinstance(data, unicode):
+        return data.encode('utf-8')
+
+    # if this is a list of values, return list of byteified values
+    if isinstance(data, list):
+        return [_byteify(item, ignore_dicts=True) for item in data]
+
+    # if this is a dictionary, return dictionary of byteified keys and values
+    # but only if we haven't already byteified it
+    if isinstance(data, dict) and not ignore_dicts:
+        return {
+            _byteify(key, ignore_dicts=True): _byteify(value, ignore_dicts=True)
+            for key, value in data.iteritems()
+        }
+
+    # if it's anything else, return it in its original form
+    return data
+
+
+def json_load_byteified(file_handle):
+    """
+    Load data from a JSON file, converting unicode to bytes if Python version is 2.x.
+
+    Intended for use only with Python 2.x, behaves the same as json.load() under Python 3.x.
+
+    Parameters
+    ----------
+    file_handle : file
+        file containing the data to be converted
+
+    Returns
+    -------
+    data item or structure
+        data item or structure with unicode converted to bytes
+    """
+    if PY2:
+        return _byteify(json.load(file_handle, object_hook=_byteify), ignore_dicts=True)
+    else:
+        return json.load(file_handle)
+
+
+def json_loads_byteified(json_str):
+    """
+    Load data from a JSON string, converting unicode to bytes if Python version is 2.x.
+
+    Intended for use only with Python 2.x, behaves the same as json.loads() under Python 3.x.
+
+    Parameters
+    ----------
+    json_str : str
+        text string containing json encoded data
+
+    Returns
+    -------
+    data item or structure
+        data item or structure with unicode converted to bytes
+    """
+    if PY2:
+        return _byteify(json.loads(json_str, object_hook=_byteify), ignore_dicts=True)
+    else:
+        return json.loads(json_str)
