@@ -6,6 +6,11 @@ from six import PY3, assertRegex
 from openmdao.core.explicitcomponent import ExplicitComponent
 
 
+def check_even(name, value):
+    if value % 2 != 0:
+        raise ValueError("Option '%s' with value %s is not an even number." % (name, value))
+
+
 class TestOptionsDict(unittest.TestCase):
 
     def setUp(self):
@@ -104,21 +109,21 @@ class TestOptionsDict(unittest.TestCase):
                          "'types' and 'values' were both specified for option 'test3'.")
 
     def test_isvalid(self):
-        self.dict.declare('even_test', types=int, is_valid=lambda x: x % 2 == 0)
+        self.dict.declare('even_test', types=int, check_valid=check_even)
         self.dict['even_test'] = 2
         self.dict['even_test'] = 4
 
         with self.assertRaises(ValueError) as context:
             self.dict['even_test'] = 3
 
-        expected_msg = "Function is_valid(3) returns False for option 'even_test'."
+        expected_msg = "Option 'even_test' with value 3 is not an even number."
         self.assertEqual(expected_msg, str(context.exception))
 
     def test_isvalid_deprecated_type(self):
 
         with warnings.catch_warnings(record=True) as w:
             warnings.simplefilter("always")
-            self.dict.declare('even_test', type_=int, is_valid=lambda x: x % 2 == 0)
+            self.dict.declare('even_test', type_=int, check_valid=check_even)
             self.assertEqual(len(w), 1)
             self.assertEqual(str(w[-1].message), "In declaration of option 'even_test' the '_type' arg is deprecated.  Use 'types' instead.")
 
@@ -128,7 +133,7 @@ class TestOptionsDict(unittest.TestCase):
         with self.assertRaises(ValueError) as context:
             self.dict['even_test'] = 3
 
-        expected_msg = "Function is_valid(3) returns False for option 'even_test'."
+        expected_msg = "Option 'even_test' with value 3 is not an even number."
         self.assertEqual(expected_msg, str(context.exception))
 
     def test_unnamed_args(self):
@@ -136,7 +141,7 @@ class TestOptionsDict(unittest.TestCase):
             self.dict['test'] = 1
 
         # KeyError ends up with an extra set of quotes.
-        expected_msg = "\"Key 'test' cannot be set because it has not been declared.\""
+        expected_msg = "\"Option 'test' cannot be set because it has not been declared.\""
         self.assertEqual(expected_msg, str(context.exception))
 
     def test_contains(self):
@@ -160,7 +165,7 @@ class TestOptionsDict(unittest.TestCase):
             self.dict.update({'test': 2})
 
         # KeyError ends up with an extra set of quotes.
-        expected_msg = "\"Key 'test' cannot be set because it has not been declared.\""
+        expected_msg = "\"Option 'test' cannot be set because it has not been declared.\""
         self.assertEqual(expected_msg, str(context.exception))
 
     def test_get_missing(self):
@@ -203,7 +208,7 @@ class TestOptionsDict(unittest.TestCase):
         with self.assertRaises(KeyError) as context:
             opt['permanent'] = 4.0
 
-        expected_msg = ("Tried to set 'permanent' on a read-only OptionsDictionary")
+        expected_msg = ("Tried to set read-only option 'permanent'.")
         assertRegex(self, str(context.exception), expected_msg)
 
     def test_bounds(self):
@@ -218,7 +223,7 @@ class TestOptionsDict(unittest.TestCase):
         with self.assertRaises(ValueError) as context:
             self.dict['x'] = -3.0
 
-        expected_msg = "Value (-3.0) of option 'x' exceeds minimum allowed value of 2.0."
+        expected_msg = "Value (-3.0) of option 'x' is less than minimum allowed value of 0.0."
         self.assertEqual(str(context.exception), expected_msg)
 
     def test_undeclare(self):
